/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** A CombEnumerator is used to enumerate all possible combinations of
    different integer possibilities. This is used for finding the best
    combination of matches in a non-sequential Vertex comparison.*/
public class CombEnumerator {
    private Vector[] numbers;
    private int current=0;
    private int enumerations=0;
    
    private int[] multisize=null;
    private int[] multiindex=null;
    private int[] prototype=null;

    private int[] counter =null;
    // takes an array of Vector of Integer
    public CombEnumerator(Vector[] ar) {
	numbers=ar;
	enumerations=1;
	int count =0;
	for (int i=0; i<numbers.length; i++) {
	    enumerations = enumerations*numbers[i].size();
	    if (numbers[i].size()>1) count++;
	}

	multisize=new int[count];
	multiindex=new int[count];
	prototype=new int[numbers.length];
	counter = new int[count];

	int multicounted=0;
	int singlecounted=0;
	for (int i=0; i<numbers.length; i++) {
	    if (numbers[i].size()>1) {
		multisize[multicounted]=numbers[i].size();
		multiindex[multicounted]=i;
		multicounted++;
	    } else {
		prototype[singlecounted]=((Integer)numbers[i].get(0)).intValue();
		singlecounted++;
	    }
	}
	for (int i=0; i<counter.length; i++) counter[i]=0;
    }

    public int getMaxEnumerations() {
	return enumerations;
    }
    
    public int[] getNext() {
	int[] result =null;
	if (current>enumerations) return null;
	if (enumerations>1) {
	    if (current==enumerations) return null;
	    result= copyIntArray(prototype);
	
	    for (int i=0; i<multisize.length; i++) {
		result[multiindex[i]]=((Integer) numbers[multiindex[i]].get(counter[i])).intValue();
	    }
	    
	} else result = copyIntArray(prototype);

	if (current+1<enumerations) incCounter(0);
	current++;
	return result;
    }

    private void incCounter(int inx) {

	if (counter[inx]+1 < multisize[inx]) {
	    // ok to increase this entry
	    counter[inx]++;
	} else {
	    // this entry returns to 0 and the next one increases
	    counter[inx]=0;
	    incCounter(inx+1);
	}
    }
    
    private int[] copyIntArray(int[] ar) {
	int[] result = new int[ar.length];
	for (int i=0; i<ar.length; i++) {
	    result[i]=ar[i];
	}
	return result;
    }


}
