/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Date;

/** FuncLib is a library grouping a number of helper functions.*/

public class FuncLib {

    public static double[][] addMatrices(double[][] m1, double[][] m2) {
	double[][] result = new double[m1.length][m1[0].length];

	for (int i=0; i<m1.length; i++) {
	    for (int j=0; j<m1[0].length; j++) {
		result[i][j]=m1[i][j]+m2[i][j];
	    }
	}
	return result;
    }

    public static double[][] linearCombineMatrices(double[][] m1, double factor1,
						   double[][] m2, double factor2) {
	double[][] result = new double[m1.length][m1[0].length];

	for (int i=0; i<m1.length; i++) {
	    for (int j=0; j<m1[0].length; j++) {
		result[i][j]=factor1*m1[i][j]+factor2*m2[i][j];
	    }
	}
	return result;
    }

    public static double[][] divideMatrixBy(double[][] m, double n) {
	double[][] result = new double[m.length][m[0].length];

	for (int i=0; i<m.length; i++) {
	    for (int j=0; j<m[0].length; j++) {
		result[i][j] = m[i][j]/n;
	    }
	}
	return result;
    }

    public static String doubleToStringOfLength(double d, int length) {
	String result = Double.toString(d);
	if (result.length()>length) result =result.substring(0,length);
	if (result.length()<length) {
	    String s="";
	    for (int i=0; i<length-result.length(); i++) {
		s+=" ";
	    }
	    result = s+result;
	}
	result = "["+result+"]";
	return result;
    }

    public static int howmanyTrue(boolean[] b) {
	int result=0;
	for (int i=0; i<b.length; i++) {
	    if (b[i]) result++;
	}
	return result;
    }

    public static int countCopies(int[] comb, boolean doSum) {
	int max=0;
	for (int i=0; i<comb.length; i++) {
	    if (comb[i]>max) max=comb[i];
	}
	int[] buckets = new int[max+1];
	for (int i=0; i<comb.length; i++) {
	    buckets[comb[i]]++;
	}
	int numOfProblems=0;
	int sumOfProblems=0;
	for (int i=0; i<buckets.length; i++) {
	    if (buckets[i]>1) {
		numOfProblems++;
		sumOfProblems+=buckets[i]-1;
	    }
	}
	if (doSum) return sumOfProblems;
	else return numOfProblems;
    }

    /** Returns n for the number 2^n that d1 should be multiplied with
       to get d2, in other words, log2(d2)/log2(d1). */
    public static int lengthInterval(double d1, double d2) {
	double min = Math.min(d1,d2);
	double max = Math.max(d1,d2);
	int log2=0;
	while (min<max) {min=min*2; log2++; }
	
	if (d1==max) log2=-log2;
	return log2;
    }

    private static long lasttime=0;

    /** Never returns the same timestamp twice */
    public static long getTimeStamp() {
	long stamp = new Date().getTime();
	while (stamp==lasttime || stamp<lasttime) {
	    stamp++;
	}
	lasttime=stamp;
	return stamp;
    }

}
