/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** Class GaTest is the program used to generate the GA tuning results
    referred to in Appendix B of our thesis. */

class GaTest {
    static int popsize=100;
    static int initsize=3;
    static double crossover=0.0;
    static double mutation=0.3;
    static int repFreq=300;
    static SimilarityMeasure sm=SimLibVertex.nonSeqVertexPLIOS;
    static Subgraph occ=null;
    static int maxsize=0;
    static double freshBloodChance=0.3;

    public static void main(String[] args) {
	if (args.length<7) {
	    System.out.println("Usage: java GaTest <midifilename> <status output filename> <generations> <value delta> <min> <max> <testruns per value>");
	    System.exit(0);
	}
	int generations = new Integer(args[2]).intValue();
	double valueDelta = new Double(args[3]).doubleValue();
	double min= new Double(args[4]).doubleValue();
	double max= new Double(args[5]).doubleValue();
	int testruns = new Integer(args[6]).intValue();

	Vector allResults=new Vector(); // of FBTresult

	for (double value=min; value<=max; value+=valueDelta) {
	    Vector testrunResults = new Vector(); // of Integer
	    // SimLibMeanDiff.seqMD_PIAL.bval=value;
	    mutation=value;
	    for (int i=0; i<testruns; i++) {
		try{
		    SimFinder sf = new SimFinder(new MotherGraph(args[0],true),
						 popsize, initsize, 
						 crossover, mutation, repFreq,
						 sm,occ,maxsize,
						 freshBloodChance,-1.0);
		    sf.run(generations,args[1]);
		    SimilarityStatement best = sf.findBest();
		    testrunResults.add(new Integer(best.size()));
		}catch (Exception e){
		    System.out.println("SimFinder: Could not initialise");
		}
		
	    } // testruns done for this value
	    allResults.add(new TestResult(testrunResults, value,generations));
	}
	writeAll(allResults,testruns);
    }

    public static void writeAll(Vector ar/*of TestResult*/, int tr) {
	TestResult fr0 = (TestResult) ar.get(0);
	System.out.println("Test results: "+tr+" testruns, "+fr0.gens+" gens each");
	System.out.println("SimMeasure="+sm.name);
	for (int i=0; i<ar.size(); i++) {
	    TestResult fr = (TestResult) ar.get(i);
	    System.out.println("\t"+fr.toString());
	}
	System.out.println();
	System.out.println();
	for (int i=0; i<ar.size(); i++) {
	    TestResult fr = (TestResult) ar.get(i);
	    System.out.println("\t"+fr.toGnuplotString());
	}
    }



    private static class TestResult {
	double meanSize;
	Vector allResults; // of Integer
	double val;
	int gens;

	public TestResult(Vector ar/* of Integer */, double fbc, int g) {
	    int total=0;
	    for (int i=0; i<ar.size(); i++) {
		total+=((Integer)ar.get(i)).intValue();
	    }

	    meanSize=((double)total)/ar.size();
	    allResults=ar;
	    val = fbc;
	    gens=g;
	}

	public String toString() {
	    return "value="+val+", meanSize="+meanSize;
	}
	public String toGnuplotString() {
	    return ""+val+" "+meanSize;
	}
    }







}
