/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.awt.*;
import java.util.Vector;
import java.awt.event.*;

/** GraphGui is a GUI-related class used to show a {@link MusicGraph}
    on screen. */

public class GraphGui extends Container implements AdjustmentListener, MouseListener {
    Scrollbar sbHor = new Scrollbar(Scrollbar.HORIZONTAL);
    Scrollbar sbVer = new Scrollbar(Scrollbar.VERTICAL);
    GraphCanvas cv;
    MotherGraph mg;
    boolean isSeq=true;

    public final int VSIZEX = 30; // vertex size
    public final int VSIZEY = 30; // vertex size
    public final int VSPACE = 30; // space between 2 vertices
    /* Note: vertical dashed lines are only drawn correctly if both
       VSPACE and VSIZEY are integer multipla of dashSize (in method
       drawLine) */

    /* Highlights are subgraphs of the musicgraph that should be hilited.*/
    Vector highlights; // of Subgraph

    public void addAndShowHighlight(Subgraph hi) {
	highlights.add(hi);
	showSubgraph(hi);
    }

    public void setAndShowHighlight(Subgraph hi) {
	highlights.clear();
	highlights.add(hi);
	// setTitle("Hilite shown: "+hi.title);
	showSubgraph(hi);
    }

    public boolean isSequential() {return isSeq;}

    public GraphGui(MotherGraph mgraph, /*int locx, int locy,*/ boolean seq) {
	super();
	highlights = new Vector();
	mg=mgraph;
	isSeq=seq;
        sbHor.addAdjustmentListener(this);
	sbVer.addAdjustmentListener(this);
	// setLocation(locx,locy);
	setLayout(new BorderLayout());

	cv = new GraphCanvas(mg, highlights);
	cv.setSize(600,300);
	cv.addMouseListener(this);
	addMouseListener(this);
	add(cv,BorderLayout.CENTER);

	add(sbHor, BorderLayout.SOUTH);
	add(sbVer, BorderLayout.WEST);
	// pack();

	try { Thread.sleep(3000);} catch(Exception e) {System.out.println("sleep exception: "+e.getMessage());}
	// show();
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
      Scrollbar sb = (Scrollbar)evt.getSource();
      /*System.out.println(((sb.getOrientation() == Scrollbar.VERTICAL) ?
                          "ver" : "hor") 
                         + " li=" + sb.getUnitIncrement()
                         + " pi=" + sb.getBlockIncrement()
                         + " max=" + sb.getMaximum()
			 + " vis=" + sb.getVisibleAmount());*/
      cv.sbChanged();
    }

    public void mousePressed(MouseEvent e) {
	// huh
    }
    public void mouseReleased(MouseEvent e) {
	// huh
    }
    public void mouseEntered(MouseEvent e) {
	// huh
    }
    public void mouseExited(MouseEvent e) {
	// huh
    }
    
    public void mouseClicked(MouseEvent e) {
	if (e.getButton()==MouseEvent.BUTTON1) {
	    Point p=e.getPoint();
	    Point translatedP = new Point(p.x-cv.originX,p.y-cv.originY);
	    MusicVertex mv = mg.findVertexAt(translatedP);
	    if (mv!=null) {
		if (mv instanceof SimpleVertex) {
		    System.out.println("Click on: "+mv.toString()+" ts="+mv.objectTimeStamp);
		} else if (mv instanceof CompoundVertex) {
		    CompoundVertex cv = (CompoundVertex) mv;
		    GraphWindow compsubgui = new GraphWindow(cv.productionRule.nt.toString()+", SimMeasure="+cv.smr.origin.name+", val="+cv.smr.measure, cv.subgraph.getMotherGraph(),500,600,isSequential());
		    compsubgui.setVisible(true); 
		    
		}
	    }
	}
	else if (e.getButton()==MouseEvent.BUTTON2) {
	    System.out.println("Hah, mouse button 2 has no effect!");
	}
	else if (e.getButton()==MouseEvent.BUTTON3) {
	    boolean change=false;
	    Point p=e.getPoint();
	    Point translatedP = new Point(p.x-cv.originX,p.y-cv.originY);
	    MusicVertex mv = mg.findVertexAt(translatedP);
	    if (mv!=null) {
		for (int i=0; i<highlights.size(); i++) {
		    Subgraph hilit = (Subgraph) highlights.get(i);
		    if (hilit.vertices.contains(mv)) {
			if (!hilit.vertexAtIndexSplitsSubgraph(hilit.vertices.indexOf(mv),hilit.getClosure(!isSequential()/*include sim-edges when computing closure*/, !isSequential() /* include weak-edges */))) {
			    hilit.removeVertexAndConnectedEdges(mv);
			    change=true;
			} else {
			    System.out.println("Removing "+mv.toString()+" would split the subgraph");
			}
		    } else {
			// ask the subgraph if it has any vertices
			// that could be connected to mv by including
			// some edge from the mothergraph
			MusicEdge me = hilit.findEdgeConnectingToVertex(mv);
			if (me!=null) {
			    if (isSequential()) {
				hilit.addVertexAndNecessaryEdges(mv,me);
				if (hilit.isStrongSequential()) {
				    change=true;
				}
				else { 
				    //we have added something that
				    //made the subgraph
				    //nonsequential. Undo it!
				    hilit.removeVertexAndConnectedEdges(mv);
				    System.out.println("Adding "+mv.toString()+" would make the subgraph non-sequential.");
				}
			    } else {
				hilit.addVertexAndNecessaryEdges(mv,me);
				change=true;
			    }
			}
		    }
		}
	    }
	    if (change) cv.repaint();
	    /*	    boolean change=false;
	    Point p=e.getPoint();
	    Point translatedP = new Point(p.x-cv.originX,p.y-cv.originY);
	    MusicVertex mv = mg.findVertexAt(translatedP);
	    if (mv!=null) {
		for (int i=0; i<highlights.size(); i++) {
		    Subgraph hilit = (Subgraph) highlights.get(i);
		    if (hilit.vertices.contains(mv)) {
			hilit.removeVertexAndConnectedEdges(mv);
			change=true;
		    } else {
			MusicEdge me = hilit.findEdgeConnectingToVertex(mv);
			if (me!=null) {
			    hilit.addVertexAndNecessaryEdges(mv,me);
			    change=true;
			}
		    }
		}
	    }
	    if (change) cv.repaint();
	    */
	}
    }
    
    /* Center view on portion of graph */ 
    private void showSubgraph(MusicGraph sub) {
	Point pos = new Point(Integer.MAX_VALUE,Integer.MAX_VALUE);
	//	System.out.println("Show subgraph:");
	for (int i=0; i<sub.vertices.size(); i++) {
	    MusicVertex mv = (MusicVertex) sub.vertices.get(i);
	    // System.out.println("\t"+mv.upperLeftPosition.toString());

	    if (mv.upperLeftPosition.x<pos.x)
		pos.x = mv.upperLeftPosition.x;
	    if (mv.upperLeftPosition.y<pos.y) 
		pos.y += mv.upperLeftPosition.y;
	}
	
	int sbHorVal = pos.x*(sbHor.getMaximum()-sbHor.getMinimum())/cv.graphWidth;
	// int sbVerVal = pos.y*(sbVer.getMaximum()-sbVer.getMinimum())/cv.graphHeight;
	/*System.out.println("min="+cv.sbHor.getMinimum());
	System.out.println("max="+cv.sbHor.getMaximum());
	System.out.println("sbVal = "+sbVal);
	System.out.println("graphWidth="+cv.graphWidth);*/
	sbHor.setValue(sbHorVal);
	//sbVer.setValue(sbVerVal);
	cv.sbChanged();
    }


    private class GraphCanvas extends Canvas {
	public final int INDENTY = 4*VSIZEY/5;
	Color highlightColor = Color.yellow;
	Color pitchColor = Color.blue;
	Color lengthColor = Color.red;
	Color headerColor = Color.black;
	Color solidArrowColor = Color.black;
	Color lightArrowColor = Color.gray;
	int originX=0, originY=0;
	// Scrollbar sbHor;
	MusicGraph musgraph;
	Vector hilites; // of MusicGraph
	int maxWidth=50000; // should be 0<maxWidth<ca.65000
	int graphWidth=0;
	int graphHeight=0;

	Vector pendingCompounds = new Vector(); // of CompoundVertex
	// Vector pendingCompoundMaxYpos = new Vector(); // of Integer

	Vector rulerTimes=new Vector(); // of Double
	Vector rulerPos = new Vector(); // of Point

	/* Constructor */
	public GraphCanvas(MusicGraph mugr, Vector hl) {
	    // this.sbHor=sb;
	    musgraph=mugr;
	    hilites=hl;
	    simultLayout(musgraph);
	}
	
	public void paint(Graphics g) {
	    g.setFont(new Font("Dialog",Font.PLAIN,10));
	    g.translate(originX, originY);
	    doLayout();
	    /* NB: simultLayout must have been run in order for
             * simultLayoutHiliteSub to work (so it is run in the
             * constructor), because all vertices must have been
             * assigned a upperLeftPosition, which simultLayout does. */
	    if (!musgraph.graphicLayoutDone()) {
		simultLayout(musgraph);
	    }

	    drawAll(musgraph,hilites,g);
	}

	// If the size of the component changes, we need to update
	// the scrollbar values.
	public void doLayout() {
	    super.doLayout();
	    int w = getSize().width;
	    int h = getSize().height;

	    if (h>graphHeight) originY=0;
	    if (w>graphWidth) originX=0;
	    
	    sbHor.setValues(sbHor.getValue(), w, sbHor.getMinimum(), graphWidth);
	    sbVer.setValues(sbVer.getValue(), h, sbVer.getMinimum(), graphHeight);
	    // System.out.println("gw="+graphWidth);
	    sbHor.setBlockIncrement(4*w/5);
	    sbHor.setUnitIncrement(w/20);
	    sbVer.setBlockIncrement(4*h/5);
	    sbVer.setUnitIncrement(h/20);
	    
	    // This ensures that if the grid is larger than the viewable
	    // area, you will never see space around the grid.
	    // originX = -Math.max(0, Math.min(gridSize-w, sbHor.getValue()));
	    // originY = -Math.max(0, Math.min(gridSize-h, sbVer.getValue()));
	}
	public void sbChanged() {
	    originX = -sbHor.getValue();
	    originY = -sbVer.getValue();
	    //System.out.println("sbChanged: "+sbHor.getValue());
	    repaint()
;
	}

	private int findMaxYposInInterval(MusicGraph mg, double fromtime, double totime){
	    int max=0;
	    for (int i=0; i<mg.vertices.size(); i++) {
		MusicVertex mv = (MusicVertex) mg.vertices.get(i);
		if (mv.graphicLayoutDone()) {
		    // check if this vertex overlaps with the vertex from
		    // fromtime to totime
		    if (isEqInInterval(mv.starttime,fromtime,totime) ||
			isInInterval(mv.starttime+mv.getLength(),fromtime,totime)) {
			// System.out.println("in interval: "+mv.toString());
			if (max<mv.upperLeftPosition.y) {
			    max=mv.upperLeftPosition.y;
			}
		    }
		} // e { System.out.println("nolayout: "+mv.toString());}
	    }
	    return max;
	}

	private boolean isInInterval(double x, double from, double to) {
	    if (x>from && x<to) return true;
	    return false;
	}

	private boolean isEqInInterval(double x, double from, double to) {
	    if (x>=from && x<to) return true;
	    return false;
	}

	/* Finds the x-position of the first ruler entry AT OR AFTER 'time' */
	private int findFirstXposAfterTime(double time) {
	    int result=0;
	    double timediff=Double.MAX_VALUE;

	    for (int i=0; i<rulerTimes.size(); i++) {
		double t = ((Double)rulerTimes.get(i)).doubleValue();
		if ((t>=time) && (t-time<timediff) ) {
		    timediff=t-time; // t>=time so no Math.abs() needed
		    result=((Point) rulerPos.get(i)).x;
		}
	    }
	    return result;
	}

	private int findXposAtTime(double time) {
	    int result=-1;
	    for (int i=0; i<rulerTimes.size(); i++) {
		double t = ((Double)rulerTimes.get(i)).doubleValue();
		if (t==time)  {
		    result = ((Point) rulerPos.get(i)).x;
		}
	    }
	    // System.out.println("xpos("+time+")="+result);
	    return result;
	}

	/* Returns a Vector of Double */
	private Vector getAllStarttimesInInterval(double from, double to) {
	    Vector result = new Vector();
	    for (int i=0; i<rulerTimes.size(); i++) {
		double t = ((Double)rulerTimes.get(i)).doubleValue();
		if (isEqInInterval(t,from,to)) {
		    result.add(new Double(t));
		}
	    }
 	    /* // debug
	    System.out.println("getAllStarttimesInInterval("+from+","+to+")=");
	    for (int i=0; i<result.size(); i++) {
		double t = ((Double)result.get(i)).doubleValue();
		System.out.print(" "+t);
	    }
	    System.out.println();
	    // end debug*/ 
	    return result;
	}

	/* finds the closest timepoint in a vector. if several have
           same distance to 'time', prefer the latest */
	private double findClosestTime(Vector severalTimes /* of Double */, 
				       double time) {
	    double result=0.0;
	    double diff=Double.MAX_VALUE;

	    for (int i=0; i<severalTimes.size(); i++) {
		double t = ((Double)severalTimes.get(i)).doubleValue();
		if (Math.abs(t-time)<diff) {
		    diff=Math.abs(t-time);
		    result=t;
		}
		else if(Math.abs(t-time)==diff) {
		    // if equal distance prefer later time
		    result = Math.max(result,t);
		}
	    }
	    // System.out.println("closest("+time+")="+result);
	    return result;
	}

	private void sortPendingCompoundsByEndtimeAscending() {
	    // bubblesort
	    for (int i=0; i<pendingCompounds.size(); i++) {
		boolean changed = false;
		for (int j=0; j<pendingCompounds.size()-1; j++) {
		    CompoundVertex cv1 = (CompoundVertex) pendingCompounds.get(j);
		    CompoundVertex cv2 = (CompoundVertex) pendingCompounds.get(j+1);
		    if (cv1.getEndtime()>cv2.getEndtime()) {
			// swap
			pendingCompounds.set(j,cv2);
			pendingCompounds.set(j+1,cv1);
			changed=true;
		    }
		}
		if (!changed) return;
	    }
	}
	
	private boolean vertexLayoutClip(int x1,int y1,int x2,int y2) {
	    Point upleft = new Point(x1,y1);
	    Point lowright = new Point(x2,y2);
	    for (int i=0; i<mg.vertices.size(); i++) {
		MusicVertex mv = (MusicVertex) mg.vertices.get(i);
		if (mv.graphicLayoutDone()) {
		    if (rect1inRect2(mv.upperLeftPosition,mv.lowerRightPosition,
				     upleft,lowright)) return true;
		    if (rect1inRect2(upleft,lowright, mv.upperLeftPosition,
				     mv.lowerRightPosition)) return true;
		}
	    }
	    return false;
	}

	private boolean rect1inRect2(Point upleft1, Point lowright1,
				     Point upleft2, Point lowright2) {
	    // check all four corners of rect1 for being in rect2 
	    // upperleft corner
	    if (upleft1.x>=upleft2.x && upleft1.x<=lowright2.x &&
		upleft1.y>=upleft2.y && upleft1.y<=lowright2.y) return true;
	    // lower left corner
	    if (upleft1.x>=upleft2.x && upleft1.x<=lowright2.x &&
		lowright1.y>=upleft2.y && lowright1.y<=lowright2.y) return true;
	    // upper right corner
	    if (lowright1.x>=upleft2.x && lowright1.x<=lowright2.x &&
		upleft1.y>=upleft2.y && upleft1.y<=lowright2.y) return true;
	    // lower right corner
	    if (lowright1.x>=upleft2.x && lowright1.x<=lowright2.x &&
		lowright1.y>=upleft2.y && lowright1.y<=lowright2.y) return true;
	    return false;
	}

	/* layout compounds */
	/* Parameters headxpos and headypos determine the position to
           add new ruler header elements at, if needed. */
	public void layoutPendingCompounds(int headxpos, int headypos, 
					   int headMaxXpos, int headMaxYpos) {
	    sortPendingCompoundsByEndtimeAscending();

	    for (int i=0; i<pendingCompounds.size(); i++) {
		CompoundVertex cv = (CompoundVertex) pendingCompounds.get(i);
		if (!cv.graphicLayoutDone()) {
		    int lowRightX=findFirstXposAfterTime(cv.starttime+cv.getLength())-VSPACE;
		    if (lowRightX==-VSPACE) {
			// findFirstXposAfterTime couldn't find any xpos
			// because the compound is ending after the last
			// ruler entry. add one.
			rulerTimes.add(new Double(cv.getEndtime()));
			rulerPos.add(new Point(headxpos, headypos));
			lowRightX=headxpos-VSPACE;
			headxpos+=VSPACE+VSIZEX;
			if (headMaxXpos<headxpos) headMaxXpos=headxpos;
			if (headxpos>=maxWidth-(2*VSPACE+VSIZEX)) {
			    // for some reason drawing on the canvas further out
			    // than about 65000 fails, so do wraparound
			    headxpos=VSPACE;
			    headypos=headMaxYpos+VSIZEY+(int)(1.5*VSPACE);
			}
			graphWidth=headMaxXpos+VSIZEX+VSPACE*2;
			
		    }
		    int maxypos = findMaxYposInInterval(musgraph, cv.starttime, 
							cv.starttime+cv.getLength());
		    int ypos =0;
		    int lowRightY=0;
		    int upLeftY=0;
		    
		    // try y positions smaller than maxypos
		    int ycandidate = maxypos-VSIZEY-VSPACE;
		    while (ypos==0 /* ain't found usable value yet */ && 
			   ycandidate>headypos) {
			if (!vertexLayoutClip(cv.upperLeftPosition.x,ycandidate,
					      lowRightX,ycandidate+VSIZEY)) {
			    ypos=ycandidate;
			} else {
			    ycandidate = ycandidate-VSIZEY-VSPACE;
			}
		    }
		    // try y positions greater than maxypos
		    ycandidate = maxypos+VSIZEY+VSPACE;
		    while (ypos==0 /* ain't found usable value yet */ ) {
			if (!vertexLayoutClip(cv.upperLeftPosition.x,ycandidate,
					      lowRightX,ycandidate+VSIZEY)) {
			    ypos=ycandidate;
			} else {
			    ycandidate = ycandidate+VSIZEY+VSPACE;
			}
		    }
		    if (ypos>headMaxYpos) headMaxYpos=ypos;
		    
		    // ypos=200;
		    lowRightY=ypos+VSIZEY;
		    upLeftY=ypos;
		    
		    graphHeight=headMaxYpos+VSIZEY+VSPACE;
		    
		    cv.upperLeftPosition.y=upLeftY;
		    cv.lowerRightPosition= new Point(lowRightX,lowRightY);
		    
		    if (cv.upperLeftPosition.x>cv.lowerRightPosition.x) {
			System.out.println("GraphGui: found compound with upperleft.x > lowerright.x");
		    }
		    if (cv.upperLeftPosition.y > cv.lowerRightPosition.y) {
			System.out.println("GraphGui: found compound with upperleft.y > lowerright.y");
		    }
		}
		
	    }
	}
	
	/* The general layout */
	public void simultLayout(MusicGraph mgraph) {
	    System.out.print("Laying out graph...");
	    
	    int xpos=VSPACE;
	    int ypos=VSPACE;
	    int maxXpos =0;
	    int maxYpos =0;
	    
	    boolean indenty=false;
	    graphWidth=xpos;
	    graphHeight=ypos;

	    rulerTimes.clear();
	    rulerPos.clear();
	    pendingCompounds=new Vector();

	    boolean dummy = mgraph.initGroupTokenizer(0.0);
	    Vector group = mgraph.getNextGroup();
	    while (group.size()>0) {
		// draw this group
		int indentedypos = ypos;
		int headerYpos = ypos - VSIZEY/3;
		if (indenty) indentedypos+= INDENTY;
		
		group = sortByPitch(group,true);

		int localMaxYpos= simultLayoutGroup(xpos,indentedypos,
						    headerYpos,group);
		if (maxYpos<localMaxYpos) maxYpos=localMaxYpos;
		if (maxXpos<xpos) maxXpos=xpos;
		
		// get the next group
		group = mgraph.getNextGroup();
		indenty=!indenty;
		xpos+=VSPACE+VSIZEX;
		if (xpos>=maxWidth-(2*VSPACE+VSIZEX)) {
		    // for some reason drawing on the canvas further out
		    // than about 65000 fails, so do wraparound
		    xpos=VSPACE;
		    ypos=maxYpos+VSIZEY+(int)(1.5*VSPACE);
		}
		
		graphWidth=maxXpos+VSIZEX+VSPACE*2;
		graphHeight=maxYpos+VSIZEY+VSPACE*2;
	    } // end while
	    
	    /* Finish off any pending compounds. There may be
	     * compounds whose layout cannot be completed because they
	     * end a piece. Therefore there is no ruler entry at the
	     * endtime of the compound, and the x-value of the
	     * compound's lowerRightPosition cannot be
	     * determined. This is why xpos and ypos are passed, so
	     * that a new ruler entry may be made whenever needed. */
	    layoutPendingCompounds(xpos,ypos-VSIZEY/3, 
				   maxXpos,maxYpos);
	    
	    for (int i=0; i<mgraph.edges.size(); i++) {
		MusicEdge me = (MusicEdge) mgraph.edges.get(i);
		
		if (me.fromPoint==null || me.toPoint==null) {
		    layoutEdge(me);
		}
		// TODO: check here for wraparound
	    }
	    System.out.println("graph layout done!");
	}
	
	/* returns the maximal y position used */
	public int simultLayoutGroup(int xpos, int ypos, int headerYpos,
				     Vector group) {
	    
	    // System.out.println("Layout GROUP size="+group.size());
	    MusicVertex somevertex = (MusicVertex) group.get(0);
	    rulerTimes.add(new Double(somevertex.starttime));
	    rulerPos.add(new Point(xpos, headerYpos));
	    
	    for (int i=0; i<group.size(); i++) {
		MusicVertex mv = (MusicVertex) group.get(i);
		if (mv instanceof SimpleVertex) {
		    //	System.out.println("S\t"+mv.toString());
		    mv.upperLeftPosition=new Point(xpos,ypos);
		    mv.lowerRightPosition=new Point(xpos+VSIZEX,ypos+VSIZEY);
		    ypos+= VSIZEY+VSPACE;
		} else if (mv instanceof CompoundVertex) {
		    // System.out.println("C\t"+mv.toString());
		    mv.upperLeftPosition=new Point(xpos,0);
		    
		    mv.lowerRightPosition=null;
		    // lowerRightPos intentionally set to null
		    // here. in this way we can see in
		    // findMaxYposInInterval() which compounds have
		    // been layouted yet

		    pendingCompounds.add(mv);
		} else {
		    // debug
		    System.out.println("simultLayoutGroup: unexpected MusicVertex class");
		}
	    }
	    
	    return ypos;
	}
	
	/* Draw highlighted subgraphs */
	/* NB: simultLayout must have been run in order for
	 * simultLayoutHiliteSub to work (so it is run in the
	 * constructor), because all vertices and edges must have been
	 * assigned a position, which simultLayout does. */
	public void drawHilites(Vector hl, Graphics g) {
	    for (int j=0; j<hl.size(); j++) {
		MusicGraph musg = (MusicGraph) hl.get(j);
		for (int i=0; i<musg.vertices.size(); i++) {
		    MusicVertex mv = (MusicVertex) musg.vertices.get(i);
		    drawVertex(mv,true,g);
		}
		for (int i=0; i<musg.edges.size(); i++) {
		    MusicEdge me = (MusicEdge) musg.edges.get(i);
		    // TODO: check here for wraparound
		    drawEdge(me,true,g);
		}
	    }
	}

	public void drawGraph(MusicGraph mgraph, Graphics g) {
	    for (int i=0; i<mgraph.vertices.size(); i++) {
		MusicVertex mv = (MusicVertex) mgraph.vertices.get(i);
		drawVertex(mv,false,g);
	    }
	    for (int i=0; i<mgraph.edges.size(); i++) {
		MusicEdge me = (MusicEdge) mgraph.edges.get(i);
		// TODO: check here for wraparound
		drawEdge(me,false,g);
	    }
	}

	public void drawAll(MusicGraph mgraph, Vector hl, Graphics g) {
	    drawRulers(rulerTimes,rulerPos,g);
	    drawHilites(hl,g);
	    drawGraph(mgraph,g);
	}

	public void drawRulers(Vector rTimes, Vector pos, Graphics g) {
	    g.setColor(headerColor);
	    FontMetrics fm = g.getFontMetrics();
	    for (int i=0; i<rTimes.size(); i++) {
		Double d = (Double) rTimes.get(i);
		String str=""+d;
		Point p = (Point) pos.get(i);
		Point pp = new Point(p.x+(VSIZEX-fm.stringWidth(str))/2,p.y);
		g.drawString(str,pp.x,pp.y);
	    }
	    g.setColor(Color.black);
	}

	public void drawVertex(MusicVertex mv, boolean hi, Graphics g) {
	    if (mv instanceof SimpleVertex) {
		drawSimpleVertex((SimpleVertex)mv,hi,g);
	    } else if (mv instanceof CompoundVertex) {
		drawCompoundVertex((CompoundVertex)mv,hi,g);
	    }
 
	}
	public void drawSimpleVertex(SimpleVertex sv, boolean hi, Graphics g) {
	    // debug
	    int x=0;
	    int y=0;
	    x=sv.upperLeftPosition.x;
	    y=sv.upperLeftPosition.y;
	    if (hi) {
		g.setColor(highlightColor);
		g.fillOval(x,y,VSIZEX,VSIZEY);
		g.setColor(Color.black);
	    } else {
		g.drawOval(x,y,VSIZEX,VSIZEY);
		
		int pitch = sv.info.theNote.getPitch();
		double length = sv.getLength();
		String str="";
		if (pitch==Integer.MIN_VALUE) str="+";
		else str=""+pitch;
		
		g.setColor(pitchColor);
		FontMetrics fm = g.getFontMetrics();
		g.drawString(str,
			     x+(VSIZEX-fm.stringWidth(str))/2,
			     y+(VSIZEY-(int)(fm.getHeight()*1.2))/4+fm.getAscent());
		g.setColor(lengthColor);
		str=""+length;
		g.drawString(str,
			     x+(VSIZEX-fm.stringWidth(str))/2,
			     y+3*(VSIZEY-(int)(fm.getHeight()*0.8))/4+fm.getAscent());
		g.setColor(Color.black);
	    }
	}

	public void drawCompoundVertex(CompoundVertex cv, boolean hi, 
				       Graphics g) {
	    int x=cv.upperLeftPosition.x;
	    int y=cv.upperLeftPosition.y;
	    int width=cv.lowerRightPosition.x-x;
	    int height=cv.lowerRightPosition.y-y;

	    if (hi) {
		g.setColor(highlightColor);
		g.fillRect(x,y,width,height);
		g.setColor(Color.black);
	    } else {
		g.drawRect(x,y,width,height);
		
		int sz = cv.subgraph.nestedSize();
		int nd = cv.subgraph.nestingDepth();
		double length = cv.getLength();
		String str=cv.productionRule.nt.toString()+", ns="+sz+", nd="+nd;
		
		g.setColor(pitchColor);
		FontMetrics fm = g.getFontMetrics();
		g.drawString(str,
			     x+(width-fm.stringWidth(str))/2,
			     y+(height-(int)(fm.getHeight()*1.2))/4+fm.getAscent());
		g.setColor(lengthColor);
		str=""+length;
		g.drawString(str,
			     x+(width-fm.stringWidth(str))/2,
			     y+3*(height-(int)(fm.getHeight()*0.8))/4+fm.getAscent());
		g.setColor(Color.black);
	    }
	}

	private boolean anchorInLeftEnd(int anchorXpos, CompoundVertex cv) {
	    if (cv.upperLeftPosition.x==anchorXpos) return true;
	    else return false;
	}
	private boolean anchorInRightEnd(int anchorXpos, CompoundVertex cv) {
	    if (cv.lowerRightPosition.x==anchorXpos+VSIZEX) return true;
	    else return false;
	}

	/* (x1,y1) is the upper left corner of the square around the
	 * from-circle, (x2,y2) is the upper left corner of the
	 * to-circle */
	public void layoutEdge(MusicEdge me) {
	    // using upperLeftPosition works for simple vertices; if
	    // compounds, x1,y1,x2,y2 are set in the if-clauses below
	    int x1=me.from.upperLeftPosition.x;
	    int y1=me.from.upperLeftPosition.y;
	    int x2=me.to.upperLeftPosition.x;
	    int y2=me.to.upperLeftPosition.y;

	    boolean fromCaseA=false; // anchor in leftmost end of compound
	    boolean fromCaseB=false; // anchor somewhere in middle
	    boolean fromCaseC=false; // anchor in rightmost end

	    boolean toCaseA=false; // same cases as for from-vertex
	    boolean toCaseB=false;
	    boolean toCaseC=false;
	    
	    if (me.from instanceof CompoundVertex) {
		// System.out.println("fromComp: ");
		CompoundVertex cv = (CompoundVertex) me.from;
		double time=me.fromAnchor().starttime;
		int xpos = findXposAtTime(time);
		if (xpos==-1) {
		    // System.out.println("from not in ruler");
		    // time does not exist as an entry in the
		    // ruler. find closest entry.
		    double closest = findClosestTime(getAllStarttimesInInterval(cv.starttime, cv.starttime+cv.getLength()),time);
		    xpos=findXposAtTime(closest);
		}
		x1=xpos;
		if (anchorInLeftEnd(x1,cv)) fromCaseA=true;
		else if(anchorInRightEnd(x1,cv)) fromCaseC=true;
		else fromCaseB=true;
	    }
	    if (me.to instanceof CompoundVertex) {
		// System.out.println("toComp: ");
		CompoundVertex cv = (CompoundVertex) me.to;
		double time=me.toAnchor().starttime;
		int xpos = findXposAtTime(time);
		if (xpos==-1) {
		    // System.out.println("to not in ruler");
		    // time does not exist as an entry in the
		    // ruler. find closest entry.
		    double closest = findClosestTime(getAllStarttimesInInterval(cv.starttime, cv.starttime+cv.getLength()),time);
		    xpos=findXposAtTime(closest);
		}
		x2=xpos;
		if (anchorInLeftEnd(x2,cv)) toCaseA=true;
		else if(anchorInRightEnd(x2,cv)) toCaseC=true;
		else toCaseB=true;
	    }
	    /* The numbered cases referred to below: f is the
	      from-vertex and the numbers represent graphically a
	      position relative to f which is over, equal or under f,
	      and to the left, equal or to the right of f. */
	    /*-------
	      |1|2|3| 
	      ------- 
	      |4|f|5| 
	      -------
	      |6|7|8| 
	      ------- */

	    int tx1=0, ty1=0, tx2=0, ty2=0;
	    // retranslate to correct side of surrounding square(s)
	    if (y1>y2) { 
		if (y1<y2+VSIZEY ) {ty2=y2+VSIZEY/2; } 
		else {ty2=y2+VSIZEY;}

		if (x1>x2) { //case 1
		    tx1=x1; ty1=y1;
		    tx2=x2+VSIZEX; 
		}
		if (x1==x2) {//case 2
		    tx1=x1+VSIZEX/2; ty1=y1;
		    tx2=x2+VSIZEX/2; 
		}
		if (x1<x2) {//case 3
		    tx1=x1+VSIZEX; ty1=y1;
		    tx2=x2;
		    // corrections if compound involved:
		    if (x2>me.from.lowerRightPosition.x) {
			if (fromCaseA) {
			    // move from-xpos to rightmost side of compound
			    tx1=me.from.lowerRightPosition.x;
			} else if (fromCaseB) {
			    // move from-xpos to rightmost side of compound
			    tx1=me.from.lowerRightPosition.x;
			} else if (fromCaseC) {
			    tx1=me.from.lowerRightPosition.x;
			    // no problem
			} 
		    }
		}
	    }
	    else if (y1==y2) {
		if (x1>x2) {//case 4
		    tx1=x1; ty1=y1+VSIZEY/2;
		    tx2=x2+VSIZEX; ty2=y2+VSIZEY/2;
		    
		    // corrections if compound involved:
		    if (fromCaseA) {
			// no problem
		    } else if (fromCaseB) {
			// move from-xpos to leftmost side of compound
			tx1=me.from.upperLeftPosition.x;
		    } else if (fromCaseC) {
			// move from-xpos to leftmost side of compound
			tx1=me.from.upperLeftPosition.x;
		    } 
		    if (toCaseA) {
			// move to-xpos to rightmost side of compound
			tx2=me.to.lowerRightPosition.x;
		    } else if (toCaseB) {
			// move to-xpos to rightmost side of compound
			tx2=me.to.lowerRightPosition.x;
		    } else if (toCaseC) {
			// no problem
		    } 
		}
		if (x1<x2) {//case 5
		    tx1=x1+VSIZEX; ty1=y1+VSIZEY/2;
		    tx2=x2; ty2=y2+VSIZEY/2;

		    // corrections if compound involved:
		    if (fromCaseA) {
			// move from-xpos to rightmost side of compound
			tx1=me.from.lowerRightPosition.x;
		    } else if (fromCaseB) {
			// move from-xpos to rightmost side of compound
			tx1=me.from.lowerRightPosition.x;
		    } else if (fromCaseC) {
			tx1=me.from.lowerRightPosition.x;
			// no problem
		    } 
		    if (toCaseA) {
			// no problem
		    } else if (toCaseB) {
			// move to-xpos to leftmost side of compound
			tx2=me.to.upperLeftPosition.x;
		    } else if (toCaseC) {
			// move to-xpos to leftmost side of compound
			tx2=me.to.upperLeftPosition.x;
		    } 
		}
	    }
	    else if (y1<y2) {
		if (y1+VSIZEY>y2) { ty1=y1+VSIZEY/2;}
		else { ty1=y1+VSIZEY; }
		
		if (x1>x2) {//case 6
		    tx1=x1; 
		    tx2=x2+VSIZEX; ty2=y2;
		}
		if (x1==x2) {//case 7
		    tx1=x1+VSIZEX/2;
		    tx2=x2+VSIZEX/2; ty2=y2;
		}
		if (x1<x2) {//case 8
		    tx1=x1+VSIZEX; 
		    tx2=x2; ty2=y2;
		    // corrections if compound involved:
		    if (x2>me.from.lowerRightPosition.x) {
			if (fromCaseA) {
			    // move from-xpos to rightmost side of compound
			    tx1=me.from.lowerRightPosition.x;
			} else if (fromCaseB) {
			    // move from-xpos to rightmost side of compound
			    tx1=me.from.lowerRightPosition.x;
			} else if (fromCaseC) {
			    tx1=me.from.lowerRightPosition.x;
			    // no problem
			} 
		    }
		}
	    }
	    me.fromPoint = new Point(tx1,ty1);
	    me.toPoint = new Point(tx2,ty2);
	}

	public void drawEdge(MusicEdge me, boolean hi, Graphics g) {
	    if (hi) {
		drawArrowHighlight(me.fromPoint.x,
				   me.fromPoint.y,
				   me.toPoint.x,
				   me.toPoint.y,g);
	    }
	    else {
		if (me.edgetype==MusicEdge.FOLLOWS) {
		    drawArrow(me.fromPoint.x,me.fromPoint.y,
			      me.toPoint.x,me.toPoint.y,
			      solidArrowColor,0,g);
		} else if (me.edgetype==MusicEdge.SIMULTANEOUS){
		    drawArrow(me.fromPoint.x,me.fromPoint.y,
			      me.toPoint.x,me.toPoint.y,
			      lightArrowColor,0,g);
		} else if (me.edgetype==MusicEdge.WEAKFOLLOWS){
		    drawArrow(me.fromPoint.x,me.fromPoint.y,
			      me.toPoint.x,me.toPoint.y,
			      solidArrowColor,5,g);
		} else if (me.edgetype==MusicEdge.WEAKSIMULTANEOUS){
		    drawArrow(me.fromPoint.x,me.fromPoint.y,
			      me.toPoint.x,me.toPoint.y,
			      lightArrowColor,5,g);
		}
	    }
	}

	public void drawArrow(int x1, int y1, int x2, int y2, 
			      Color arrowColor, int dashsize, Graphics g) {
	    int arrowHeadSize=0;
	    if (arrowColor==solidArrowColor)
		arrowHeadSize = VSIZEX/3;
	    else 
		arrowHeadSize = VSIZEX/5;

	    boolean yswap=false;
	    if (x1==x2) {
		// always draw vertical lines from top to bottom 
		if (y2<y1) {
		    int tmp = y2;
		    y2=y1;
		    y1=tmp;
		    yswap=true;
		}
		if (arrowColor==solidArrowColor) {
		    // move the compound-side point a little to the
		    // left/right, because otherwise this WEAKFOLLOW edge
		    // may be drawn on top of (or under) some WEAKSIM
		    // edges.
		    if (yswap) { x2-= VSIZEX  ;}
		    else { x2+= VSIZEX  ;}
		}
	    }
	    drawLine(x1,y1,x2,y2,dashsize,arrowColor,g);

	    if (!yswap)
		drawArrowHead(x1,y1,x2,y2,arrowHeadSize,arrowColor,g);
	    else {
		drawArrowHead(x2,y2,x1,y1,arrowHeadSize,arrowColor,g);
	    }
	}

	public void drawArrowHead(int x1, int y1, int x2, int y2, 
				  int arrowHeadSize, Color arrowColor, Graphics g) {
	    g.setColor(arrowColor);
	    int[] polyx = new int[3];
	    int[] polyy = new int[3];
	    
	    double v1=x2-x1;
	    double v2=y2-y1;
	    double vlength =Math.sqrt(Math.pow(v1,2)+Math.pow(v2,2));
	    double omega = Math.acos(v1/vlength);
	    if (y2<y1) {
		omega = 2*Math.PI - omega;
	    }
	    double p1x = x2+arrowHeadSize*(Math.cos(omega+Math.PI*5/6));
	    double p1y = y2+arrowHeadSize*(Math.sin(omega+Math.PI*5/6));

	    double p2x = x2+arrowHeadSize*(Math.cos(omega+Math.PI*7/6));
	    double p2y = y2+arrowHeadSize*(Math.sin(omega+Math.PI*7/6));
	    
	    polyx[0]=(int)x2;
	    polyx[1]=(int)p1x;
	    polyx[2]=(int)p2x;
					   
	    polyy[0]=(int)y2;
	    polyy[1]=(int)p1y;
	    polyy[2]=(int)p2y;

	    g.fillPolygon(polyx,polyy,3);
	    g.setColor(Color.black);
	}


	public void drawLine(int x1, int y1, int x2, int y2, 
			     int dashSize, Color arrowColor, Graphics g){
	    g.setColor(arrowColor);
	    if (dashSize<=0) {
		// means a solid line
		g.drawLine(x1,y1,x2,y2);
		return;
	    } else {
		// a dashed line

		int dx=0;
		int dy=0;
		if (x2==x1) {
		    dx=0;
		    dy=dashSize;
		} else {
		    double alpha=((double)(y2-y1))/(double)(x2-x1);
		    double tmp= Math.sqrt(Math.pow(dashSize,2)/(1+Math.pow(alpha,2)));
		    if (x2<x1) tmp=-tmp;
		    dx = roundToInt(tmp);
		    dy = roundToInt(alpha*tmp);
		}

		g.drawLine(x1,y1,x1+dx,y1+dy);

		if (Math.sqrt(Math.pow(x2-(x1+2*dx),2)+
			      Math.pow(y2-(y1+2*dy),2))>2*dashSize) {
		    drawLine(x1+2*dx,y1+2*dy,x2,y2,dashSize,arrowColor,g);
		}
	    }
	    g.setColor(Color.black);
	}
	
	private int roundToInt(double d) {
	    return (int)Math.rint(d);
	}

	public void drawArrowHighlight(int x1, int y1, int x2, int y2, Graphics g) {
	    /* compute bounding box of highlighting */
	    int boxwidth = (VSIZEX+VSIZEY)/4;

	    int[] polyx = new int[4];
	    int[] polyy = new int[4];
	    
	    double v1=x2-x1;
	    double v2=y2-y1;
	    double vlength =Math.sqrt(Math.pow(v1,2)+Math.pow(v2,2));
	    double omega = Math.acos(v1/vlength);
	    if (y2<y1) {
		omega = 2*Math.PI - omega;
	    }
	    double deltax =Math.cos(omega+Math.PI/2);
	    double deltay =Math.sin(omega+Math.PI/2);
	    double k = Math.sqrt(Math.pow(boxwidth/2,2)/(Math.pow(deltax,2)+Math.pow(deltay,2)));
	    
	    
	    polyx[0] = x2+(int)(k*deltax);
	    polyy[0] = y2+(int)(k*deltay);
	    polyx[1] = x2-(int)(k*deltax);
	    polyy[1] = y2-(int)(k*deltay);

	    polyx[2] = x1-(int)(k*deltax);
	    polyy[2] = y1-(int)(k*deltay);
	    polyx[3] = x1+(int)(k*deltax);
	    polyy[3] = y1+(int)(k*deltay);

	    g.setColor(highlightColor);
	    g.fillPolygon(polyx,polyy,4);
	    g.setColor(Color.black);
	}

	Point calculateVertexCoordinates(int placeInSequence, 
					 int xnum, int ynum) {
	    int x,y;
	    x= VSPACE+(VSPACE+VSIZEX)*(placeInSequence % xnum);
	    y= VSPACE+(VSPACE+VSIZEY)*(placeInSequence / xnum);
	    return new Point(x,y);
	}
	
	Vector sortByPitch(Vector group, boolean inverse) {
	    Vector result = new Vector();
	    MusicVertex[] gr = new MusicVertex[group.size()];
	    for (int i=0; i<group.size(); i++) {
		gr[i] = (MusicVertex) group.get(i);
	    }
	    
	    // bubblesort
	    for (int i=0; i<gr.length; i++) {
		for (int j=0; j<gr.length-1; j++) {
		    boolean swap=false;
		    if(gr[j] instanceof SimpleVertex && 
		       gr[j+1] instanceof SimpleVertex) {
			// both vertices are simple, compare their pitches
			SimpleVertex sv1=(SimpleVertex) gr[j];
			SimpleVertex sv2=(SimpleVertex) gr[j+1];
			
			if (!inverse && (sv1.info.theNote.getPitch() > sv2.info.theNote.getPitch()) ||
			    inverse && (sv1.info.theNote.getPitch() < sv2.info.theNote.getPitch())
			    ) { swap=true;}
		    } else {
			// at least one vertex is not simple, compare
			// their timestamps
			if (!inverse && (gr[j].objectTimeStamp > gr[j+1].objectTimeStamp) ||
			    inverse && (gr[j].objectTimeStamp < gr[j+1].objectTimeStamp)
			    ) { swap=true;}
		    }
		    if (swap) {
			// swap
			MusicVertex tmp = gr[j];
			gr[j]=gr[j+1];
			gr[j+1]=tmp;
		    }
		}
	    }
	    for (int i=0; i<gr.length; i++) {
		result.add(gr[i]);
	    }
	    return result;
	}
    } // end class GraphCanvas
    
}
