/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.awt.*;
import java.util.Vector;
import java.awt.event.*;
import jm.music.data.*;
import jm.util.*;
import jm.gui.show.*;

/** GraphList is a GUI-related class used to show a list of MusicGraphs. */
public class GraphList extends Frame implements ActionListener, 
						MouseListener,
						ItemListener {

    MotherGraph piece;
    List ssList;
    Button playButton;
    Button writeButton;
    Button runButton;
    Button stopButton;
    TextField runText;
    Vector population;
    GraphWindow gg1,gg2;
    VertexUsageGui vug;
    TextArea ta;
    boolean seq=true;
    SimilarityStatement best=null;
    String statusFileName;
    boolean stopRequest=false;

    public GraphList(Vector pop/* of SimStmt */, int locx, int locy, SimilarityStatement b, boolean sequential /*, String file*/) {
	// super("List of subgraphs for "+piece.title);
	if (pop.size()==0) {
	    System.out.println("Empty population vector passed to GraphList");
	    System.exit(0);
	}
	SimilarityStatement ss = (SimilarityStatement) pop.get(0);
	/* in occurrence search subgraph1's mothergraph is not the
	   real mothergraph, but subgraph2's is!  (subgraph1 is no
	   longer in the real mothergraph since it has been
	   substituted with a compound) */
	piece = ss.subgraph2.getMotherGraph();
	population=pop;
	best = b;
	seq=sequential;
	//statusFileName=file;
	setLocation(locx,locy-100);

	gg1=new GraphWindow(piece,locx+600,100,sequential);
	gg2=new GraphWindow(piece,locx+600,500,sequential);
	vug=new VertexUsageGui(piece,ss.myOwner,locx+550,900,8);

	ssList = new List(population.size(),false);
	updateList(ssList,population,best);
	ssList.addActionListener(this);
	ssList.addItemListener(this);

	ta=new TextArea("",15,40);
	playButton=new Button("Play");
	playButton.addActionListener(this);

	writeButton=new Button("Write");
	writeButton.addActionListener(this);

	runButton=new Button("Run");
	runButton.addActionListener(this);

	stopButton=new Button("Stop");
	stopButton.addActionListener(this);

	runText=new TextField("1",4);

	addMouseListener(this);
	add(ssList,BorderLayout.CENTER);
	add(ta, BorderLayout.SOUTH);
	
	Container c = new Container();
	c.setLayout(new GridLayout(5,1));
	c.add(playButton);
	c.add(writeButton);
	c.add(runText);
	c.add(runButton);
	c.add(stopButton);
	add(c,BorderLayout.WEST);
	
	pack();
	setSize(580,800);
	show();
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getSource()==ssList) {
	    int index = ssList.getSelectedIndex();
	    SimilarityStatement ss = (SimilarityStatement) population.get(index);
	    gg1.setAndShowHighlight(ss.subgraph1);
	    gg2.setAndShowHighlight(ss.subgraph2);

	}	    
	else if (e.getSource()==playButton) {
	    int index = ssList.getSelectedIndex();
	    SimilarityStatement ss = (SimilarityStatement) population.get(index);
	    gg1.setAndShowHighlight(ss.subgraph1);
	    gg2.setAndShowHighlight(ss.subgraph2);
	    ta.setText("Playing subgraph 1");
	    Score s;
	    if (seq) s=ss.subgraph1.convertToScore();
	    else s=ss.subgraph1.convertToScore();
	    playScore(s);
	    
	    try {Thread.sleep(2000);}
	    catch(Exception exc) { System.out.println("Sleep exception in graphlist: " +exc.getMessage());}

	    ta.setText("Playing subgraph 2");
	    if (seq) s=ss.subgraph2.convertToScore();
	    else s=ss.subgraph2.convertToScore();
	    playScore(s);
	    
	    try {Thread.sleep(2000);}
	    catch(Exception exc) { System.out.println("Sleep exception in graphlist: " +exc.getMessage());}
	    ta.setText("");
	    
	} else if (e.getSource()==writeButton) {
	    int index = ssList.getSelectedIndex();
	    SimilarityStatement ss = (SimilarityStatement) population.get(index);
	    gg1.setAndShowHighlight(ss.subgraph1);
	    gg2.setAndShowHighlight(ss.subgraph2);
	    String writing="Writing: subgraph1.mid";
	    ta.setText(writing);
	    Score s;
	    if (seq) s=ss.subgraph1.convertToScore();
	    else s=ss.subgraph1.convertToScore();
	    writeScore(s,1);
	    ta.setText(writing+"\nWriting subgraph2.mid");
	    if (seq) s=ss.subgraph2.convertToScore();
	    else s=ss.subgraph2.convertToScore();
	    writeScore(s,2);
	    try {Thread.sleep(2000);}
	    catch(Exception exc) { System.out.println("Sleep exception in graphlist: " +exc.getMessage());}
	    ta.setText("");

	}else if(e.getSource()==runButton){
	    SimilarityStatement firstSimStmt=(SimilarityStatement) population.get(0);
	    SimFinder sf = firstSimStmt.myOwner;
	    String input=(runText.getText()).trim();
	    int generations=new Integer(input).intValue();
	    for(int i=0;i<generations && !stopRequest;i++){
		try{
		    sf.run(1,"sf.out");
		}catch(Exception excptn){}
		population=sf.population;
		best=sf.findBest();
		updateList(ssList,population,best);
		ssList.select(0);
		ta.setText(best.toStringWithSubgraphs());
		gg1.setAndShowHighlight(best.subgraph1);
		gg2.setAndShowHighlight(best.subgraph2);
		show();
	    }
	    stopRequest=false;
	}else if(e.getSource()==runText){
	}else if(e.getSource()==stopButton){
	    stopRequest=true;

	} else {
	    System.out.println("Debug: Got ActionEvent from unexpected source:"+e.getSource().toString());
	}
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange()==ItemEvent.SELECTED) {
	    int index = ssList.getSelectedIndex();
	    //	    ta.setText(((SimilarityStatement)population.get(index)).
	    SimilarityStatement ss = ((SimilarityStatement)population.get(index));
	    // force re-evaluation
	    ss.smResult=null;
	    ta.setText(ss.toStringWithSubgraphs());
	} else {
	    System.out.println("Debug: ItemEvent != ItemEvent.SELECTED");
	}
    }

    public void mousePressed(MouseEvent e) {
	// huh
    }
    public void mouseReleased(MouseEvent e) {
	// huh
    }
    public void mouseEntered(MouseEvent e) {
	// huh
    }
    public void mouseExited(MouseEvent e) {
	// huh
    }

    public void mouseClicked(MouseEvent e) {
	/* MusicGraph mg = (MusicGraph) highlights.get(0);
	if (e.getButton()==MouseEvent.BUTTON1) {
	    try {mg.shortenRight(1);}
	    catch(Exception ex) { System.out.println(ex.getMessage());}
	}
	else if (e.getButton()==MouseEvent.BUTTON2) {
	    System.out.println("Hah, mouse button 2 has no effect!");
	}
	else if (e.getButton()==MouseEvent.BUTTON3) {
	    try {mg.extendRight(1);}
	    catch(Exception ex) { System.out.println(ex.getMessage());}
	}
	showSubgraph(mg);*/ 
    }

    public void playScore(Score s){
	s.setTempo(100);
	Play.midi(s);
    }
    public void writeScore(Score s,int i){
	s.setTempo(100);
	Write.midi(s,"subgraph"+i+".mid");
    }

    private void updateList(List l, Vector simStmts, SimilarityStatement best){
	l.removeAll();
	for (int i=0; i<simStmts.size(); i++) {
	    SimilarityStatement sims = (SimilarityStatement) simStmts.get(i);
	    String str = i+" ";
	    if (sims==best) str+="BEST ";
	    str+= sims.toListString();
	    l.add(str);
	}
	vug.updateColours();
    }

    public void simFinderCallBack(){
	
    }
	
}
