/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;

/** GroupingStructure is a class containing methods for the valuation
    of grouping structure. The inner workings of the grouping
    viewpoint is contained in this class. */

public class GroupingStructure{
    
    static MotherGraph piece;
    public static Vector partsWithBoundaries;
    
    /** The constructor. */
    public GroupingStructure(){
    }
    
    /** Evaluates how well the subgrap is grouped according the
        grouping strucure rules implemented. */
    public double evaluateGroupings(Subgraph sub, double base){
	
	// find sources
	// se 2 noder tilbage/frem fra hver source der ikke er en pause
	int numberOfSinks;

	//hver sink skal have sin egen vurdering for om den er en god
	//afslutning, grupperingsmssigt
	Vector sinks = sub.getStrongSinks();//disse kan vre pauser!!!!!!
	numberOfSinks = sinks.size();

	//heri gemmes vurderingen for hver sink
	double[] sinkValues=new double[numberOfSinks];

	//for hver sink
	for(int i=0;i<sinkValues.length;i++){
	    Vector seqOfPrevAndThis=new Vector();
	    Vector seqOfNext2=new Vector();
	    MusicVertex mv=(MusicVertex)sinks.get(i);
	    if(mv instanceof SimpleVertex && ((SimpleVertex)mv).info.isRest()){
		sinkValues[i]=1.0;
	    }else{
		findPrevAndThis(mv,2,new MusicVertex[2], seqOfPrevAndThis);
		findNext2(mv,2,new MusicVertex[2],seqOfNext2);
		//merge
		Vector fourMusicVertices = combineTheArrays(seqOfPrevAndThis,seqOfNext2);
		if(fourMusicVertices.size()!=0){
		    double groupValue=evaluate(fourMusicVertices, base);
		    sinkValues[i]=groupValue;
		}else{
		    //one of the vectors is empty, it was not possible
		    //to get the surounding notes - we are close to
		    //the beginning/end of the piece -- do something
		    //else!
		    //the next to last note is not a good bound
		    if(mv.minDistanceFromEnd == 1 ){
			sinkValues[i]=1.0;
		    }else if(mv.minDistanceFromEnd == 0 ){//the last note is a good bound
			//sinkValues[i]=0.0;
			sinkValues[i]=base;
		    }else{//the sink the start (first note). This is
			//maybe ok, but not preferred
			sinkValues[i]=1.0;
		    }
		}
	    }
	}
	
	//gr det samme for sources
	int numberOfSources;
	//hver source skal have sin egen vurdering for om den er en god
	//begyndelse, grupperingsmssigt
	Vector sources=new Vector();
	sources = sub.getStrongSources();//disse kan vre pauser!!!!!!
	numberOfSources = sources.size();

	//heri gemmes vurderingen for hver source
	double[] sourceValues=new double[numberOfSources];

	//for hver source
	for(int i=0;i<sources.size();i++){
	    Vector seqOfPrev2=new Vector();
	    Vector seqOfThisAndNext=new Vector();
	    MusicVertex mv=(MusicVertex)sources.get(i);
	    if(mv instanceof SimpleVertex && ((SimpleVertex)mv).info.isRest()){
		sourceValues[i]=1.0;
	    }else{
		findPrev2(mv,2,new MusicVertex[2], seqOfPrev2);
		findThisAndNext(mv,2,new MusicVertex[2],seqOfThisAndNext);
		//merge
		Vector fourMusicVertices = combineTheArrays(seqOfPrev2,seqOfThisAndNext);
		if(fourMusicVertices.size()!=0){
		    double groupValue=evaluate(fourMusicVertices, base);
		    sourceValues[i]=groupValue;
		}else{
		    //one of the vectors is empty, it was not possible
		    //to get the surounding notes - we are close to
		    //the beginning/end of the piece -- do something
		    //else!
		    //the next-first note is not a good bound
		    if(mv.minDistanceFromStart == 1){
			sourceValues[i]=1.0;
		    }else if(mv.minDistanceFromStart == 0){
			//the first note is a good bound
			//sourceValues[i]=0.0;
			sourceValues[i]=base;
		    }else{//the source is the last note. This is
			//maybe ok, but not preferred
			sourceValues[i]=1.0;
		    }
		}
	    }
	}

	//nu haves source values og sink values - 
	//beregn gennemsnittet af alle vrdierne
	double sum1=0.0;
	double sum2=0.0;
	for(int i=0;i<sourceValues.length;i++){
	    sum1+=sourceValues[i];
	}
	sum1=sum1/(double)sourceValues.length;
	for(int i=0;i<sinkValues.length;i++){
	    sum2+=sinkValues[i];
	}
	sum2=sum2/(double)sinkValues.length;
	double res=(sum1+sum2)/2.0;
	return res;
	
    }

    private double evaluate(Vector v, double base){
	double size=(double)v.size();
	double bounds=0.0;
	for(int i=0;i<v.size();i++){
	    MusicVertex[] mva=(MusicVertex[])v.get(i);
	    bounds += evaluatePosition(mva,base);
	}
	return bounds/size;
    }

    private double evaluatePosition(MusicVertex[] mva, double base){
	double res=1.0;
	if(mva[1] instanceof CompoundVertex ||
	   mva[2] instanceof CompoundVertex){
	    res=base;
	}else if(mva[0] instanceof CompoundVertex ||
		 mva[3] instanceof CompoundVertex){
	    res=1.0;
	}else{//no compounds
	    int numOfappliances=0;
	    //	    double gpr=1.0;
	    if(evalGPR2a(mva) != null) numOfappliances++;
	    if(evalGPR2b(mva) != null) numOfappliances++;
	    //if(evalGPR3RegisterDiatonic(mva) != null)
	    if(evalFermata(mva) != null)  numOfappliances++;
	    if(evalGPR3Length(mva) != null)  numOfappliances++;
	    res=Math.pow(base,(double)numOfappliances);
	}
//  	}else{//no compounds
//  	    double gpr=0.0;
//  	    if(evalGPR2a(mva) != null ||
//  	       evalGPR2b(mva) != null ||
//  	       //if(evalGPR3RegisterDiatonic(mva) != null)
//  	       evalFermata(mva) != null ||
//  	       evalGPR3Length(mva) != null){
//  		res= 0.0;
//  	    }
//  	}
	return res;
    }

    //merge
    private Vector combineTheArrays(Vector beginnings, Vector endings){
	Vector v=new Vector();
	for(int i=0;i<beginnings.size();i++){
	    MusicVertex[] mv_i=(MusicVertex[])beginnings.get(i);
	    for(int j=i;j<endings.size();j++){
		MusicVertex[] mv_j=(MusicVertex[])endings.get(j);
		MusicVertex[] mva=new MusicVertex[4];
		mva[0]=mv_i[0];
		mva[1]=mv_i[1];
		mva[2]=mv_j[0];
		mva[3]=mv_j[1];
		v.add(mva);
	    }
	}
	return v;
    }

    private void findNext2(MusicVertex mv, int notesResting, MusicVertex[] found, Vector v){
	//da ingen af disse er pauser kan findThisAndNext kaldes p
	//alle out knuderne
	for(int i=0;i<mv.outEdges.size();i++){
	    if(((MusicEdge)(mv.outEdges.get(i))).isStrongFollowEdge()){
		MusicVertex musv=((MusicEdge)(mv.outEdges.get(i))).to;
		findThisAndNext(musv,2,new MusicVertex[2],v);
	    }
	}
    }
    
    private void findPrev2(MusicVertex mv, int notesResting, MusicVertex[] found, Vector v){
	//da ingen af disse er pauser kan findThisAndNext kaldes p
	//alle out knuderne
	for(int i=0;i<mv.inEdges.size();i++){
	    if(((MusicEdge)(mv.inEdges.get(i))).isStrongFollowEdge()){
		MusicVertex musv=((MusicEdge)(mv.inEdges.get(i))).from;
		findPrevAndThis(musv,2,new MusicVertex[2],v);
	    }
	}
    }
    
    private void findThisAndNext(MusicVertex mv, int notesResting, MusicVertex[] found, Vector v){
	
	if(mv instanceof SimpleVertex && ((SimpleVertex)mv).info.isRest() ){
	    
	}else{	//if not a rest
	    found[2-notesResting]=mv;
	    notesResting--;
	    if(notesResting==0){
		v.add(found);
		return;
	    }
	}
	// if this are in the last note of the piece, then no more
	// will happen - we will not add more 2-note-arrays. So if
	// seqOf2Notes is empty, we have started from either the last
	// note or the next to last note! This is important since the next to last note!
	for(int i=0;i<mv.outEdges.size();i++){
	    if(((MusicEdge)(mv.outEdges.get(i))).isStrongFollowEdge()){
		MusicVertex musv=((MusicEdge)(mv.outEdges.get(i))).to;
		findThisAndNext(musv,notesResting, arrayCopy(found),v);
	    }
	}
    }
    
    private void findPrevAndThis(MusicVertex mv, int notesResting, MusicVertex[] found, Vector v){
	
	if(mv instanceof SimpleVertex && ((SimpleVertex)mv).info.isRest() ){
	    
	}else{	//if not a rest
	    found[notesResting-1]=mv;
	    notesResting--;
	    if(notesResting==0){
		v.add(found);
		return;
	    }
	}
	for(int i=0;i<mv.inEdges.size();i++){
	    if(((MusicEdge)(mv.inEdges.get(i))).isStrongFollowEdge()){
		MusicVertex musv=((MusicEdge)(mv.inEdges.get(i))).from;
		findPrevAndThis(musv,notesResting, arrayCopy(found),v);
	    }
	}
    }

    /* evaluate SimpleVertices */

    private static String evalGPR2a(MusicVertex[] mva){
	NoteInfo note1=((SimpleVertex)mva[0]).info;
	NoteInfo note2=((SimpleVertex)mva[1]).info;
	NoteInfo note3=((SimpleVertex)mva[2]).info;
	NoteInfo note4=((SimpleVertex)mva[3]).info;

	double n1n2=note2.starttime-note1.endTime();
	double n2n3=note3.starttime-note2.endTime();
	double n3n4=note4.starttime-note3.endTime();

	String boundary=null;
	if (n2n3>n1n2 && n2n3>n3n4) {
	    boundary="GPR 2a";
	}
	return boundary;
    }

    private static String evalGPR2b(MusicVertex[] mva){
	NoteInfo note1=((SimpleVertex)mva[0]).info;
	NoteInfo note2=((SimpleVertex)mva[1]).info;
	NoteInfo note3=((SimpleVertex)mva[2]).info;
	NoteInfo note4=((SimpleVertex)mva[3]).info;
	double n1n2=note2.starttime-note1.starttime;
	double n2n3=note3.starttime-note2.starttime;
	double n3n4=note4.starttime-note3.starttime;
	
	String boundary=null;
	if (n2n3>n1n2 && n2n3>n3n4) {
	    boundary="GPR 2b";
	}
	return boundary;
    }
    
    private static String evalGPR3RegisterDiatonic(MusicVertex[] mva){
	NoteInfo note1=((SimpleVertex)mva[0]).info;
	NoteInfo note2=((SimpleVertex)mva[1]).info;
	NoteInfo note3=((SimpleVertex)mva[2]).info;
	NoteInfo note4=((SimpleVertex)mva[3]).info;
	int[] names={note1.name, note2.name, note3.name, note4.name};
	int[] octaves={note1.octave, note2.octave, note3.octave, note4.octave};
	int[] jumps=SeqViewComparator.getDiatonicIntervalArray(names,octaves);
	// skal konverteres til abs spring (ingen negative)
	double n1n2=Math.abs(jumps[0]);
	double n2n3=Math.abs(jumps[1]);
	double n3n4=Math.abs(jumps[2]);
	String boundary=null;
	if (n2n3>n1n2 && n2n3>n3n4 && n2n3>2) { //alts mere end en terts
	    boundary="GPR 3 a register, diatonic";
	}
	return boundary;
    }
    
    private static String evalGPR3Length(MusicVertex[] mva){
	NoteInfo note1=((SimpleVertex)mva[0]).info;
	NoteInfo note2=((SimpleVertex)mva[1]).info;
	NoteInfo note3=((SimpleVertex)mva[2]).info;
	NoteInfo note4=((SimpleVertex)mva[3]).info;

	double rvn1=note1.theNote.getRhythmValue();
	double rvn2=note2.theNote.getRhythmValue();
	double rvn3=note3.theNote.getRhythmValue();
	double rvn4=note4.theNote.getRhythmValue();

	String boundary=null;
	if (rvn2!=rvn3 && rvn1==rvn2 && rvn3==rvn4) {
	    boundary="GPR 3d length";
	}
	return boundary;
    }

    private static String evalFermata(MusicVertex[] mva){
	NoteInfo note2=((SimpleVertex)mva[1]).info;

	String boundary=null;
	if (note2.fermata) {
	    boundary="fermata";
	}
	return boundary;
    }



    /*************************** for label inserting **************************************/

    //last real note/Comp -- not rest.
    private boolean isLastNote(MusicVertex mv){

	for(int i=0;i<mv.outEdges.size();i++){
	    MusicVertex musv=((MusicEdge)(mv.outEdges.get(i))).to;
	}

	return false;
    }


    private MusicVertex[] arrayCopy(MusicVertex[] a){
	MusicVertex[] copy=new MusicVertex[a.length];
	for(int i=0;i<a.length;i++){
	    copy[i]=a[i];
	}
	return copy;
    }

    private void printSequences(Vector v){
	System.out.println("---");
	for(int i=0;i<v.size();i++){
	    MusicVertex[] mva = (MusicVertex[])v.get(i);
	    System.out.print("["+mva[0]+", ");
	    System.out.println(mva[1]+"]");
	}
    }
    
    /* used for debugging - fills in labels telling where the rules apply */
    public GroupingStructure(MotherGraph piece){
	
	Vector boundedParts = (Vector)(piece.partsFromMusedata.clone());
	for(int i=0;i<boundedParts.size();i++){
	    Vector pa=(Vector)boundedParts.get(i);
	    System.out.println("part "+i+" length="+pa.size());
	    pa=findBoundaries(pa);
	    System.out.println("pa.size()="+pa.size());
	    printThePart(pa);
	}
    }
    
    public static void main(String[] args){
	piece = new MotherGraph(args[0],
				true, //interpartEdges
				false, //combineTies
				false //combineRests
				);
	Subgraph s=null;
	try{
	    //	    s = piece.getRandomSeqSubgraph(10);
	    s = piece.getRandomNonSeqSubgraph(3);
	    System.out.println(s.toString());
	}catch (Exception e){}
	GroupingStructure g=new GroupingStructure();
	
	//	GroupingStructure g=new GroupingStructure(piece);
    }

    //    private static int winStart=-1;
    private static int winEnd=-1;
    private static int n1=-1;
    private static int n2=-1;
    private static int n3=-1;
    private static int n4=-1;
    private static Vector thePart=new Vector();
    
    
    public static Vector findBoundaries(Vector part){
	/* init */
	thePart.clear();
	thePart=part;
	winEnd=-1;
	n1=-1;
	n2=-1;
	n3=-1;
	n4=-1;
	
	boolean cont=true;
	if(cont) {
	    cont=fillWindow();
	}
	//nu er der et fyldt vindue
	//indstter en startmarkr fr frste rigtige node (n1)
	insertStartBoundBefore_n1();
	while(cont){
	    //printWindow();
	    //apply rules - indst passende boundaries
	    evalGPR2a();
	    evalGPR2b();
	    //evalGPR3RegisterDiatonic();
	    evalGPR3Length();
	    //move
	    cont = moveWindow();
	}
	Vector res=(Vector)thePart.clone();
	thePart.clear();
	return res;
    }

    private static void evalGPR2a(){
	NoteInfo note1=(NoteInfo)thePart.get(n1);
	NoteInfo note2=(NoteInfo)thePart.get(n2);
	NoteInfo note3=(NoteInfo)thePart.get(n3);
	NoteInfo note4=(NoteInfo)thePart.get(n4);

	double n1n2=note2.starttime-note1.endTime();
	double n2n3=note3.starttime-note2.endTime();
	double n3n4=note4.starttime-note3.endTime();

	if (n2n3>n1n2 && n2n3>n3n4) {
	    insertBoundaries_n2n3("GPR 2a");
	}
    }
    /*
    private static boolean evalGPR2a(MusicVertex[] prev, MusicVertex[] next){
	String res = null;
	NoteInfo note1=prev[0];
	NoteInfo note2=prev[1];
	NoteInfo note3=next[0];
	NoteInfo note4=next[1];

	double n1n2=note2.starttime-note1.endTime();
	double n2n3=note3.starttime-note2.endTime();
	double n3n4=note4.starttime-note3.endTime();

	if (n2n3>n1n2 && n2n3>n3n4) {
	    res = "evalGPR2a";
	}
	return res;
    }
    */
    private static void evalGPR2b(){
	NoteInfo note1=(NoteInfo)thePart.get(n1);
	NoteInfo note2=(NoteInfo)thePart.get(n2);
	NoteInfo note3=(NoteInfo)thePart.get(n3);
	NoteInfo note4=(NoteInfo)thePart.get(n4);
	double n1n2=note2.starttime-note1.starttime;
	double n2n3=note3.starttime-note2.starttime;
	double n3n4=note4.starttime-note3.starttime;
	if (n2n3>n1n2 && n2n3>n3n4) {
	    insertBoundaries_n2n3("GPR 2b");
	}
    }

    private static void evalGPR3RegisterDiatonic(){
	NoteInfo note1=(NoteInfo)thePart.get(n1);
	NoteInfo note2=(NoteInfo)thePart.get(n2);
	NoteInfo note3=(NoteInfo)thePart.get(n3);
	NoteInfo note4=(NoteInfo)thePart.get(n4);
	int[] names={note1.name, note2.name, note3.name, note4.name};
	int[] octaves={note1.octave, note2.octave, note3.octave, note4.octave};
	int[] jumps=SeqViewComparator.getDiatonicIntervalArray(names,octaves);
	/* skal konverteres til abs spring (ingen negative)*/
	double n1n2=Math.abs(jumps[0]);
	double n2n3=Math.abs(jumps[1]);
	double n3n4=Math.abs(jumps[2]);
	if (n2n3>n1n2 && n2n3>n3n4 && n2n3>2 /*alts mere end en terts*/) {
	    insertBoundaries_n2n3("GPR 3 a register, diatonic");
	}
    }
    
    private static void evalGPR3Length(){
	NoteInfo note1=(NoteInfo)thePart.get(n1);
	NoteInfo note2=(NoteInfo)thePart.get(n2);
	NoteInfo note3=(NoteInfo)thePart.get(n3);
	NoteInfo note4=(NoteInfo)thePart.get(n4);

	double rvn1=note1.theNote.getRhythmValue();
	double rvn2=note2.theNote.getRhythmValue();
	double rvn3=note3.theNote.getRhythmValue();
	double rvn4=note4.theNote.getRhythmValue();

	if (rvn2!=rvn3 && rvn1==rvn2 && rvn3==rvn4) {
	    insertBoundaries_n2n3("GPR 3d length");
	}
    }

     /* fills the window with 4 notes and possible some rests */
    private static boolean fillWindow(){
	boolean success=false;
	success=moveWinEndToNextNote();
	if(success) {
	    n1=winEnd;
	    success=moveWinEndToNextNote();
	}
	if(success) {
	    n2=winEnd;
	    success=moveWinEndToNextNote();
	}
	if(success) { 
	    n3=winEnd;
	    success=moveWinEndToNextNote();
	}
	if(success) { 
	    n4=winEnd;
	}
	return success;
    }

    private static void printWindow(){
	System.out.println(/*"winStart="+winStart+*/
			   "winEnd="+winEnd+", n1="+n1+", n2="+n2+", n3="+n3+", n4="+n4);
    }

    /* invariant: window skal altid begynde og slutte med en rigtig node 
       og indeholde 4 noder
    */

    public static void printThePart(Vector part){
	System.out.println("Printing the part...");
	System.out.println("----------------------------");
	for(int i=0;i<part.size();i++){
	    Object o = part.get(i);
	    if(o instanceof NoteInfo){System.out.println(((NoteInfo)o).toString());}
	    else if (o instanceof String){System.out.println(o.toString());}
	    else {System.out.println("error");}
	}
	System.out.println("----------------------------");
	System.out.println("printing done.");
    }

    private static boolean moveWindow(){
	boolean success=false;
	success=moveWinEndToNextNote();
	if(success){
	    n1=n2;
	    n2=n3;
	    n3=n4;
	    n4=winEnd;
	}
	return success;
    }
    
    /* tilfjer en startmarkr til thePart p plads n1 */
    private static void insertStartBoundBefore_n1(){
	thePart.add(n1,"start of piece");
	n1++;
	n2++;
	n3++;
	n4++;
	winEnd++;
    }
    
    private static void insertEndBoundAfter_n4(){
	thePart.add(n4+1,"end of piece");
	winEnd++;
    }

    /* [n2, r, r, n3] -> [n2, end, r, r, start, n3] */
    private static void insertBoundaries_n2n3(String label){
	thePart.add(n2+1,"end   "+label);
	n3++; n4++;
	thePart.add(n3,  "start "+label);
	n3++; n4++;
	winEnd+=2;
	
    }
    /*
    static private boolean moveWinStartToNextNote(){
	boolean succeeded = false;
	boolean cont=true;
	while(cont){
	    winStart++;
	    try{
		NoteInfo ni = (NoteInfo)thePart.get(winStart);
		if(ni.isNote()){cont=false; succeeded = true;}
		
	    }catch (Exception e){
		//System.out.println(e);
		succeeded=false; cont=false;
	    }
	}
	return succeeded;
    }
    */
    
    static private boolean moveWinEndToNextNote(){
	boolean succeeded = true;
	boolean cont=true;
	while(cont){
	    winEnd++;
	    try{
		Object o = thePart.get(winEnd);
		if(o instanceof NoteInfo){
		    NoteInfo ni = (NoteInfo)o;
		    if(ni.isNote()){
			cont=false; 
			succeeded = true;
		    }
		}else{
		    System.out.println("not NoteInfo: "+o.toString());
		    //printThePart(thePart);
		    System.exit(0);
		}
	    }catch (ArrayIndexOutOfBoundsException e){
		/* sidste node i parten 
		   indst end symbol efter n4 (sidste node)
		 */
		insertEndBoundAfter_n4();
		succeeded=false; cont=false;
	    }
	    catch (ClassCastException e){
		System.out.println(e);
		
	    }
	}
	return succeeded;
    }
    

    
}
