/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.io.*;
import java.util.*;
import jm.music.data.*;

public class MusedataTokenizer{
    
    BufferedReader file;
    int lineNumber;
    double starttime;
    double lastStarttime;
    Score dummyScore;
    boolean printWarnings=false;
    boolean printInfo=false;
    
    /** @param mdFile the MuseData file. Must be in format 2. */
    public MusedataTokenizer(String mdFile){
	try{
	    file = new BufferedReader(new FileReader(mdFile));
	}catch(FileNotFoundException e){
	    System.out.println(e);
	    System.exit(1);
	}
    }
    
    /** @return a Vector containing the parts of the score represented
        as Vectors of {@link NoteInfo} objects.*/
    public Vector getGraph(boolean combineTies, boolean combineRests){
	System.out.println("Reading Musedata file...");
	Vector verticesInParts=new Vector();
	verticesInParts=addNotes(verticesInParts);
	
	//	printVector(verticesInParts);
	//udfr repetitioner
	verticesInParts=makeRepetitions(verticesInParts);
	//printVector(verticesInParts);
	
	//husk at samle ties til een noteInfo
	if(combineTies) verticesInParts=combineTies(verticesInParts);
	if(combineRests) verticesInParts=combineRests(verticesInParts);
	System.out.println("Musedata reading done.");
	
	return verticesInParts;
    }
    
    private Vector addNotes(Vector parts){//parts er her tom
	boolean inPart=false;
	int partNumber=1;
	Part dummyPart=new Part();
	Vector thisPart=new Vector();
	dummyScore=new Score();
	String line=null;
	boolean inComment=false;
	starttime=0.0;
	lastStarttime=0.0;
	//Musedata field identifier variables:
	int divisionsPrQuarterNote=0;
	int key=100;//-3 means 3 notated flats, 4 means 4 notated sharps
	//100 means not properly initialised
	int timeSigUpper=0;
	int timeSigLower=0;

	while((line=getLine()) != null){//den store lkke
	    //System.out.print(lineNumber+", ");
	    //for hver part: husk at lave om i:
	    // partNumber, divisionsPrQuarterNote, 

	    
	    //$
	    //in the attribute fields
	    char c='';
	    if (!line.equals("")) {
		c=line.charAt(0);
	    }
	    if (c==''){}
	    else if(c=='&'){
		inComment=!inComment;
	    }//comment mode toggle
	    else if(c=='$' && !inComment){//
		inPart=true;
		
		StringTokenizer tok=new StringTokenizer(line," ");
		//if(printWarnings) {System.out.println("Tokenizing field identifiers");}
		while(tok.hasMoreTokens()){
		    String field=tok.nextToken();
		    field=field.trim();
		    if(field.length()>0){
			char fieldChar=field.charAt(0);
			if(fieldChar=='K'){
			    key=new Integer(field.substring(2,field.length())).intValue();
			}else if(fieldChar=='Q'){
			    divisionsPrQuarterNote=new Integer(field.substring(2,field.length())).intValue();
			}else if(fieldChar=='T'){
			    String timeSig=field.substring(2,field.length());
			    int slash=timeSig.indexOf("/");
			    timeSigUpper=new Integer(timeSig.substring(0,slash)).intValue();
			    timeSigLower=new Integer(timeSig.substring(slash+1,timeSig.length())).intValue();
			}else if(fieldChar=='C'){
			    /*Cn..*/ if(printWarnings) reportError("Clef information discarded!",false);
			}else if(fieldChar=='X'){
			    if(printWarnings) reportError("Field identifier X not implemented yet!",false);
			}else if(fieldChar=='S'){
			    if(printWarnings) reportError("Field identifier S not implemented yet!",false);
			}else if(fieldChar=='I'){
			    if(printWarnings) reportError("Field identifier I not implemented yet!",false);
			}else if(fieldChar=='D'){/*Dn..*/
			    if(printWarnings) reportError("Field identifier D not implemented yet!",false);}
		    }
		}
		// System.out.println("Tokenizing line over");
	    }else if(c=='/' && !inComment){
		inPart=false;
		dummyScore.addPart(dummyPart);
		parts.add(thisPart);
		thisPart=new Vector();
		dummyPart=new Part(""+partNumber);
		partNumber++;
		starttime=0.0;
		lastStarttime=0.0;
		//	    }
		//  	    if(c=='/' && inComment){
		//  		System.out.println("n");
		//  		System.exit(0);
	    }else if((c=='P' || c=='S' || c=='a' || c=='b' || c=='c' || c=='f' || c=='g' || c=='i' ||
		     c=='*') && inPart && !inComment){
		if(printWarnings) {
		    reportError("Control key "+c+" not implemented yet! see page 410 in Selfridge-Field",false);
		}
	    }else if(inPart && c!='$' && !inComment){
		//System.out.println("Going in for a Note in line "+lineNumber);
		if(c=='r'){//rest
		    double rhythmValue=0.0;
		    double divisionValue=1.0/(double)divisionsPrQuarterNote;
		    //find rhythmValue
		    String divisions=line.substring(5,8);
		    divisions=divisions.trim();
		    rhythmValue=(new Double(divisions).doubleValue())*divisionValue;
		    rhythmValue=roundRhythmValue(rhythmValue);
		    //find ties
		    boolean tied=false;
		    if(line.length()>8){
			String tie=line.substring(8,9);
			tied=(tie.equals("-"));
		    }
		    int tiedToTrack=-1;
		    if(tied){
			tiedToTrack=getTiedTrackNumber(line);
		    }
		    //alts er rv klar

		    //find fermata
		    boolean fermata=fermataApplied(line);

		    thisPart.add(new NoteInfo(
					      partNumber,dummyPart,-1,-1,-1,-1,
					      rhythmValue,starttime,
					      key,timeSigUpper,timeSigLower,
					      tied,tiedToTrack,fermata,lineNumber));
		    
		    //opdater starttime *efter* add
		    updateStarttime(rhythmValue);

		}else if(c=='m'){//measure
		    if(line.length()>8){//s findes et measure number
			String mn="";
			if(line.length()>12){mn=(line.substring(8,12)).trim();}
			else {mn=(line.substring(8,line.length())).trim();}
			if(mn.length()>0){//i takt 1 er intet measure number
			    int measurenumber=(new Integer(mn).intValue());
			    //thisPart.add("measure obj?");
			}
			String flags="";
			if(line.length()>16){
			    flags=(line.substring(16,line.length())).trim();
			    if(flagsContain(flags,":|")){
				thisPart.add(":|");
			    }
			    if(flagsContain(flags,"|:")){
				thisPart.add("|:");
			    }
			}

		    }else{
			//thisPart.add("heavy measure obj?");
		    }
		    
		}else if(c==' '){//chord note found
		    double rhythmValue=0.0;
		    double divisionValue=1.0/(double)divisionsPrQuarterNote;
		    int sharps=0;
		    int flats=0;
		    String pitch=line.substring(1,5);
		    pitch=pitch.trim();
		    //find name
		    int name=getName(pitch.charAt(0));
		    //find octave
		    int octave=new Integer(pitch.substring(pitch.length()-1,pitch.length())).intValue();
		    //find alterations
		    if(pitch.length()==2){
			sharps=flats=0;
		    }else if(pitch.length()==3){
			if(pitch.charAt(1)=='#'){
			    sharps=1;
			}else if(pitch.charAt(1)=='f'){
			    flats=1;
			}else{reportError("encoding length of not is 3, but no sharps/flats!, but instead"+pitch.charAt(1));}
		    }else if(pitch.length()==4){
			String ss=pitch.substring(1,3);
			if(ss.equals("##")){sharps=2;}
			else if(ss.equals("ff")){sharps=2;}
			else{reportError("encoding length of note is 4, but no sharps/flats!");}
		    }else{
			reportError("not known character: "+c+" pitchlinelength is "+pitch.length());
		    }
		    //find rhythmValue
		    if(line.length()>5){
			String divisions=line.substring(5,8);
			divisions=divisions.trim();
			if(divisions.length()!=0){
			    rhythmValue=(new Double(divisions).doubleValue())*divisionValue;
			    rhythmValue=roundRhythmValue(rhythmValue);
			}else{//ingen indikation af rv => brug den forrige nodes rv
			    rhythmValue=((NoteInfo)(thisPart.lastElement())).theNote.getRhythmValue();
			    rhythmValue=roundRhythmValue(rhythmValue);
			    if (printInfo) {reportError("Info: old rhythmValue used",false);}
			}
		    }
		    //find ties
		    boolean tied=false;
		    if(line.length()>8){
			String tie=line.substring(8,9);
			tied=(tie.equals("-"));
		    }
		    int tiedToTrack=-1;
		    if(tied){
			tiedToTrack=getTiedTrackNumber(line);
		    }
		    
		    //find ties, slurs, ornaments, technical violon indications, 
		    //articulation accents, dynamic expressions and ##fermatas##
		    boolean fermata=fermataApplied(line);
		    
		    //alts er nu flg klar:
		    //name, octave, flats, sharps, rhythmValue, tied, fermata


		    //lav nu et nodeInfo obj af dette
		    //			NoteInfo n = new Noteinfo(...);
		    // thisPart.add

		    thisPart.add(new NoteInfo(partNumber,dummyPart,name,octave,
					      sharps,flats,rhythmValue,
					      lastStarttime,
					      key,timeSigUpper,timeSigLower,
					      tied,tiedToTrack,fermata,lineNumber));
		    //opdater *ikke* starttime (da den er som de andre i akkorden)


		}else if(c=='C' || c=='D' ||c=='E'|| c=='F' ||c=='G' ||
			 c=='A' || c=='B'){//note found
		    double rhythmValue=0.0;
		    double divisionValue=1.0/(double)divisionsPrQuarterNote;
		    
		    int sharps=0;
		    int flats=0;
		    String pitch=line.substring(0,4);
		    pitch=pitch.trim();

		    //find name
		    int name=getName(pitch.charAt(0));
		    //find octave
		    int octave=new Integer(pitch.substring(pitch.length()-1,pitch.length())).intValue();

		    //find alterations
		    if(pitch.length()==2){
			sharps=flats=0;
		    }else if(pitch.length()==3){
			if(pitch.charAt(1)=='#'){
			    sharps=1;
			}else if(pitch.charAt(1)=='f'){
			    flats=1;
			}else{reportError("encoding length of not is 3, but no sharps/flats!, but instead"+pitch.charAt(1));}
		    }else if(pitch.length()==4){
			String ss=pitch.substring(1,3);
			if(ss.equals("##")){sharps=2;}
			else if(ss.equals("ff")){sharps=2;}
			else{reportError("encoding length of note is 4, but no sharps/flats!");}
		    }else{
			reportError("not known character"+c+"pitchlinelength is "+pitch.length());
		    }

		    //find rhythmValue
		    String divisions=line.substring(5,8);
		    divisions=divisions.trim();
		    /* very useful debugging line */
		    //	    System.out.println(lineNumber+", "+divisions);
		    rhythmValue=(new Double(divisions).doubleValue())*divisionValue;
		    rhythmValue=roundRhythmValue(rhythmValue);
		    //find ties
		    boolean tied=false;
		    if(line.length()>8){
			String tie=line.substring(8,9);
			tied=(tie.equals("-"));
		    }
		    int tiedToTrack=-1;
		    if(tied){
			tiedToTrack=getTiedTrackNumber(line);
		    }
		    
		    //find ties, slurs, ornaments, technical violon indications, 
		    //articulation accents, dynamic expressions and ##fermatas##
		    boolean fermata=fermataApplied(line);

		    //alts er nu flg klar:
		    //name, octave, flats, sharps, rhythmValue, tied, fermata
	    
		    //lav nu et nodeInfo obj af dette
		    //			NoteInfo n = new Noteinfo(...);
		    // thisPart.add

		    thisPart.add(new NoteInfo(partNumber,dummyPart,name,
					      octave,sharps,flats,
					      rhythmValue,starttime,
					      key,timeSigUpper,timeSigLower,
					      tied,tiedToTrack, fermata, lineNumber));

		    //opdater starttime *efter* add
		    updateStarttime(rhythmValue);
		}//end note
		else{
		    if(printWarnings){reportError("Control key "+c+" not implemented",false);}
		}
	    }
	}
	return parts;
    }

    private void updateStarttime(double timespan){
	/*
	lastStarttime=starttime;
	starttime+=timespan;
	
	starttime=roundRhythmValue(starttime);
	starttime=checkDivisionRounding(starttime);
	*/        
	timespan=roundRhythmValue(timespan);
	starttime=checkDivisionRounding(starttime);
	
	lastStarttime=starttime;
	starttime+=timespan;
	
	starttime=roundRhythmValue(starttime);
	starttime=checkDivisionRounding(starttime);
    }
    
    private int getName(char c){
	int n=-1;
	if(c=='C'){n=0;}
	if(c=='D'){n=1;}
	if(c=='E'){n=2;}
	if(c=='F'){n=3;}
	if(c=='G'){n=4;}
	if(c=='A'){n=5;}
	if(c=='B'){n=6;}
	if(c==-1){reportError("unknown name encoding!");}
	return n;
    }

    private int getTiedTrackNumber(String theLine){
	int track=-1;
	if(theLine.length()>=15){
	    String s=theLine.substring(14,15);
	    if(!s.equals(" ")){
		System.out.println("tied to track "+s);
		if(printInfo) reportError("wahuu: track-fun finally found!\ndo something with the combineTies function!",true);
		track=new Integer(s).intValue();
	    }
	}
	return track;
    }
    
    private void reportError(String s){
	 System.out.println("Encoding error in line: "+lineNumber+":");
	 System.out.println(s);
	 System.exit(1);
    }
    private void reportError(String s, boolean stopExecution){
	 System.out.println("Encoding warning in line: "+lineNumber+":");
	 System.out.println(s);
	 if(stopExecution){
	     System.exit(1);
	 }
    }
    
    private String getLine(){
    	String s=null;
	try{
	    s=file.readLine();
	}catch(Exception e){
	    System.out.println(e);
	    System.exit(1);
	}
	/*
	if(s.length()>1){
	    String ss=s.substring(s.length()-2,s.length());
	    if(ss.equals("^M")){
		s=s.substring(0,s.length()-2);
		System.out.println("appendix removed!");
	    }
	}
	*/
	lineNumber++;
	return s;
    }

    private Vector makeRepetitions(Vector vOfVectors){
	int repeatStartIndex=0;
	int repeatEndIndex=-1;
	System.out.println("Making Repetitions");
	for(int i=0;i<vOfVectors.size();i++){
	    Vector vOfNoteInfos=(Vector)vOfVectors.get(i);
	    boolean moreRepetitions=true;
	    while(moreRepetitions){
		int j=0;
		boolean changed=false;
		while(j<vOfNoteInfos.size()){
		    Object o=vOfNoteInfos.get(j);
		    if (o instanceof String){
			if (((String)o).equals(":|")){
			    repeatEndIndex=j-1;
			    vOfNoteInfos.remove(j);
			    vOfNoteInfos=insertRepetition(repeatStartIndex,repeatEndIndex,vOfNoteInfos);
			    changed=true;
			}else if (((String)o).equals("|:")){
			    repeatStartIndex=j;
			    vOfNoteInfos.remove(j);
			    repeatEndIndex=-1;
			}
		    }
		    if(changed){
			j=0;
			changed=false;
		    }
		    else{j++;}
		}
		moreRepetitions=false;
	    }
	    repeatStartIndex=0;
	    repeatEndIndex=-1;
	}
	return vOfVectors;
    }

    /* the sequence of notes to be added have index start..end
       the sequence should be added after index end
     */
    private Vector insertRepetition(int start, int end, Vector v){
	//System.out.println("(start,end)=("+start+","+end+")");
	Vector repetition=new Vector();
	double passageLength=0.0;
	for(int i=start;i<=end;i++){
	    NoteInfo n=(NoteInfo)v.get(i);
	    NoteInfo copy=(NoteInfo)n.clone();
	    repetition.add(copy);
	    passageLength+=copy.theNote.getRhythmValue();
	}
	//System.out.println("rep size="+repetition.size());
	//System.out.println("passageLength="+passageLength);
	int afterRep=-1;
	for(int i=0;i<repetition.size();i++){
	    Object o=repetition.get(i);
	    if(o instanceof NoteInfo){
		NoteInfo n=(NoteInfo)o;
		n.starttime=n.starttime+passageLength;
		v.add(end+1+i,n);
		afterRep=end+1+i+1;
	    }
	}
	for(int i=afterRep;i<v.size();i++){
	    Object o=v.get(i);
	    if(o instanceof NoteInfo){
		NoteInfo n=(NoteInfo)o;
		n.starttime=n.starttime+passageLength;
	    }
	}
	
	return v;
    }

    private void printVector(Vector vOfVectors){
	System.out.println("********");
	for(int i=0;i<vOfVectors.size();i++){
	    Vector vOfNoteInfos=(Vector)vOfVectors.get(i);
	    for(int j=0;j<vOfNoteInfos.size()-1;j++){
		Object o=vOfNoteInfos.get(j);
		if(o instanceof String){System.out.println(o.toString());}
	    }
	}
	System.out.println("********");
    }

    private Vector combineRests(Vector vOfVectors){
	for(int i=0;i<vOfVectors.size();i++){
	    Vector vOfNoteInfos=(Vector)vOfVectors.get(i);
	    for(int j=0;j<vOfNoteInfos.size()-1;j++){
		NoteInfo n1 = (NoteInfo)vOfNoteInfos.get(j);
		NoteInfo n2 = (NoteInfo)vOfNoteInfos.get(j+1);
		if(n1.theNote.getPitch()==Integer.MIN_VALUE &&
		   n2.theNote.getPitch()==Integer.MIN_VALUE){
		    n1.theNote=new Note(Integer.MIN_VALUE,
					n1.theNote.getRhythmValue()+
					n2.theNote.getRhythmValue()
					);
		    vOfNoteInfos.remove(j+1);
		    j--;
		}
	    }
	}
	return vOfVectors;
    }

    private Vector combineTies(Vector vOfVectors){
	for(int i=0;i<vOfVectors.size();i++){
	    Vector vOfNoteInfos=(Vector)vOfVectors.get(i);
	    for(int j=0;j<vOfNoteInfos.size()-1;j++){
		NoteInfo n = (NoteInfo)vOfNoteInfos.get(j);
		if(n.isTied){
		    // findTheFirstMatchingNote
		    boolean found=false;
		    for(int k=j+1;k<vOfNoteInfos.size() && !found;k++){
			NoteInfo partner=(NoteInfo)vOfNoteInfos.get(k);
			if(tiePartners(n,partner)){
			    n=combineTheNoteInfos(n,partner);
			    found=true;
			    vOfNoteInfos.remove(k);
			}
		    }
		}
	    }
	}
	return vOfVectors;
    }
    
    private boolean tiePartners(NoteInfo n1, NoteInfo n2){
	boolean partners=false;
	if(n1.md_part==n2.md_part &&
	   n1.name==n2.name &&
	   n1.octave==n2.octave &&
	   n1.sharps==n2.sharps &&
	   n1.flats==n2.flats &&
	   n1.octave==n2.octave &&
	   n2.starttime==n1.starttime+n1.theNote.getRhythmValue() &&
	   n1.tiedToTrackNumber==n2.tiedToTrackNumber
	       ){
	    partners=true;
	}
	return partners;
    }
    
    private NoteInfo combineTheNoteInfos(NoteInfo n1,NoteInfo n2){
	Note note=new Note(n1.theNote.getPitch(),
			   n1.theNote.getRhythmValue()+n2.theNote.getRhythmValue());
	n1.theNote=note;
	n1.isTied=n2.isTied;
	return n1;
    }

    private boolean flagsContain(String flagLine, String flag){
	//	System.out.println("in flagsContain");
	//	System.out.println("flagLine="+flagLine+", flag="+flag);

	int fl=flagLine.length();
	int f=flag.length();
	if(f>fl) {
	    return false;
	}

	for(int i=0;i<=fl-f;i++){
	    if (flagLine.substring(i,i+f).equals(flag)){
		//System.out.println("in flagsContain, flag \""+flag+"\" found in line "+lineNumber);
		return true;
	    }
	}
	return false;
    }

    private double roundRhythmValue(double d){
	//if(print)System.out.println("value:"+d);
	int time1000=(int)(1000.0*d);
	d=(double)time1000/1000.0;
	//System.out.println("Rounded value:"+d);
	return d;
    }

    private double checkDivisionRounding(double sTime){
	sTime=roundRhythmValue(sTime);
	double intPart=Math.floor(sTime);
	/* */
	double decimals=sTime-intPart;
	decimals=roundRhythmValue(decimals);

	boolean print=false;
	//System.out.print("time:"+sTime+", ");
	//System.out.println("intPart:"+intPart);
	//System.out.println("decimals:"+decimals);
	double slack=0.002;
	if(decimals==0.999){// 1/1
	    decimals=1.0;
	}else if(decimals==0.001){// 1/1
	    decimals=0.0;
	}else if(decimals>=0.666-slack && decimals<=0.666+slack){// 2/3
	    decimals=0.666;
	}else if(decimals>=0.25-slack && decimals<=0.25+slack){// 1/4
	    decimals=0.25;
	}else if(decimals>=0.5-slack && decimals<=0.5+slack){// 2/4
	    decimals=0.5;
	}else if(decimals>=0.75-slack && decimals<=0.75+slack){// 3/4
	    decimals=0.75;

	}else if(decimals>=0.583-slack && decimals<=0.583+slack){// 1/4+1/3
	    decimals=0.583;
	}else if(decimals>=0.833-slack && decimals<=0.833+slack){// 2/4+1/3
	    decimals=0.833;
	}else if(decimals>=0.083-slack && decimals<=0.083+slack){// 3/4+1/3
	    decimals=0.083;

	}else if(decimals>=0.916-slack && decimals<=0.916+slack){// 1/4+2/3
	    decimals=0.916;
	}else if(decimals>=0.166-slack && decimals<=0.166+slack){// 2/4+2/3
	    decimals=0.166;
	}else if(decimals>=0.416-slack && decimals<=0.416+slack){// 3/4+2/3
	    decimals=0.416;
	}
	    //   1/4+1/3=0.583
	    //   2/4+1/3=0.833
	    //   3/4+1/3=1.083
	    
	    //   1/4+2/3=0.916
	    //   2/4+2/3=1.166
	    //   3/4+2/3=1.416
	
	    
	return intPart+decimals;
    } 
    private boolean fermataApplied(String theLine){
	boolean fermata=false;
	if(theLine.length()>31){
	    String additionalNotations=theLine.substring(31,theLine.length());
	    if(additionalNotations.length()>12){
		additionalNotations=additionalNotations.substring(0,12);
	    }
	    fermata=( flagsContain(additionalNotations,"F") ||
		      flagsContain(additionalNotations,"E"));
	}
	//	if(fermata)System.out.println("Fermata found in line "+lineNumber);
	return fermata;
    }

}
