/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** This is the result of applying an NSSMEdge measure to a
    non-sequential similarity statement. */
public class NSSMEdgeResult extends SimMeasureResult {

    /** The mean similarities matrix. */
    public double[][] meanSimilarities;
    ViewDifference groupingValue=null;
    
    /** bestMatch is set to null (in the similarity measure) if
       noEdgeSubgraphs() is true. Otherwise it is an array of length=
       the longest side of the similarity matrix */
    public int[] bestMatch=null;

    public int multipleMatchingsInBest=-1;
    // Vector allMatches=null;

    /** For user interface purposes. */
    public final int doubleStringLength=4;

    public NSSMEdgeResult(double meas, NSSMEdge or, 
			    double[][] meansims, int[] bmatch,
			    int doublematchings,
			    Vector allMatch) {
	super(meas,or);
	meanSimilarities=meansims;
	bestMatch=bmatch;
	multipleMatchingsInBest = doublematchings;
	// allMatches = allMatch;
    }
    public NSSMEdgeResult( NSSMEdge or) {
	super(-1.0,or);
    }

    /** Returns true if one subgraph has no edges of the desired type.
    It may happen that one subgraph has no edges of the desired
    type. The desired type is decided by which 'getXXXSimMatrix()'
    method in {@link NSVCEdge} is called from the similarity
    measure. In that case some dimensions in the similarity matrix
    will be zero. We ask about this instead of the number of edges,
    because otherwise we have to remember which desired edge type that
    was called for in the 'getXXXSimMatrix()' method mentioned
    above.*/
    public int noEdgeSubgraphs() {
	if (meanSimilarities.length==0) return 2;
	else if (meanSimilarities[0].length==0) return 1;
	else return 0;
    }

    /** Tells if the two subgraphs are swapped for comparison
        purposes. If subgraph2 has more edges than subgraph1, then it
        is not the first index (that of subgraph1) in meanSimilarities
        that runs longest. But we always try to match all edges of
        that subgraph which has most edges, to some edge in the smaller
        (in number of edges) subgraph, so the matching should be
        "mirrored", starting from edges varying in index 2 rather than
        index 1 of the similarity matrix meanSimilarities. */
    public boolean mirrored() {
	if (noEdgeSubgraphs()>0) return false;
	if (meanSimilarities!=null) {
	    if (meanSimilarities.length<meanSimilarities[0].length) {
		return true;
	    }
	}
	return false;
    }

    /** Overrides this method because we don't want to see all the
        viewdiffs. */
    public String toStringWithViewDifferences() {
	return toString()+"\n";
    }

    /** Computes the mean similarities matrix. */
    public double[][] getMeanEdgeSimMatrix() {
	NSVDEdge vd = (NSVDEdge) viewDifferences.get(0);
	if (vd.similarities.length==0) return null;
	double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
	
	for (int i=0; i<viewDifferences.size(); i++) {
	    vd = (NSVDEdge) viewDifferences.get(i);
	    edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
	}
	return FuncLib.divideMatrixBy(edgeSimilarities,viewDifferences.size());
    }

    /** Computes the number of multiple matchings and the vector
        allMatches. */
    public void completeMatchData(NonSeqSimStmt nsss) {
	if (bestMatch!=null) {
	    multipleMatchingsInBest = FuncLib.countCopies(bestMatch,true);
	} else multipleMatchingsInBest=0;

	/*	allMatches = new Vector();
	for (int i=0; i<bestMatch.length; i++) {
	    allMatches.add(new MusicEdge((MusicVertex)nsss.subgraph1.vertices.get(i),
					 (MusicVertex)nsss.subgraph2.vertices.get(bestMatch[i]),
					 MusicEdge.MATCH));
					 }*/ 
    }

    public String toString() {
	String result="";
	result += "   "+origin.name+"="+measure+"\n";
	if (groupingValue!=null){
	    result += "   grouping="+groupingValue.view;
	}
	result += "   Similarities matrix=\n"+meanSimilaritiesToString();
	result += "   Best match=\n"+bestMatchToString();
	result += "   Multiple matchings in best="+multipleMatchingsInBest+"\n";

	return result;
    }

    private String meanSimilaritiesToString() {
	String result="";
	if (meanSimilarities==null || meanSimilarities.length==0) return "empty mean SimMatrix";
	for (int i=0; i</*subgraph1.size()*/ meanSimilarities.length; i++) {
	    result+="\t";
	    for (int j=0; j</*subgraph2.size()*/meanSimilarities[0].length; j++) {
		result+=FuncLib.doubleToStringOfLength(meanSimilarities[i][j],
						       doubleStringLength);
	    }
	    result+="\n";
	}
	return result;
    }
    private String bestMatchToString() {
	String result="";
	if (bestMatch==null) {
	    
	} else {
	    for (int i=0; i<bestMatch.length; i++) {
		try {
		    if (mirrored()) {
			result+="\tbestMatch["+i+"]="
			    +bestMatch[i]+ "("+FuncLib.doubleToStringOfLength(meanSimilarities[bestMatch[i]][i],doubleStringLength)+")\n";
		    } else {
			result+="\tbestMatch["+i+"]="
			    +bestMatch[i]+ "("+FuncLib.doubleToStringOfLength(meanSimilarities[i][bestMatch[i]],doubleStringLength)+")\n";
		    }
		} catch(Exception e) {
		    System.out.println("NSSMEdgeResult: i="+i);
		    System.out.println("NSSMEdgeResult: bestMatch.length="+bestMatch.length);
		    System.out.println("NSSMEdgeResult: meanSim.length="+meanSimilarities.length);
		    System.out.println("NSSMEdgeResult: meanSim[0].length"+meanSimilarities[0].length);
		}
		
	    }
	}
	return result;
    }
    
}
