/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** The non-sequential similarity measure "VERTEX" class contains a
    few methods proper to vertex comparison. */
public abstract class NSSMVertex extends NonSeqSimMeasure {

    public NSSMVertex(String s) {
	super(s);
    }
    
    /** Tries to find a best matching of vertices in the two subgraphs
        of a non-sequential similarity statement. */
    protected int[] getBestMatch(NonSeqSimStmt nsss,
				 NSSMVertexResult vr) {
	/* Try to find best matching */

	Vector [] ibest= new Vector[nsss.subgraph1.vertices.size()];
	// holds Vectors of Integers which are indices to the best matches

	for (int i=0; i<nsss.subgraph1.vertices.size(); i++) {
	    double best = Double.MAX_VALUE;
	    ibest[i]=new Vector();
	    for (int j=0; j<nsss.subgraph2.vertices.size(); j++) {
		if (vr.meanSimilarities[i][j]==best) {
		    ibest[i].add(new Integer(j));
		}
		else if (vr.meanSimilarities[i][j]<best) {
		    best=vr.meanSimilarities[i][j];
		    ibest[i].clear();
		    ibest[i].add(new Integer(j));
		}
	    }
	}
	
	int[] imatch= findBestCombination2(ibest);
	return imatch;
    }


    /** Uses max matching algorithm to match vertices. */
    protected void assignBestMatch2(NonSeqSimStmt nsss,
				    NSSMVertexResult vr) {
	/* Try to find best matching */

	Vector [] ibest= new Vector[nsss.subgraph1.vertices.size()];
	// holds Vectors of Integers which are indices to the best matches

	for (int i=0; i<nsss.subgraph1.vertices.size(); i++) {
	    double best = Double.MAX_VALUE;
	    ibest[i]=new Vector();
	    for (int j=0; j<nsss.subgraph2.vertices.size(); j++) {
		if (vr.meanSimilarities[i][j]==best) {
		    ibest[i].add(new Integer(j));
		}
		else if (vr.meanSimilarities[i][j]<best) {
		    best=vr.meanSimilarities[i][j];
		    ibest[i].clear();
		    ibest[i].add(new Integer(j));
		}
	    }
	}
	
	boolean[] unmatchedLeft = new boolean[ibest.length];
	boolean[] candidatesRight = new boolean[ibest.length];
	for(int i=0; i<unmatchedLeft.length; i++) {
	    unmatchedLeft[i]=false;
	    candidatesRight[i]=false;
	}
	
	for (int i=0; i<ibest.length; i++) {
	    if (ibest[i].size()>1) {
		unmatchedLeft[i]=true;
		for (int j=0; j<ibest[i].size(); j++) {
		    int cr = ((Integer) ibest[i].get(j)).intValue();
		    candidatesRight[cr]=true;
		}
	    }
	}
	int leftsize=FuncLib.howmanyTrue(unmatchedLeft);
	int rightsize=FuncLib.howmanyTrue(candidatesRight);

	int[] imatch= new int[ibest.length];
	for (int i=0; i<imatch.length; i++) imatch[i]=-1;

	if (leftsize>0) {
	    int[][] incidence = new int[leftsize][rightsize];
	    //System.out.println("leftsize="+leftsize+", rightsize="+rightsize);

	    // how to translate from indices in incidence[][] to
	    // indices in meanSimilarities[][]
	    int[] translateLeft = new int[leftsize];
	    int[] translateRight = new int[rightsize];
	    
	    int found=0;
	    for (int i=0; i<unmatchedLeft.length; i++) {
		if (unmatchedLeft[i]) {
		    translateLeft[found]=i;
		    found++;
		}
	    }
	    
	    found=0;
	    //	    System.out.println("candRight length="+candidatesRight.length);
	    for (int i=0; i<candidatesRight.length; i++) {
		if (candidatesRight[i]) {
		    translateRight[found]=i;
		    found++;
		}
	    }
	    
	    // build incidence matrix to run bipartiteMaxMatch on
	    for (int i=0; i<incidence.length; i++) {
		for (int j=0; j<incidence[0].length; j++) {
		    incidence[i][j]=Matching.NOEDGE;
		}
	    }
	    for (int i=0; i<ibest.length; i++) {
		if (ibest[i].size()>1) {
		    for (int j=0; j<ibest[i].size(); j++) {
			int cr = ((Integer) ibest[i].get(j)).intValue();
			int leftindex=reverseTranslate(i,translateLeft);
			int rightindex=reverseTranslate(cr,translateRight);
			incidence[leftindex][rightindex]=Matching.RIGHTEDGE;
		    }
		} else if (ibest[i].size()==1) {
		    imatch[i]=((Integer)ibest[i].get(0)).intValue();
		}
	    }
	    int[] thematch=Matching.bipartiteMaxMatch(incidence);
	    
	    /*	    for (int i=0; i<thematch.length; i++) {
		System.out.println("thematch["+i+"]="+thematch[i]);
		}*/
	    //	    System.out.println();

	    //System.out.println("gonna remove -1");
	    for (int i=0; i<thematch.length; i++) {
		int imInxLeft=0;
		int imInxRight=0;
		if (thematch[i]==-1) {
		    int inx = translateLeft[i];
		    //System.out.println("replacing -1 at index "+i);
		    imInxLeft=translateLeft[i];
		    imInxRight=((Integer) ibest[inx].get(0)).intValue();

		} else {
		    imInxLeft=translateLeft[i];
		    imInxRight=translateRight[thematch[i]];
		}
		
		imatch[imInxLeft]=imInxRight;
	    }
	    /*	    for (int i=0; i<imatch.length; i++) {
		System.out.println("imatch["+i+"]="+imatch[i]);
	    }
	    System.out.println();*/

	}
	else {
	    for (int i=0; i<ibest.length; i++) {
		int cr = ((Integer) ibest[i].get(0)).intValue();
		imatch[i] =cr;
	    }

	}

	vr.bestMatch=imatch;
	vr.multipleMatchingsInBest=FuncLib.countCopies(imatch,true);

	Vector allMatches = new Vector();
	for (int i=0; i<imatch.length; i++) {
	    Vector fromAnch= new Vector();
	    Vector toAnch = new Vector();
	    fromAnch.add(new Integer(0));
	    toAnch.add(new Integer(0));
	    allMatches.add(new MusicEdge((MusicVertex)nsss.subgraph1.vertices.get(i),
					 (MusicVertex)nsss.subgraph2.vertices.get(imatch[i]),
					 MusicEdge.MATCH,
					 fromAnch, toAnch));
	}
	vr.allMatches = allMatches;
    }

    private int reverseTranslate(int findthis, int[] translation) {
	for (int i=0; i<translation.length; i++) {
	    if (translation[i]==findthis) return i;
	}
	return -1;
    }

    protected double getFinalValue_deprecated(NonSeqSimStmt nsss,
					      NSSMVertexResult vr) {
	double result=1.0;
	int[] buckets = new int[nsss.subgraph2.vertices.size()];
	for (int i=0; i<vr.bestMatch.length; i++) {
	    buckets[vr.bestMatch[i]]++;
	}
	double nonSeqoverlapPenaltyBase=1.6;
	double doubleMatchPenaltyBase=1.5;
	double bonusValue = 0.9;
	double k=Math.pow(2,vr.bestMatch.length);

	for (int i=0; i<vr.bestMatch.length; i++) {
	    double val = vr.meanSimilarities[i][vr.bestMatch[i]];

	    if (val==0.0 && buckets[i]==1) val=bonusValue;
	    else { 
		val=val+1;
		// maybe the if-sentence should be outside elsepart?
		if (buckets[i]>1) val=Math.pow(doubleMatchPenaltyBase,Math.pow(buckets[i],val));
	    }
	    
	    result=result*val;
	}
	result = result/(k-(2*(k-1)/Math.pow(Math.sqrt(2),nsss.subgraph1.size())));
	return result;
    }

    protected double getFinalValue(NonSeqSimStmt nsss,
				   NSSMVertexResult vr) {
	int maxsize=vr.bestMatch.length;
	int othersize=vr.meanSimilarities[0].length;
	    
	int[] buckets = new int[othersize];
	for (int i=0; i<maxsize; i++) {
	    buckets[vr.bestMatch[i]]++;
	}
	double multipleMatchPenaltyBase=1.5;

	int men=maxsize;
	int women=othersize;
	int happyWomen=0; // chosen by exactly one man
	// invariant: #happyWomen=#happyMen
	int womenInDoubt=0; // chosen by more than one man 
	int angryWomen=0; // not chosen by any men

	for (int i=0; i<othersize; i++) {
	    if (buckets[i]>1) womenInDoubt++;
	    if (buckets[i]==1) happyWomen++;
	    if (buckets[i]==0) angryWomen++;
	}

	double happyPunish=3;
	double unhappyPunish=6;

	int perfectMatches=0;
	double result=1.0;
	for (int i=0; i<maxsize; i++) {
	    double val=0.0;
	    int matchIndex=vr.bestMatch[i];
	    val = vr.meanSimilarities[i][matchIndex];

	    if (val==0.0 && buckets[matchIndex]==1) {
		//val=bonusValue;
		perfectMatches++;
	    } else { 
		if (buckets[matchIndex]>1) {
		    //multipleMatchings++;
		    val=unhappyPunish+val;
		} else val=happyPunish+val;
	    }
	    result=result+val;
	}
	result=result/(1+(unhappyPunish+1)*maxsize);

	/*
	// GROUPING
	vr.groupingValue=SimLib.nonSeqGroupingStructure2.nonSeqCompare(nsss);
	// combine mean gamma-value (see description in speciale.tex)
	// with grouping value.
	double grw = 0.2;
	result = (1-grw)*result+grw*vr.groupingValue.view;
	*/

	//double bonusValue = 1.0;
	//result = result * Math.pow(bonusValue,perfectMatches);
	double phi=40;
	result=result*sizemod3(result,phi,nsss.size());

	return result;
    }

    private double sizemod3(double w, double phi, int n) {
	return w+(1-w)*phi/(n+phi-1);
    }
}
