/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** This is the result of applying an NSSMVertex measure to a
    non-sequential similarity statement. */
public class NSSMVertexResult extends SimMeasureResult {

    /** The similarity matrix with the mean value of all vertex
        viewpoints used. */
    public double[][] meanSimilarities;
    /** The best match found. */
    public int[] bestMatch=null;
    public int multipleMatchingsInBest=-1;
    /** A Vector of MusicEdge of type MATCH, each of which points to
        two vertices that have been matched. */
    public Vector allMatches=null;
    ViewDifference groupingValue=null;

    public final int doubleStringLength=4;

    public NSSMVertexResult(double meas, NSSMVertex or, 
			    double[][] meansims, int[] bmatch,
			    int doublematchings,
			    Vector allMatch) {
	super(meas,or);
	meanSimilarities=meansims;
	bestMatch=bmatch;
	multipleMatchingsInBest = doublematchings;
	allMatches = allMatch;
    }
    public NSSMVertexResult( NSSMVertex or) {
	super(-1.0,or);
    }

    /** Overrides this method because we don't want to see all the
      viewdiffs. */
    public String toStringWithViewDifferences() {
	return toString()+"\n";
    }

    /** Computes the mean similarities matrix. */
    public double[][] meanVertexSimMatrices() {
	NSVDVertex vd = (NSVDVertex) viewDifferences.get(0);
	double[][] vertexSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
	
	for (int i=0; i<viewDifferences.size(); i++) {
	    vd = (NSVDVertex) viewDifferences.get(i);
	    vertexSimilarities = FuncLib.addMatrices(vertexSimilarities,vd.similarities);  
	}
	return FuncLib.divideMatrixBy(vertexSimilarities,viewDifferences.size());
    }

    /** Computes the number of multiple matchings and the vector allMatches */
    public void completeMatchData(NonSeqSimStmt nsss) {
	multipleMatchingsInBest = FuncLib.countCopies(bestMatch,true);

	allMatches = new Vector();
	for (int i=0; i<bestMatch.length; i++) {
	    Vector fromAnch= new Vector();
	    Vector toAnch = new Vector();
	    fromAnch.add(new Integer(0));
	    toAnch.add(new Integer(0));
	    allMatches.add(new MusicEdge((MusicVertex)nsss.subgraph1.vertices.get(i),
					 (MusicVertex)nsss.subgraph2.vertices.get(bestMatch[i]),
					 MusicEdge.MATCH,
					 fromAnch,toAnch) );
	}
    }

    public String toString() {
	String result="";
	result += "   "+origin.name+"="+measure+"\n";
	if (groupingValue!=null) {
	    result += "   grouping="+groupingValue.view;
	}
	result += "   Similarities matrix=\n"+meanSimilaritiesToString();
	result += "   Best match=\n"+bestMatchToString();
	result += "   Matched notes=\n"+allMatchesToString();
	result += "   Multiple matchings in best="+multipleMatchingsInBest+"\n";
	// result += allViewsToString();
	return result;
    }

    private String allViewsToString() {
	String result ="";
	for (int i=0; i<viewDifferences.size(); i++) {
	    NSVDVertex vd = (NSVDVertex) viewDifferences.get(i);
	    result+= "View "+vd.origin.name+"\n";
	    result+= matrixToString(vd.similarities)+"\n";  
	}
	return result;
    }

    private String meanSimilaritiesToString() {
	String result="";
	for (int i=0; i</*subgraph1.size()*/ meanSimilarities.length; i++) {
	    result+="\t";
	    for (int j=0; j</*subgraph2.size()*/meanSimilarities[0].length; j++) {
		result+=FuncLib.doubleToStringOfLength(meanSimilarities[i][j],
						       doubleStringLength);
	    }
	    result+="\n";
	}
	return result;
    }
    private String matrixToString(double[][] m) {
	String result="";
	for (int i=0; i<m.length; i++) {
	    result+="\t";
	    for (int j=0; j<m[0].length; j++) {
		result+=FuncLib.doubleToStringOfLength(m[i][j],
						       doubleStringLength);
	    }
	    result+="\n";
	}
	return result;
    }
    private String bestMatchToString() {
	String result="";
	for (int i=0; i<bestMatch.length; i++) {
	    result+="\tbestMatch["+i+"]="
		+bestMatch[i]+ "("+FuncLib.doubleToStringOfLength(meanSimilarities[i][bestMatch[i]],doubleStringLength)+")\n";
	}
	return result;
    }

    private String allMatchesToString() {
	String result="";
	if (allMatches!=null) {
	    for (int i=0; i<allMatches.size(); i++) {
		MusicEdge me = (MusicEdge) allMatches.get(i);
		result+= 
		    "  ("+me.from.toString()+")="+
		    "("+me.to.toString()+")\n";
	    }
	}
	return result;
    }

}
