/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;
import java.util.*;

/** NSVCBagIR is extended and instantiated in {@link SimLib}. */
public abstract class NSVCBagIR extends NonSeqViewComparator {
    
    int bag1size=-1;
    int bag2size=-1;
    int bagLength=-1;
    int matchSize=-1;
    
    String viewType;
    Trie trieFromLastSearch=null;
    Vector bag1FromLastSearch=null;
    
    SimFinder sfLast=null;//the sf in last evaluation
    SimFinder sfNow=null;

    public NSVCBagIR(String s, String type) {
	super(s);
	viewType=type;
    }
    
    public double bagIntersectionRatio(int size, NonSeqSimStmt ns){
	sfLast=sfNow;
	sfNow=ns.myOwner;
	boolean sameSubgraph1AsLastTime=false;
	//s behver trien for subgraph1 kun at blive regnet ud een gang
	//da der ikke er ndringer 
	if(sfNow.occurrenceSearch() && sfLast==sfNow){//samme sf som forrige evaluering
	    sameSubgraph1AsLastTime=true;
	}

	//split in sequential subgraphs
	Vector bag1;
	Vector bag2;

	if(sameSubgraph1AsLastTime){
	    bag1=bag1FromLastSearch;
	}else{
	    bag1=ns.subgraph1.getAllSequentialSubgraphsOfSize(size);//of simple and Comp's
	    bag1FromLastSearch=bag1;
	}
	bag2=ns.subgraph2.getAllSequentialSubgraphsOfSize(size);

	bag1size=bag1.size();
	bag2size=bag2.size();
		
	//insert encoding of each seq. in subgraph1 in a Trie
	if(bag1size==0 || bag2size==0){
	    bag1size=-1;
	    bag2size=-1;
	    bagLength=-1;
	    matchSize=-1;
	    return -1.0;   
	}else{	
	    Trie trie;
	    if(sameSubgraph1AsLastTime){
		trie=trieFromLastSearch;
	    }else{
		trie=fillTrie(bag1);
		trieFromLastSearch=trie;
	    }
	    matchSize=testMembership(trie,bag2);
	    return calculateRating(bag1size,bag2size,matchSize);
	}
    }

    public double calculateRating(int bag1siz, int bag2siz, int matchSiz){
	double rat=1.0-(0.5*(
			     ((double)matchSiz/(double)bag1siz)+
			     ((double)matchSiz/(double)bag2siz)
			     )
			);
	if(bag1siz==0||bag2siz==0) {
	    rat=-1.0;
	}
	return rat;
    }
    
    // bag1 of SeqSubgraphs
    public Trie fillTrie(Vector bag1){
	Trie trie=new Trie();
	for(int i=0;i<bag1.size();i++){
	    String encoding=null;
	    SeqSubgraph gr = (SeqSubgraph)bag1.get(i);
//  	    if(viewType=="ap"){
//  		encoding = encodeAbs(gr,"ap");
	    if(viewType=="dap"){
		encoding = encodeDAbs(gr,"dap");
	    }else if(viewType=="pi" || viewType=="di" || viewType=="dfi"){
		    encoding = encodeDelta(gr,viewType);//er defineret i NSVCBagIR
	    }else{
		    System.out.println("BagIR: unknown type");
		    System.exit(0);
	    }
	    trie.insert(encoding);
	}
	return trie;
    }
    
    
    // test membership rating of subgraph2
    // bag 2 of SeqSubgraphs
    public int testMembership(Trie trie, Vector bag2){
	if(trie==null){System.out.println("the trie is null");System.exit(0);}
	int members=0;
	int notMembers=0;
	    for(int i=0;i<bag2.size();i++){
		String encoding=null;
		SeqSubgraph gr = (SeqSubgraph)bag2.get(i);
//  		if(viewType=="ap"){
//  		    encoding = encodeAbs(gr,"ap");
		if(viewType=="dap"){
		    encoding = encodeDAbs(gr,"dap");
		}else if(viewType=="pi" || viewType=="di" || viewType=="dfi"){
		    encoding = encodeDelta(gr,viewType);//er defineret i NSVCBagIR
		}else{
		    System.out.println("BagIR: unknown type");
		    System.exit(0);
		}
		if(trie.member(encoding)){//member tller forekomsten en ned hvis den findes
		    members++;
		}else{
		    notMembers++;
		}
	    }
	    return members;
    }

    //er beregnet til at blive kaldt med typerne {dap,pi,di,dfi}
    public double bagIntersectionRatioForSimEdges(NonSeqSimStmt ns, String type){
	Trie trie=new Trie();
	int simEdges1=0;
	int simEdges2=0;
	for(int i=0;i<ns.subgraph1.edges.size();i++){
	    MusicEdge me = (MusicEdge)ns.subgraph1.edges.get(i);
	    if(me.isSimultaneousEdge()){
		trie.insert(encodeDelta(me,type));
		simEdges1++;
	    }
	}
	int members=0;
	for(int i=0;i<ns.subgraph2.edges.size();i++){
	    MusicEdge me = (MusicEdge)ns.subgraph2.edges.get(i);
	    if(me.isSimultaneousEdge()){
		String s=encodeDelta(me,type);
		simEdges2++;
		if(trie.member(s)) members++;
	    }
	}
	double rating=calculateRating(simEdges1,simEdges2,members);
	return rating;
    }

    public double bagIntersectionRatioForVertices(NonSeqSimStmt ns){
	//split in sequential subgraphs
	
	Vector bag1=(Vector)ns.subgraph1.vertices.clone();//of simple and Comp's
	Vector bag2=(Vector)ns.subgraph2.vertices.clone();

	int verticesBag1size=bag1.size();
	int verticesBag2size=bag2.size();
	int verticesBagLength=-1;	
	int verticesMatchSize=-1;
	
	//insert encoding of each MusicVertex in subgraph1 in a Trie
	if(verticesBag1size==0 || verticesBag2size==0){
	    verticesBag1size=-1;
	    verticesBag2size=-1;
	    verticesBagLength=-1;
	    verticesMatchSize=-1;
	    return 1.0;
	}else{	
	    Trie trie=new Trie();
	    for(int i=0;i<verticesBag1size;i++){
		String encoding=null;
		MusicVertex mv = (MusicVertex)bag1.get(i);
		encoding=encodeMusicVertex(mv);
		trie.insert(encoding);
	    }
	    //test membership rating of subgraph2
	    int members=0;
	    int notMembers=0;
	    for(int i=0;i<verticesBag2size;i++){
		String encoding=null;
		MusicVertex mv = (MusicVertex)bag2.get(i);
		encoding=encodeMusicVertex(mv);
		if(trie.member(encoding)){//member tller forekomsten en ned hvis den findes
		    members++;
		}
	    }
	    matchSize=members;
	    return calculateRating(verticesBag1size,verticesBag2size,members);
	}
    }
    
    public int getSizeOfShortestSeqSubgraph(NonSeqSimStmt nsss){
	return Math.min(nsss.subgraph1.getMaximalSequentialSubgraphSize(),
			nsss.subgraph2.getMaximalSequentialSubgraphSize());
    }
    
    public int decideSize(int shortest){
	int bagLength=2;
	if(shortest==3){
	    bagLength=3;
	}else if(shortest>=4 && shortest<=6){
	    bagLength=3;
	}else if(shortest>6){
	    //bagLength=5;
	    bagLength=shortest-2;
	}
	//		if(shortest>6) bagLength=shortest;
	return bagLength;
    }
    
    // type in {pi, di, dfi}
    private String encodeJump(SimpleVertex first, SimpleVertex last, String type){
	String res="";
	if(type=="pi"){//midi pitch interval
	    res=""+SeqViewComparator.midiJump(first.info,last.info);
	}else if(type=="di"){//diatonic interval
	    res=""+SeqViewComparator.diatonicInterval(first.info,last.info);
	}else if(type=="dfi"){//dfi interval
	    res=""+SeqViewComparator.getDiatonicForwardInterval(first.info.name,last.info.name);
	}else{
	    System.out.println("NSVCBagIR: unknown encoding type");
	    System.exit(0);
	}
	return res;
    }

    // kun fra IRforVertices - alts kun for diaAbsPitch
    private String encodeMusicVertex(MusicVertex mv){
	String res="";
	if(mv instanceof SimpleVertex){
	    SimpleVertex sv=(SimpleVertex)mv;
	    res=""+sv.info.name+"_"+sv.info.octave+"_"+
		sv.info.sharps+"_"+sv.info.flats+"_"+
		sv.info.theNote.getRhythmValue()+";";
	}else if(mv instanceof CompoundVertex){
	    CompoundVertex cv=(CompoundVertex)mv;
	    res=encodeCompound(cv)+";";
	}
	
	return res;
    }

    // fra fillTrie
    // type in {dap} abs pitch
    private String encodeSimpleAbs(SimpleVertex sv, String type){
	String res = "";
	if(type=="dap"){
	    res=""+sv.info.name+"_"+sv.info.octave+"_"+
		sv.info.sharps+"_"+sv.info.flats;
		//   res=""+sv.info.theNote.getPitch();
	}else{
	    System.out.println("NSVCBagIR: unknown encoding type");
	    System.exit(0);
	}
	return res;
    }

    /* returns only the rhythm value */
    private String encodeSimpleDelta(SimpleVertex sv){
	return ""+sv.info.theNote.getRhythmValue();
    }

    /* the encoding is a bit long... => slow */
    private String encodeCompound(CompoundVertex cv){
	return cv.productionRule.nt.name+"_"+cv.productionRule.measure.name;
    }

    private String encodeDelta(SeqSubgraph ss, String type){
	StringBuffer sb = new StringBuffer();
	for(int i=0;i<ss.edges.size();i++){
	    MusicVertex mv1=((MusicEdge)ss.edges.get(i)).from;
	    MusicVertex mv2=((MusicEdge)ss.edges.get(i)).to;
	    //enc vertex 1
	    if(mv1 instanceof SimpleVertex){
		sb.append(encodeSimpleDelta((SimpleVertex)mv1));
	    }else{
		sb.append(encodeCompound((CompoundVertex)mv1));
	    }
	    sb.append("_");
	    //enc jump
	    sb.append(encodeJump(
				 ((MusicEdge)ss.edges.get(i)).fromAnchor(),
				 ((MusicEdge)ss.edges.get(i)).toAnchor(), 
				 type));
	    sb.append("_");
	    //enc vertex 2
	    if(mv2 instanceof SimpleVertex){
		sb.append(encodeSimpleDelta((SimpleVertex)mv2));
	    }else{
		sb.append(encodeCompound((CompoundVertex)mv2));
	    }
	    sb.append(";");
	}
	return sb.toString();
    }

    // encode music edge
    private String encodeDelta(MusicEdge me, String type){
	String res=null;
	if(type=="dap"){
	    res=encodeMESimAP(me);
	}else{
	    StringBuffer sb = new StringBuffer();
	    MusicVertex mv1=me.from;
	    MusicVertex mv2=me.to;
	    //enc vertex 1
	    if(mv1 instanceof SimpleVertex){
		sb.append(encodeSimpleDelta((SimpleVertex)mv1));
	    }else{
		sb.append(encodeCompound((CompoundVertex)mv1));
	    }
	    sb.append("_");
	    //enc jump
	    sb.append(encodeJump(me.fromAnchor(),me.toAnchor(),type));
	    sb.append("_");
	    //enc vertex 2
	    if(mv2 instanceof SimpleVertex){
		sb.append(encodeSimpleDelta((SimpleVertex)mv2));
	    }else{
		sb.append(encodeCompound((CompoundVertex)mv2));
	    }
	    sb.append(";");
	    res=sb.toString();
	}
	return res;
    }

    private String encodeMESimAP(MusicEdge me){
	MusicVertex mv1=me.from;
	MusicVertex mv2=me.to;
	return ""+encodeMusicVertex(mv1)+""+encodeMusicVertex(mv2);
    }
    
    //kaldes fra fillTrie
    private String encodeDAbs(SeqSubgraph ss, String type){
	StringBuffer sb = new StringBuffer();
	for(int i=0;i<ss.vertices.size();i++){
	    MusicVertex mv=(MusicVertex)ss.vertices.get(i);
	    sb.append(encodeMusicVertex(mv));
	    sb.append(";");
	}
	return sb.toString();
    }
    
//      /* not Compound enabled - compares the nested vertices again */
//      private String seqSubgraphToJumpsAndRhythmString(SeqSubgraph mg){
//  	NoteInfo[] info = mg.getNoteInfoArray();
//  	int[] name=mg.getNameArray();
//  	int[] octaves=mg.getOctaveArray();
//  	/* *********** Change here for another kind of jump-value ****************** */
	
//  	//int[] jumps=SeqViewComparator.getDiatonicForwardArray(name);
//  	int[] jumps=SeqViewComparator.getPitchIntervalArray(info);
	

//  	//int[] jumps=SeqViewComparator.getDiatonicAbsIntervalArray(name,octaves);
//  	double[] durations=getRhythmValueArrayExcludeLast(info);
//  	if(jumps.length!=durations.length){
//  	    System.out.println("no! in NSVCBagIR");
//  	    System.exit(0);
//  	}
//  	return jumpAndRhythmarraysToString(jumps,durations);
//      }

//      private String jumpAndRhythmarraysToString(int[] jmps, double[] rvs){
//  	StringBuffer sb = new StringBuffer();
//  	for(int i=0;i<jmps.length;i++){
//  	    sb.append(""+jmps[i]+","+rvs[i]+";");
//  	}	
//  	String res = sb.toString();
//  	//System.out.println("input: "+res);
//  	return res;
//      }
    
//      //this method excludes the last NoteInfo's rhythmValue
//      private double[] getRhythmValueArrayExcludeLast(NoteInfo[] ni){
//  	double[] res=new double[ni.length-1];
//  	for(int i=0;i<res.length;i++){
//  	    res[i]=ni[i].theNote.getRhythmValue();
//  	}
//  	return res;
//      }
}
