/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** NSVCEdge is extended and instantiated in {@link SimLibEdge}.*/
public abstract class NSVCEdge extends NonSeqViewComparator {

    public NSVCEdge(String s) {
	super(s);
    }
    
    protected abstract double compareEdges(MusicEdge mv1,
					   MusicEdge mv2,
					   NonSeqSimStmt nsss);

    protected double[][] getEdgeSimMatrix(NonSeqSimStmt nsss) {
	//return getAllEdgeSimMatrix(nsss);
	return getFollowEdgeSimMatrix(nsss);
    }

    protected double[][] getAllEdgeSimMatrix(NonSeqSimStmt nsss) {
	Vector edg1 = nsss.subgraph1.edges;
	Vector edg2 = nsss.subgraph2.edges;

	//Vector edg1 = nsss.subgraph1.getFollowEdges();
	//Vector edg2 = nsss.subgraph2.getFollowEdges();

	double[][] esim = new double[edg1.size()][edg2.size()];

	for (int i=0; i<edg1.size(); i++) {
	    MusicEdge me1 = (MusicEdge) edg1.get(i);
	    for (int j=0; j<edg2.size(); j++) {
		MusicEdge me2 = (MusicEdge) edg2.get(j);
		esim[i][j]=compareEdges(me1,me2,nsss);
	    }
	    // System.out.print("."); //tmp
	}
	return esim;
    }

    protected double[][] getFollowEdgeSimMatrix(NonSeqSimStmt nsss) {
	/* for all vertices in graph 1, compare with all vertices in
           graph 2 */
	Vector fol1 = nsss.subgraph1.getFollowEdges();
	Vector fol2 = nsss.subgraph2.getFollowEdges();

	double[][] fesim = new double[fol1.size()][fol2.size()];

	for (int i=0; i<fol1.size(); i++) {
	    MusicEdge me1 = (MusicEdge) fol1.get(i);
	    for (int j=0; j<fol2.size(); j++) {
		MusicEdge me2 = (MusicEdge) fol2.get(j);
		fesim[i][j]=compareEdges(me1,me2,nsss);
	    }
	    // System.out.print("."); //tmp
	}
	return fesim;
    }

    protected double[][] getSimultEdgeSimMatrix(NonSeqSimStmt nsss) {
	/* for all vertices in graph 1, compare with all vertices in
           graph 2 */
	Vector simedges1 = nsss.subgraph1.getSimultaneousEdges();
	Vector simedges2 = nsss.subgraph2.getSimultaneousEdges();

	double[][] sesim = new double[simedges1.size()][simedges2.size()];

	for (int i=0; i<simedges1.size(); i++) {
	    MusicEdge me1 = (MusicEdge) simedges1.get(i);
	    for (int j=0; j<simedges2.size(); j++) {
		MusicEdge me2 = (MusicEdge) simedges2.get(j);
		sesim[i][j]=compareEdges(me1,me2,nsss);
	    }
	    // System.out.print("."); //tmp
	}
	return sesim;
    }
  
}
