/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;

/** A non-sequential similarity measure is used to evaluate
    non-sequential similarity statements ({@link SeqSimStmt}). */
public abstract class NonSeqSimMeasure extends SimilarityMeasure {

    public NonSeqSimMeasure(String s) {
	super(s);
    }
    public NonSeqSimMeasure(String s, Vector nsvps /* of NonSeqViewComparator */) {
	super(s,nsvps);
    }

    public void evaluate(SimilarityStatement ss) {
	if (ss instanceof NonSeqSimStmt) {
	    evaluateNonSeq((NonSeqSimStmt)ss);
	} else {
	    System.out.println("warning: returning without evaluating from NonSeqSimMeasure.evaluate()");
	}
    }

    public abstract void evaluateNonSeq(NonSeqSimStmt nsss);

    protected Vector evaluateAllViewComparators(NonSeqSimStmt nsss) {
	Vector viewdiffs = new Vector(); // of NonSeqViewDifference
	for (int i=0; i<viewComparators.size(); i++) {
	    NonSeqViewComparator vp= (NonSeqViewComparator) viewComparators.get(i);
	    //System.out.println("Evaluating for comparator: "+vp.name);
	    viewdiffs.add(vp.compare(nsss));
	}
	return viewdiffs;
    }

    /* ---------- Helper functions ---------- */

    /** DON'T USE THIS method, use version2 of it below.  Constructs a
       vector of all possible combinations, which is very big and will
       make the program crash with an OutOfMemoryError. */
    protected int[] findBestCombination(Vector[] ibest /* of Integer */) {
	int[] result=null;
	Vector allComb = findAllCombinations(ibest,0);

	int bestCombValue=Integer.MAX_VALUE;

	for (int i=0; i<allComb.size(); i++) {
	    Vector somecomb = (Vector) allComb.get(i);
	    int[] comb = new int[somecomb.size()];
	    for (int j=0; j<somecomb.size(); j++) {
		comb[j]=((Integer)somecomb.get(j)).intValue();
	    }
	    int combValue = FuncLib.countCopies(comb,true);
	    if (combValue<bestCombValue) {
		bestCombValue=combValue;
		result=comb;
	    }
	}
	return result;
    }

    // debug
    NonSeqSimStmt nsdebug=null;

    /** Instead of constructing a list of all combinations, this
       method enumerates them on the fly. Uses less memory. */
    protected int[] findBestCombination2(Vector[] ibest /* of Integer */) {
	int[] result=null;

	int maxSecondsForComparison=60;
	long startTime=(new Date()).getTime();

	CombEnumerator ce= new CombEnumerator(ibest);
	int bestCombValue=Integer.MAX_VALUE;
	int[] comb = ce.getNext();

	boolean printStatus=true;
	int secs=10;
	while (comb!=null) {
	    int combValue = FuncLib.countCopies(comb,true);
	    if (combValue<bestCombValue) {
		bestCombValue=combValue;
		result=comb;
	    }
	    // System.out.print(".");//tmp
	    comb=ce.getNext();

	    long nowTime=(new Date()).getTime();
	    if ((nowTime-startTime)>secs*1000) {
		int countCombinations=1;
		System.out.println("findBestCombination 2 has been working >"+secs+" seconds on:");
		if ((nowTime-startTime)>maxSecondsForComparison*1000) {
		    System.out.println("NonSeqSimMeasure.findBestCombination2: maximum number of seconds for finding a combination has been passed, dropping out.");
		    // return the best found until now, or just some dummy value
		    if (result==null) result=new int[ibest.length];
		    return result;
		}
		for (int i=0; i<ibest.length; i++) {
		    countCombinations=countCombinations*ibest[i].size();
		    for (int j=0; j<ibest[i].size(); j++) {
			System.out.print(""+((Integer)ibest[i].get(j)).intValue()+", ");
		    }
		    System.out.println();
		}
		System.out.println("Combinations: "+countCombinations);
		secs+=10;

		/*		GraphWindow gw1 = new GraphWindow(nsdebug.subgraph1.getMotherGraph(),600,100,false);
		gw1.setAndShowHighlight(nsdebug.subgraph1);
		GraphWindow gw2 = new GraphWindow(nsdebug.subgraph2.getMotherGraph(),600,600,false);
		gw2.setAndShowHighlight(nsdebug.subgraph2);
		System.out.println("sub1=\n"+nsdebug.subgraph1.toString());
		System.out.println("sub2=\n"+nsdebug.subgraph2.toString());
		
		try { Thread.sleep(1800000); } catch(Exception e){System.out.println("Exception while trying to sleep in findBestCombination2: "+e.getMessage());}*/
	    }

	}
	return result;
    }

    /** Returns a Vector of Vector of Integer */
    protected Vector findAllCombinations(Vector[] ibest, int from) {
	Vector allComb=new Vector();;
	if (from+1<ibest.length) {
	    // recursive call
	    Vector allPostfixes = findAllCombinations(ibest,from+1);
	    // add all combinations
	    for (int i=0; i<ibest[from].size(); i++) {
		Integer someInteger = (Integer) ibest[from].get(i);
		for (int j=0; j<allPostfixes.size(); j++) {
		    Vector postfix = (Vector) allPostfixes.get(j);
		    Vector pfixClone = (Vector) postfix.clone();
		    pfixClone.insertElementAt(someInteger,0);
		    allComb.add(pfixClone);
		}
	    }
	} else {
	    allComb = new Vector();
	    for (int i=0; i<ibest[from].size(); i++) {
		Vector comb= new Vector();
		comb.add(ibest[from].get(i));
		allComb.add(comb);
	    }
	}
	return allComb;
    }

}
