/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import jm.music.data.*;

/** An instance of this class contains information on one note or
     rest.  */
public class NoteInfo {

    public Part thePart;
    public Phrase thePhrase;
    public Note theNote;
    public int noteNumberInPhrase;

    /* the only reason to keep variable starttime here (now that we
     * have moved it to class MusicVertex) is that the constructor in
     * MotherGraph uses NoteInfo objects to store all info before
     * vertices are constructed. If the constructor is changed, this
     * variable can be removed. */
    public double starttime;

    //musedata attributter
    public static final int[] NATURAL_SCALE = {0,2,4,5,7,9,11};

    //name skal muligvis vre en char
    /** C=0, D=1, E=2, F=3, G=4, A=5, B=6, -1 is a rest
     */
    public int name=-10;//c=0,d=1,e=2,f=3,g=4,a=5,h=6, -1 is a rest
    // if name is set to -10, it signals that there is no musedata in this noteinfo

    /** The octave is represented as an integer (0..8),  -1 is a rest.
     */
    public int octave;//0..8, -1 is for a rest
    public int sharps;
    public int flats;
    public int md_lineNumber;
    public int md_part;
    public boolean isTied;
    public int tiedToTrackNumber;
    
    /** An integer telling the number of fixed accidentals notated - A
	negative number means flats, and a positive sharps. */
    public int key;
    public int timeSigUpper;
    public int timeSigLower;
    public boolean fermata=false;

    /** Constructs a NoteInfo object from MIDI information
     */
    public NoteInfo(Part p, Phrase f, Note n, int notenum, double start) {
	thePart=p;
	thePhrase=f;
	theNote=n;
	noteNumberInPhrase=notenum;
	theNote.setDuration(theNote.getRhythmValue());
	starttime=start;
    }

    /** Constructs a NoteInfo object from MuseData information. 
     */
    public NoteInfo(int md_prt, Part dummy, int md_name, int md_octave,
		    int md_sharps, int md_flats, 
		    double md_rhythmVal, double md_starttime,
		    int md_key, int md_timeSigUp, int md_timeSigLow,
		    boolean md_isTied, int md_tiedToTrackNumber, boolean fermat,
		    int md_line){
	
	md_part=md_prt;
	thePart = dummy;
	name=md_name;
	octave=md_octave;
	sharps=md_sharps;
	flats=md_flats;
	
	if(md_octave==-1){
	    theNote=new Note(Integer.MIN_VALUE,md_rhythmVal);
	}else{//calculate the midi pitch
	    theNote=new Note(12*(octave+1)+
			     NATURAL_SCALE[name]+
			     sharps-flats,md_rhythmVal);
	}
	
	starttime=md_starttime;
	key=md_key;
	timeSigUpper=md_timeSigUp;
	timeSigLower=md_timeSigLow;
	isTied=md_isTied;
	tiedToTrackNumber=md_tiedToTrackNumber;
	fermata=fermat;
	md_lineNumber=md_line;
	theNote.setDuration(theNote.getRhythmValue());
    }
    
    public boolean isRest(){
	return theNote.getPitch()==Integer.MIN_VALUE;
    }

    public boolean isNote(){
	return theNote.getPitch()!=Integer.MIN_VALUE;
    }

    public double endTime(){
	return starttime+theNote.getRhythmValue();
    }

    public boolean diatonicAbsolutePitchEquals(NoteInfo other){
	return (
		octave == other.octave &&
		name == other.name &&
		flats == other.flats &&
		sharps == other.sharps
		);
	
    }

    public String toString(){
	return "name:"+name+" #:"+sharps+" b:"+flats+" oct:"+octave+
	    " rhVal:"+theNote.getRhythmValue()+" starttime:"+starttime+
	    " line:"+md_lineNumber+" part:"+md_part;

    }
    
    public Object clone(){
	NoteInfo copy=new NoteInfo(this.thePart,this.thePhrase,this.theNote,
				   this.noteNumberInPhrase,this.starttime);
	copy.name=this.name;
	copy.octave = this.octave;
	copy.sharps=this.sharps;
	copy.flats=this.flats;
	copy.md_lineNumber=this.md_lineNumber;
	copy.md_part=this.md_part;
	copy.isTied=this.isTied;
	copy.tiedToTrackNumber=this.tiedToTrackNumber;
	copy.key=this.key;
	copy.timeSigUpper=this.timeSigUpper;
	copy.timeSigLower=this.timeSigLower;
	copy.fermata=this.fermata;
	return copy;
    }
    
    public boolean hasMusedata() {
	return (name!=-10);
    }
    

}
