/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;
import java.awt.*;

/** RunSelfSim is a program which is used to calculate and show a
    matrix of self-similarities of an entire music piece according to
    some vertex-based similarity measure. E.g. the similarity matrix
    shown on p.101 of our thesis was created using RunSelfSim. */
public class RunSelfSim {
    /** RunSelfSim should be run with one command line parameter
        specifying a midi- or MuseData-file to be analysed. */
    public static void main(String[] args) {
	if (args.length<1) {
	    System.out.println("Usage: java RunSelfSim <midi/md-filename>");
	    System.exit(0);
	}
	MotherGraph mother = getGraph(args[0]);
	NonSeqSubgraph ns1 = new NonSeqSubgraph("Subgraph("+mother.title+")", mother);
	NonSeqSubgraph ns2 = new NonSeqSubgraph("Subgraph("+mother.title+")", mother);

	ns1.vertices = (Vector) mother.vertices.clone();
	ns1.edges = (Vector) mother.edges.clone();
	ns2.vertices = (Vector) mother.vertices.clone();
	ns2.edges = (Vector) mother.edges.clone();

	/*-------------------------------------------------------------*/
	/* Change the used SimilarityMeasure HERE! */
	NonSeqSimStmt sim = new NonSeqSimStmt(ns1,ns2,SimLibVertex.nonSeqVertexPLIOS,null);
	/*-------------------------------------------------------------*/

	System.out.println("Calculating value:");
	double dummy = sim.value();
	System.out.println("DONE calculating value.");

	SimPicture sp = new SimPicture((NSSMVertexResult) sim.smResult);
    }

    private static void similaritiesToString(NSSMVertexResult res) {
	for (int i=0; i</*subgraph1.size()*/ res.meanSimilarities.length; i++) {
	    System.out.println(" ");
	    for (int j=0; j</*subgraph2.size()*/res.meanSimilarities[0].length; j++) {
		System.out.print(FuncLib.doubleToStringOfLength(res.meanSimilarities[i][j],
								res.doubleStringLength));
	    }
	    System.out.println();
	}
    }

    private static MotherGraph getGraph(String filename) {
	MotherGraph mom=null;
	if (filename.substring(filename.length()-3,filename.length()).equals(".md") ||
	filename.substring(filename.length()-4,filename.length()).equals(".md2"    )) {
	    // musedata constructor
	    mom = new MotherGraph(filename,
				  true /*includeInterPartEdges*/,
				  true /*combineRests*/,
				  true /*combineTies*/);
	}
	else {
	    // midi constructor
	    mom = new MotherGraph(filename,true);
	}
	return mom;
    }

    private static class SimPicture extends Frame {
	Canvas canvas;
	NSSMVertexResult vcr;
	private final int blowup = 2;
	private final float threshold = 0.01f;

	public SimPicture(NSSMVertexResult res) {
	    vcr=res;
	    canvas = new Canvas() 
		{ 
		    public void paint(Graphics g) {
			super.paint(g);
			drawSim(g);
		    }
		};
	    canvas.setSize(blowup*vcr.meanSimilarities.length,
			   blowup*vcr.meanSimilarities[0].length);
	    add(canvas,BorderLayout.CENTER);
	    setLocation(350,100);
	    pack();
	    show();
	}

	public void drawSim(Graphics g) {
	    for (int i=0; i<vcr.meanSimilarities.length; i++) {
		for (int j=0; j<vcr.meanSimilarities[0].length; j++) {
		    float f = (float) (1-vcr.meanSimilarities[i][j]);
		    g.setColor(new Color(f,f,f));
		    if (f>1-threshold)
			g.setColor(new Color(1.0f,0.0f,0.0f));
		    g.fillRect(i*blowup,j*blowup,blowup,blowup);
		}
	    }
	}


    }

}
