/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** A sequential similarity measure is used to evaluate sequential
    similarity statements ({@link SeqSimStmt}). */
public abstract class SeqSimMeasure extends SimilarityMeasure {

    public SeqSimMeasure(String s) { super(s); }

    /* shouldn't be declared here, but they must be for testing */ 
    double bval=0.9;
    double phi=40;
    double mew=0.0000001;
    /* --- */
	    
    /** Translates to the use of evaluateSeq() by casting the
        similarity statement to a sequential similarity statement. */
    public void evaluate(SimilarityStatement ss) {
	if (ss instanceof SeqSimStmt) {
	    evaluateSeq((SeqSimStmt)ss);
	} else {
	    System.out.println("warning: returning without evaluating from SeqSimMeasure.evaluate()");
	}
    }

    /** Makes a sequential evaluation. */
    public abstract void evaluateSeq(SeqSimStmt sss);

    /** This method automates the evaluation of all view comparators. */
    protected Vector evaluateAllViewComparators(SeqSimStmt sss) {
	// System.out.println("viewComparators.size="+viewComparators.size());
	//	System.out.println("evalAllVP");
	Vector viewdiffs = new Vector(); // of SeqViewDifference
	for (int i=0; i<viewComparators.size(); i++) {
	    SeqViewComparator vp= (SeqViewComparator) viewComparators.get(i);
	    // 	    if (viewdiffs==null) System.out.println("viewdiffs null");
	    //if (vp==null) System.out.println("vp null");
	    viewdiffs.add(vp.compare(sss));
	}
	return viewdiffs;
    }

}
