/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Random;

/** A sequential similarity statement is a statement about the
       similarity of two sequential subgraphs. */
public class SeqSimStmt extends SimilarityStatement {

    public SeqSimStmt(SeqSubgraph s1, SeqSubgraph s2, SeqSimMeasure ssm, SimFinder sf) {
	super(s1,s2,sf);
	simMeasure=ssm;
    }

    public String toString() {
	String result = "Seq"+super.toString();
	return result;
    }

    public String toStringWithSubgraphs() {
	String result = super.toStringWithSubgraphs();
	return result;
    }

    /** Mutates this similarity statement by either substituting one
        subgraph with an entirely new randomly built sequential
        subgraph, or extending, shortening or sliding both
        subgraphs.*/
    public void mutate(double mutrate) throws Exception {

	Subgraph temp=subgraph1;
	// chance of substituting one subgraph by an entirely new one
	double freshBloodChance = myOwner.freshBloodChance; 

	int times = 1; // (int)(10*mutrate); //1
	int muttimes =1;
	for (int i=0; i<times; i++) {

	    int mutation = rand.nextInt(3);
	    if (myOwner.maxSubSize>0) {
		if (myOwner.maxSubSize<Math.max(subgraph1.size(),subgraph2.size())) {
		    mutation=1;
		} else if (myOwner.maxSubSize==Math.max(subgraph1.size(),subgraph2.size())) {
		    mutation=2;
		}
	    }
	    if(myOwner.occurrenceSearch()) mutation=2;//only slide allowed!

	    if (rand.nextFloat()<freshBloodChance) {
		if (rand.nextFloat()<0.5) {
		    if(!myOwner.occurrenceSearch()){
			subgraph1 = subgraph1.getMotherGraph().getRandomSeqSubgraph(subgraph2.size());
		    }
		} else {
		    subgraph2 = subgraph2.getMotherGraph().getRandomSeqSubgraph(subgraph1.size());
		}
	    } else {
		switch (mutation) {
		case 0: // enlarge
		    subgraph1.extend(muttimes); 
		    subgraph2.extend(muttimes); 
		    break;
		case 1: // shrink
		    if (subgraph1.vertices.size()>=2+muttimes) {
			subgraph1.shorten(muttimes);
			subgraph2.shorten(muttimes);
		    } else { 
			/* we need to enlarge more often than shrink, 
			 * otherwise we run aground on size 2 windows 
			 * with perfect match */
			subgraph1.extend(muttimes); 
			subgraph2.extend(muttimes);
		    }
		    break;
		case 2: // slide
		    if(!myOwner.occurrenceSearch())
			subgraph1.slide(muttimes);
		    subgraph2.slide(muttimes);
		    break;
		default:
		}
	    }
	}
	if(!temp.equals(subgraph1) && myOwner.occurrenceSearch()){
	    System.out.println("!temp.equals(subgraph1)");
	    System.exit(0);
	}
	smResult=null;
    }

    /** Returns a SeqSimStmt with clones of the two subgraphs. */
    public Object clone() {
	SeqSimStmt ss = new SeqSimStmt((SeqSubgraph)subgraph1.clone(), (SeqSubgraph)subgraph2.clone(), (SeqSimMeasure) simMeasure, myOwner);
	ss.smResult=null;
	return ss;
    }

}
