/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;
import java.util.Vector;

/** A class not to be instantiated. Contains static instances of
   anonymous class extensions of SimilarityMeasure and
   ViewComparator. */
public class SimLibDiffCount{

    /* ---------- VIEWCOMPARATORS ---------- */

    /* Compound enabled */ 
    public static final SeqViewComparator diatonicForwardIntervalDiffCount=
	new SeqViewComparator("diatonicForwardIntervalDiffCount"){
		
		public ViewDifference /*getView ? */ seqCompare(SeqSimStmt sss){
		    
		    MusicEdge[] mes1 = ((SeqSubgraph)sss.subgraph1).getMusicEdgeArray();
		    MusicEdge[] mes2 = ((SeqSubgraph)sss.subgraph2).getMusicEdgeArray();
		    
		    double differences=0.0;
		    for(int i=0;i<mes1.length;i++){
			//find de simple knuder inden i compound'erne
			//(skal bruges til at udregne dfi spring)
			SimpleVertex mv1_1 = mes1[i].fromAnchor();
			SimpleVertex mv1_2 = mes1[i].toAnchor();
			SimpleVertex mv2_1 = mes2[i].fromAnchor();
			SimpleVertex mv2_2 = mes2[i].toAnchor();
			
			//find jumps - vi ved endnu ikke noget om MusicVertex typerne
			int jump1=SeqViewComparator.getDiatonicForwardInterval(mv1_1.info.name, mv1_2.info.name);
			int jump2=SeqViewComparator.getDiatonicForwardInterval(mv2_1.info.name, mv2_2.info.name);
			//denne straf tilfalder alle der ikke har samme spring
			if(jump1!=jump2){differences+=1.0;}			    
			
			//find ud af om knuderne i enderne af kanterne er af parvis samme type
			//comp skal matches med comp og simpel med simpel
			MusicVertex from1=mes1[i].from;
			MusicVertex to1=mes1[i].to;
			MusicVertex from2=mes2[i].from;
			MusicVertex to2=mes2[i].to;
			
			/* frst behandles fraknuderne... */
			//comp-comp i fraknuderne
			if(from1 instanceof CompoundVertex && from2 instanceof CompoundVertex) {
			    CompoundVertex from1c=(CompoundVertex)from1;
			    CompoundVertex from2c=(CompoundVertex)from2;
			    //check om forskellig slags compound
			    if(from1c.productionRule.nt!=from2c.productionRule.nt){
				//yderligere straf
				//evaluer evt.  forskellene mellem comp'erne
				/*
				SeqSimStmt diff=
				    new SeqSimStmt((SeqSubgraph)sss.subgraph1, 
						   (SeqSubgraph)sss.subgraph2,
 						   seqDFIAndLength,//SeqSimMeasure
						   new SimFinder(null,0,0,0.0,0.0,0,
								 seqDFIAndLength//SimMeasure
								 ));
				differences+=diff.value();
				*/
				differences+=1.0;
			    }
			}else if(from1 instanceof SimpleVertex && from2 instanceof SimpleVertex) {
			    // simpel-simpel i fraknuderne - godt
			}else{//forsk type =>straf
			    differences+=1.0;
			}

			/* ... og nu til tilknuderne */
			//comp-comp i tilknuderne
			if(to1 instanceof CompoundVertex && to2 instanceof CompoundVertex) {
			    CompoundVertex to1c=(CompoundVertex)to1;
			    CompoundVertex to2c=(CompoundVertex)to2;
			    //check om forskellig slags compound
			    if(to1c.productionRule.nt!=to2c.productionRule.nt){
				//yderligere straf
				//evaluer evt. forskellene mellem comp'erne som ovenfor
				differences+=1.0;
			    }//ellers er de ens! og skal ikke straffes
			}else if(to1 instanceof SimpleVertex && to2 instanceof SimpleVertex) {
			    // simpel-simpel i tilknuderne - godt
			}else{//forsk type =>straf
			    differences+=1.0;
			}
			
		    }
		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    /* Compound enabled */ 
    public static final SeqViewComparator absLengthDiffCount=
	new SeqViewComparator("absLengthDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicVertex[] mvs1 = ((SeqSubgraph)sss.subgraph1).getMusicVertexArray();
		    MusicVertex[] mvs2 = ((SeqSubgraph)sss.subgraph2).getMusicVertexArray();
		    double differences=0.0;
		    for(int i=0;i<mvs1.length;i++){
			//begge simple
			if(mvs1[i] instanceof SimpleVertex && mvs2[i] instanceof SimpleVertex){
			    SimpleVertex first = (SimpleVertex)mvs1[i]; 
			    SimpleVertex next = (SimpleVertex)mvs2[i];
			    if(first.info.theNote.getRhythmValue()!=next.info.theNote.getRhythmValue()){
				differences+=1.0;
			    }//ellers er alt ok
			}else if (mvs1[i] instanceof CompoundVertex && 
				  mvs2[i] instanceof CompoundVertex){
			    //begge Comps
			    CompoundVertex first = (CompoundVertex)mvs1[i]; 
			    CompoundVertex next = (CompoundVertex)mvs2[i];
			    if(first.productionRule.nt!=next.productionRule.nt){
				differences += 1.0;//stor straf
			    }
			}else{//forskellig type
			    differences+=1.0;//stor straf
			}
		    }

		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    /* Compound enabled */ 
    public static final SeqViewComparator lengthIntervalDiffCount=
	new SeqViewComparator("lengthIntervalDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicVertex[] mvs1 = ((SeqSubgraph)sss.subgraph1).getMusicVertexArray();
		    MusicVertex[] mvs2 = ((SeqSubgraph)sss.subgraph2).getMusicVertexArray();
		    double differences=0.0;
		    for(int i=0;i<mvs1.length-1;i++){
			double interval1=mvs1[i+1].getLength()/mvs1[i].getLength();
			double interval2=mvs2[i+1].getLength()/mvs2[i].getLength();
			if (interval1!=interval2) differences+= 1.0;

			//begge simple
			/*			if(mvs1[i] instanceof SimpleVertex && mvs2[i] instanceof SimpleVertex){
			    SimpleVertex first = (SimpleVertex)mvs1[i]; 
			    SimpleVertex next = (SimpleVertex)mvs2[i];
			    if(first.info.theNote.getRhythmValue()!=next.info.theNote.getRhythmValue()){
				differences+=1.0;
			    }//ellers er alt ok
			}else if (mvs1[i] instanceof CompoundVertex && 
				  mvs2[i] instanceof CompoundVertex){
			    //begge Comps
			    CompoundVertex first = (CompoundVertex)mvs1[i]; 
			    CompoundVertex next = (CompoundVertex)mvs2[i];
			    if(first.productionRule.nt!=next.productionRule.nt){
				differences += 1.0;//stor straf
			    }
			}else{//forskellig type
			    differences+=1.0;//stor straf
			    }*/
		    }

		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    /* Compound enabled */ 
    /* Uses midi pitch */
    public static final SeqViewComparator absPitchDiffCount=
	new SeqViewComparator("absPitchDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicVertex[] mvs1 = ((SeqSubgraph)sss.subgraph1).getMusicVertexArray();
		    MusicVertex[] mvs2 = ((SeqSubgraph)sss.subgraph2).getMusicVertexArray();
		    
		    double differences=0.0;
		    if(mvs1.length!=mvs2.length){
			System.out.println("SimLibDiffCount: size-error");
			System.exit(0);
		    }		    
		    for(int i=0;i<mvs1.length;i++){
			//begge simple
			if(mvs1[i] instanceof SimpleVertex && mvs2[i] instanceof SimpleVertex){
			    SimpleVertex first = (SimpleVertex)mvs1[i]; 
			    SimpleVertex next = (SimpleVertex)mvs2[i];
			    //todo
			    /* midi pitch comparation */
			    if(first.info.theNote.getPitch()!=next.info.theNote.getPitch()){
				differences+=1.0;
			    }//ellers er alt ok - ingen straf
			    
			}else if (mvs1[i] instanceof CompoundVertex && 
				  mvs2[i] instanceof CompoundVertex){
			    //begge Comps
			    CompoundVertex first = (CompoundVertex)mvs1[i]; 
			    CompoundVertex next = (CompoundVertex)mvs2[i];
			    if(first.productionRule.nt!=next.productionRule.nt){
				differences += 1.0;//stor straf
			    }
			}else{//forskellig type simple-comp
			    differences+=1.0;//stor straf
			}
		    }
		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };   
    
    /* Compound enabled */ 
    /* Uses diatonic pitch */
    public static final SeqViewComparator diatonicAbsPitchDiffCount=
	new SeqViewComparator("diatonicAbsPitchDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicVertex[] mvs1 = ((SeqSubgraph)sss.subgraph1).getMusicVertexArray();
		    MusicVertex[] mvs2 = ((SeqSubgraph)sss.subgraph2).getMusicVertexArray();
		    
		    double differences=0.0;
		    if(mvs1.length!=mvs2.length){
			System.out.println("SimLibDiffCount: size-error");
			System.exit(0);
		    }		    
		    for(int i=0;i<mvs1.length;i++){
			//begge simple
			if(mvs1[i] instanceof SimpleVertex && mvs2[i] instanceof SimpleVertex){
			    SimpleVertex first = (SimpleVertex)mvs1[i]; 
			    SimpleVertex next = (SimpleVertex)mvs2[i];
			    //todo
			    /* diatonic comparation */
			    
			    if(!first.info.diatonicAbsolutePitchEquals(next.info)){
				differences+=1.0;
			    }//ellers er alt ok - ingen straf
			    
			}else if (mvs1[i] instanceof CompoundVertex && 
				  mvs2[i] instanceof CompoundVertex){
			    //begge Comps
			    CompoundVertex first = (CompoundVertex)mvs1[i]; 
			    CompoundVertex next = (CompoundVertex)mvs2[i];
			    if(first.productionRule.nt!=next.productionRule.nt){
				differences += 1.0;//stor straf
			    }
			}else{//forskellig type simple-comp
			    differences+=1.0;//stor straf
			}
		    }
		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    /* Compound enabled */ //sammenligner p midi pitch forskelle
    public static final SeqViewComparator pitchIntervalDiffCount=
	new SeqViewComparator("pitchIntervalDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicEdge[] mes1 = ((SeqSubgraph)sss.subgraph1).getMusicEdgeArray();
		    MusicEdge[] mes2 = ((SeqSubgraph)sss.subgraph2).getMusicEdgeArray();
		    double differences=0.0;
		    for(int i=0;i<mes1.length;i++){
			//find de simple knuder inden i compound'erne
			//(skal bruges til at udregne spring)
			SimpleVertex mv1_1 = mes1[i].fromAnchor();
			SimpleVertex mv1_2 = mes1[i].toAnchor();
			SimpleVertex mv2_1 = mes2[i].fromAnchor();
			SimpleVertex mv2_2 = mes2[i].toAnchor();
			//find jumps - vi ved endnu ikke noget om MusicVertex typerne
			int jump1=SeqViewComparator.midiJump(mv1_1.info, mv1_2.info);
			int jump2=SeqViewComparator.midiJump(mv2_1.info, mv2_2.info);
			//denne straf tilfalder alle der ikke har samme spring
			if(jump1!=jump2){differences+=1.0;}			    
			//find ud af om knuderne i enderne af kanterne 
			//er af parvis samme type
			//comp skal matches med comp og simpel med simpel
			MusicVertex from1=mes1[i].from;
			MusicVertex to1=mes1[i].to;
			MusicVertex from2=mes2[i].from;
			MusicVertex to2=mes2[i].to;
			/* frst behandles fraknuderne... */
			//comp-comp i fraknuderne
			if(from1 instanceof CompoundVertex && from2 instanceof CompoundVertex) {
			    CompoundVertex from1c=(CompoundVertex)from1;
			    CompoundVertex from2c=(CompoundVertex)from2;
			    //check om forskellig slags compound
			    if(from1c.productionRule.nt!=from2c.productionRule.nt){
				//forsk nonterminal yderligere straf
				//evaluer evt.  forskellene mellem comp'erne
				differences+=1.0;
			    }
			}else if(from1 instanceof SimpleVertex && from2 instanceof SimpleVertex) {
			    // simpel-simpel i fraknuderne - godt
			    // jump-diff er allerede udregnet straffet/belnnet
			}else{//forsk type =>straf
			    differences+=1.0;
			}
			/* ... og nu til tilknuderne */
			//comp-comp i tilknuderne
			if(to1 instanceof CompoundVertex && to2 instanceof CompoundVertex) {
			    CompoundVertex to1c=(CompoundVertex)to1;
			    CompoundVertex to2c=(CompoundVertex)to2;
			    //check om forskellig slags compound
			    if(to1c.productionRule.nt!=to2c.productionRule.nt){
				//yderligere straf
				differences+=1.0;
			    }//ellers er de ens! og skal ikke straffes
			}else if(to1 instanceof SimpleVertex && to2 instanceof SimpleVertex) {
			    // simpel-simpel i tilknuderne - godt
			}else{//forsk type =>straf
			    differences+=1.0;
			}
			
		    }
		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}

	    };
    
    /* Compound enabled */ 
    public static final SeqViewComparator diatonicIntervalDiffCount=
	new SeqViewComparator("diatonicIntervalDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){
		    MusicEdge[] mes1 = ((SeqSubgraph)sss.subgraph1).getMusicEdgeArray();
		    MusicEdge[] mes2 = ((SeqSubgraph)sss.subgraph2).getMusicEdgeArray();
		    double differences=0.0;
		    for(int i=0;i<mes1.length;i++){
			//find de simple knuder inden i compound'erne
			//(skal bruges til at udregne spring)
			SimpleVertex mv1_1 = mes1[i].fromAnchor();
			SimpleVertex mv1_2 = mes1[i].toAnchor();
			SimpleVertex mv2_1 = mes2[i].fromAnchor();
			SimpleVertex mv2_2 = mes2[i].toAnchor();
			//find jumps - vi ved endnu ikke noget om MusicVertex typerne
			int jump1=SeqViewComparator.diatonicInterval(mv1_1.info, mv1_2.info);
			int jump2=SeqViewComparator.diatonicInterval(mv2_1.info, mv2_2.info);
			//denne straf tilfalder alle der ikke har samme spring
			if(jump1!=jump2){differences+=1.0;}			    
			//find ud af om knuderne i enderne af kanterne 
			//er af parvis samme type
			//comp skal matches med comp og simpel med simpel
			MusicVertex from1=mes1[i].from;
			MusicVertex to1=mes1[i].to;
			MusicVertex from2=mes2[i].from;
			MusicVertex to2=mes2[i].to;
			//frst behandles fraknuderne...
			//comp-comp i fraknuderne
			if(from1 instanceof CompoundVertex && from2 instanceof CompoundVertex) {
			    CompoundVertex from1c=(CompoundVertex)from1;
			    CompoundVertex from2c=(CompoundVertex)from2;
			    //check om forskellig slags compound
			    if(from1c.productionRule.nt!=from2c.productionRule.nt){
				//forsk nonterminal yderligere straf
				//evaluer evt.  forskellene mellem comp'erne
				differences+=1.0;
			    }
			}else if(from1 instanceof SimpleVertex && from2 instanceof SimpleVertex) {
			    // simpel-simpel i fraknuderne - godt
			    // jump-diff er allerede udregnet straffet/belnnet
			}else{//forsk type =>straf
			    differences+=1.0;
			}
			// ... og nu til tilknuderne
			//comp-comp i tilknuderne
			if(to1 instanceof CompoundVertex && to2 instanceof CompoundVertex) {
			    CompoundVertex to1c=(CompoundVertex)to1;
			    CompoundVertex to2c=(CompoundVertex)to2;
			    //check om forskellig slags compound
			    if(to1c.productionRule.nt!=to2c.productionRule.nt){
				//yderligere straf
				differences+=1.0;
			    }//ellers er de ens! og skal ikke straffes
			}else if(to1 instanceof SimpleVertex && to2 instanceof SimpleVertex) {
			    // simpel-simpel i tilknuderne - godt
			}else{//forsk type =>straf
			    differences+=1.0;
			}
		    } 
		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    /* Compound enabled */ 
    public static final SeqViewComparator diatonicInversionIntervalDiffCount=
	new SeqViewComparator("diatonicInversionIntervalDiffCount"){
		public ViewDifference seqCompare(SeqSimStmt sss){

		    MusicEdge[] mes1 = ((SeqSubgraph)sss.subgraph1).getMusicEdgeArray();
		    MusicEdge[] mes2 = ((SeqSubgraph)sss.subgraph2).getMusicEdgeArray();
		    double differences=0.0;
		    for(int i=0;i<mes1.length;i++){
			//find de simple knuder inden i compound'erne
			//(skal bruges til at udregne spring)
			SimpleVertex mv1_1 = mes1[i].fromAnchor();
			SimpleVertex mv1_2 = mes1[i].toAnchor();
			SimpleVertex mv2_1 = mes2[i].fromAnchor();
			SimpleVertex mv2_2 = mes2[i].toAnchor();
			//find jumps - vi ved endnu ikke noget om MusicVertex typerne
			int jump1=SeqViewComparator.diatonicInterval(mv1_1.info, mv1_2.info);
			int jump2=SeqViewComparator.diatonicInterval(mv2_1.info, mv2_2.info);
			//denne straf tilfalder alle der ikke har samme 
			//inverterede spring 
			if(jump1 != (-1)*jump2){differences+=1.0;}			    
			//find ud af om knuderne i enderne af kanterne 
			//er af parvis samme type
			//comp skal matches med comp og simpel med simpel
			MusicVertex from1=mes1[i].from;
			MusicVertex to1=mes1[i].to;
			MusicVertex from2=mes2[i].from;
			MusicVertex to2=mes2[i].to;
			//frst behandles fraknuderne...
			//comp-comp i fraknuderne
			if(from1 instanceof CompoundVertex && from2 instanceof CompoundVertex) {
			    CompoundVertex from1c=(CompoundVertex)from1;
			    CompoundVertex from2c=(CompoundVertex)from2;
			    //check om forskellig slags compound
			    if(from1c.productionRule.nt!=from2c.productionRule.nt){
				//forsk nonterminal yderligere straf
				//evaluer evt.  forskellene mellem comp'erne
				//deres nt'er skal vre de samme, men ikke deres measure!
				differences+=1.0;
			    }
			}else if(from1 instanceof SimpleVertex && from2 instanceof SimpleVertex) {
			    // simpel-simpel i fraknuderne - godt
			    // jump-diff er allerede udregnet straffet/belnnet
			}else{//forsk type =>straf
			    differences+=1.0;
			}
			// ... og nu til tilknuderne
			//comp-comp i tilknuderne
			if(to1 instanceof CompoundVertex && to2 instanceof CompoundVertex) {
			    CompoundVertex to1c=(CompoundVertex)to1;
			    CompoundVertex to2c=(CompoundVertex)to2;
			    //check om forskellig slags compound
			    if(to1c.productionRule.nt!=to2c.productionRule.nt){
				//yderligere straf
				differences+=1.0;
			    }//ellers er de ens! og skal ikke straffes
			}else if(to1 instanceof SimpleVertex && to2 instanceof SimpleVertex) {
			    // simpel-simpel i tilknuderne - godt
			}else{//forsk type =>straf
			    differences+=1.0;
			}
		    } 

		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };




    /* IMPORTANT!! When placed in this same file, the similarity
       measures should be placed AFTER the ViewComparator definitions,
       otherwise the latter will not have been initialised at the time
       when the SimilarityMeasures are initialised, resulting in null
       pointer exceptions. */
		
    /* ---------- SIMILARITY MEASURES ---------- */

    /* */
    public static final SeqSimMeasure seqDFI_Grp =
	new SeqSimMeasure("seqDFI_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicForwardIntervalDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double dfi=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("diatonicForwardIntervalDiffCount")){
			    dfi=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=dfi;
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    public static final SeqSimMeasure seqDI_Grp =
	new SeqSimMeasure("seqDI_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicIntervalDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double di=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("diatonicIntervalDiffCount")){
			    di=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=di;
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    /* */

    public static final SeqSimMeasure seqDFIAndLength_Grp =
	new SeqSimMeasure("seqDFIAndLength_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicAbsPitchDiffCount);
		    viewComparators.add(SimLibDiffCount.pitchIntervalDiffCount);//midi
		    viewComparators.add(SimLibDiffCount.diatonicIntervalDiffCount);
		    viewComparators.add(SimLibDiffCount.diatonicForwardIntervalDiffCount);
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    
		    double dfi=Double.MAX_VALUE;
		    double absLength=Double.MAX_VALUE;
		    double absPitch=Double.MAX_VALUE;
		    double pitchInterval=Double.MAX_VALUE;
		    double diaInterval=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("diatonicForwardIntervalDiffCount")){
			    dfi=vpr.view;
			}else if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else if((vpr.origin.name).equals("diatonicAbsPitchDiffCount")){
			    absPitch=vpr.view;
			}else if((vpr.origin.name).equals("pitchIntervalDiffCount")){
			    pitchInterval=vpr.view;
			}else if((vpr.origin.name).equals("diatonicIntervalDiffCount")){
			    diaInterval=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    
		    result=dfi;
		    result+=absLength*LENGTH_CONST;
		    result+=sizeValue(sss.meanNestedSize());
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    
    public static final SeqSimMeasure seqDiatonicIntervalAndLength_Grp = 
	new SeqSimMeasure("seqDiatonicIntervalAndLength_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicAbsPitchDiffCount);
		    viewComparators.add(SimLibDiffCount.pitchIntervalDiffCount);//midi
		    viewComparators.add(SimLibDiffCount.diatonicIntervalDiffCount);
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    
		    double absLength=Double.MAX_VALUE;
		    double absPitch=Double.MAX_VALUE;
		    double pitchInterval=Double.MAX_VALUE;
		    double diaInterval=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else if((vpr.origin.name).equals("diatonicAbsPitchDiffCount")){
			    absPitch=vpr.view;
			}else if((vpr.origin.name).equals("pitchIntervalDiffCount")){
			    pitchInterval=vpr.view;
			}else if((vpr.origin.name).equals("diatonicIntervalDiffCount")){
			    diaInterval=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    
		    result=diaInterval;
		    result+=absLength*LENGTH_CONST;
		    result+=sizeValue(sss.meanNestedSize());
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    public static final SeqSimMeasure seqPitchIntervalAndLength_Grp = 
	new SeqSimMeasure("seqPitchIntervalAndLength_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicAbsPitchDiffCount);
		    viewComparators.add(SimLibDiffCount.pitchIntervalDiffCount);//midi
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double absLength=Double.MAX_VALUE;
		    double absPitch=Double.MAX_VALUE;
		    double pitchInterval=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			//if((vpr.origin.name).equals("diatonicForwardIntervalDiffCount")){
			    //dfi=vpr.view;
			//}else
			if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else if((vpr.origin.name).equals("diatonicAbsPitchDiffCount")){
			    absPitch=vpr.view;
			}else if((vpr.origin.name).equals("pitchIntervalDiffCount")){
			    pitchInterval=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount.seqPitchIntervalAndLength: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=pitchInterval;
		    result+=absLength*LENGTH_CONST;
		    result+=sizeValue(sss.meanNestedSize());
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };
    

    /* Uses diatonic Abs Pitch */
    public static final SeqSimMeasure seqDiatonicAbsPitchAndLength_Grp = 
	new SeqSimMeasure("seqDiatonicAbsPitchAndLength_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.diatonicAbsPitchDiffCount);
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);
		    /* OBS Grouping */
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double absLength=Double.MAX_VALUE;
		    double absPitch=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;

		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else if((vpr.origin.name).equals("diatonicAbsPitchDiffCount")){
			    absPitch=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("SimLibDiffCount: Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=absPitch;
		    result+=absLength*LENGTH_CONST;
		    result+=sizeValue(sss.meanNestedSize());
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    
    public static final SeqSimMeasure seqDiatonicInversionAndLength_Grp = 
	new SeqSimMeasure("seqDiatonicInversionAndLength_Grp"){
		public void initialize(){
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);
		    viewComparators.add(SimLibDiffCount.diatonicInversionIntervalDiffCount);
		    viewComparators.add(SimLib.seqGroupingStructure);
		}
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double absLength=Double.MAX_VALUE;
		    double diaInversion=Double.MAX_VALUE;
		    double group=Double.MAX_VALUE;
		    
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else if((vpr.origin.name).equals("diatonicInversionIntervalDiffCount")){
			    diaInversion=vpr.view;
			}else if((vpr.origin.name).equals("seqGroupingStructure")){
			    group=vpr.view;
			}else{
			    System.out.println("Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=diaInversion;
		    result+=absLength*LENGTH_CONST;
		    result+=sizeValue(sss.meanNestedSize());
		    result+=group;
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };

    public static double sizeValue(double d){return (1.0-(d*(0.01)));}
    public static final double LENGTH_CONST=0.001;
    
    public static double groupingValue(SimilarityStatement ss){
	GroupingStructure g=new GroupingStructure();
	double g1=g.evaluateGroupings(ss.subgraph1, ss.myOwner.groupBaseGamma);
	double g2=g.evaluateGroupings(ss.subgraph2, ss.myOwner.groupBaseGamma);
	return(g1+g2)/2.0;
    }
}
