/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;
import java.util.Vector;

/** SimLibEdge is a class not to be instantiated. Contains static
   instances of anonymous class extensions of SimilarityMeasure and
   ViewComparator. */
public class SimLibEdge {


    static Random rand=new Random();

    /** If instead of assigning exactly one to bad matches, we assign
     1-some small random amount, there will not be as many equally
     good (or bad, as it is) favorites to combine in the
     findBestCombination2 method (in NonSeqSimMeasure).*/
    public static double almostOne() {
	return 1-(Math.abs(rand.nextGaussian())*0.001);
    }

    public static double one() {
	return 1.0;
    }

    /* ---------- VIEWCOMPARATORS ---------- */
     public static final NSVCEdge nsvcEdgeAbsPitch = new NSVCEdge("nsvcEdgeAbsPitch") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int fromdiff=Math.abs(me1.fromAnchor().info.theNote.getPitch()-me2.fromAnchor().info.theNote.getPitch());
		    int todiff=Math.abs(me1.toAnchor().info.theNote.getPitch()-me2.toAnchor().info.theNote.getPitch());
		    if (fromdiff==0 && todiff==0) result=0;
		    else result=almostOne();
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeAbsPitch2 = new NSVCEdge("nsvcEdgeAbsPitch") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int fromdiff=Math.abs(me1.fromAnchor().info.theNote.getPitch()-me2.fromAnchor().info.theNote.getPitch());
		    int todiff=Math.abs(me1.toAnchor().info.theNote.getPitch()-me2.toAnchor().info.theNote.getPitch());
		    result += 1-1/(1+(double)fromdiff);
		    result += 1-1/(1+(double)todiff);
		    result = result/2;
		    // if (p1!=p2) result=1.0;
		    // else result=0.0;
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgePitchInterval = new NSVCEdge("nsvcEdgePitchInterval") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=me1.toAnchor().info.theNote.getPitch()-me1.fromAnchor().info.theNote.getPitch();
		    int p2=me2.toAnchor().info.theNote.getPitch()-me2.fromAnchor().info.theNote.getPitch();

		    if (p1!=p2) result=almostOne();
		    else result=0.0;
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgePitchInterval2 = new NSVCEdge("nsvcEdgePitchInterval2") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=me1.toAnchor().info.theNote.getPitch()-me1.fromAnchor().info.theNote.getPitch();
		    int p2=me2.toAnchor().info.theNote.getPitch()-me2.fromAnchor().info.theNote.getPitch();
		    result += 1-1/(1+(double)Math.abs(p1-p2));
		    // if (p1!=p2) result=1.0;
		    // else result=0.0;
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgePitchContour = new NSVCEdge("nsvcEdgePitchContour") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=me1.toAnchor().info.theNote.getPitch()-me1.fromAnchor().info.theNote.getPitch();
		    int p2=me2.toAnchor().info.theNote.getPitch()-me2.fromAnchor().info.theNote.getPitch();
		    if (p1<0 && p2<0 ||
			p1==0 && p2==0 ||
			p1>0 &&p2>0 ) result= 0.0;
		    else result=almostOne();
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeDAP = new NSVCEdge("nsvcEdgeDAP") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		double result = 0.0;
		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    NoteInfo from1=me1.fromAnchor().info;
		    NoteInfo to1=me1.toAnchor().info;
		    NoteInfo from2=me2.fromAnchor().info;
		    NoteInfo to2=me2.toAnchor().info;

		    if (from1.diatonicAbsolutePitchEquals(from2) && 
			to1.diatonicAbsolutePitchEquals(to2) )  result=0.0;
		    else result=almostOne();
		}
		return result;
	    }
	};
     public static final NSVCEdge nsvcEdgeDI = new NSVCEdge("nsvcEdgeDI") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		double result = 0.0;
		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=SeqViewComparator.getDiatonicInterval(me1);
		    int p2=SeqViewComparator.getDiatonicInterval(me2);

		    if (p1!=p2) result=almostOne();
		    else result=0.0;
		}
		return result;
	    }
	};
     public static final NSVCEdge nsvcEdgeDFI = new NSVCEdge("nsvcEdgeDFI") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		double result = 0.0;
		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=SeqViewComparator.getDiatonicForwardInterval(me1);
		    int p2=SeqViewComparator.getDiatonicForwardInterval(me2);

		    if (p1!=p2) result=almostOne();
		    else result=0.0;
		}
		return result;
	    }
	};
    


     public static final NSVCEdge nsvcEdgeAbsLength = new NSVCEdge("nsvcEdgeAbsLength") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    double fromdiff=Math.abs(me1.from.getLength()-me2.from.getLength());
		    double todiff=Math.abs(me1.to.getLength()-me2.to.getLength());
		    if (fromdiff==0 && todiff==0) {
			result=0;
		    } else {
			result=almostOne();
		    }
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeAbsLength2 = new NSVCEdge("nsvcEdgeAbsLength2") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    double fromdiff=Math.abs(me1.from.getLength()-me2.from.getLength());
		    double todiff=Math.abs(me1.to.getLength()-me2.to.getLength());
		    result += (1-1/(1+fromdiff))/2;
		    result += (1-1/(1+todiff))/2;
		    // if (p1!=p2) result=1.0;
		    // else result=0.0;
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeAbsAnchorLength = new NSVCEdge("nsvcEdgeAbsAnchorLength") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    double fromdiff=Math.abs(me1.fromAnchor().getLength()-me2.fromAnchor().getLength());
		    double todiff=Math.abs(me1.toAnchor().getLength()-me2.toAnchor().getLength());
		    result += (1-1/(1+(double)fromdiff))/2;
		    result += (1-1/(1+(double)todiff))/2;
		    // if (p1!=p2) result=1.0;
		    // else result=0.0;
		}
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeLengthInterval = new NSVCEdge("nsvcEdgeLengthInterval") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=FuncLib.lengthInterval(me1.to.getLength(),
						  me1.from.getLength());
		    int p2=FuncLib.lengthInterval(me2.to.getLength(),
						  me2.from.getLength());
		    result += 1-1/(1+(double)Math.abs(p1-p2));
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeLengthContour = new NSVCEdge("nsvcEdgeLengthContour") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int p1=FuncLib.lengthInterval(me1.to.getLength(),
						  me1.from.getLength());
		    int p2=FuncLib.lengthInterval(me2.to.getLength(),
						  me2.from.getLength());
		    if (p1<0 && p2<0 ||
			p1==0 && p2==0 ||
			p1>0 &&p2>0 ) result= 0.0;
		    else result=1.0;
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeWeakIn = new NSVCEdge("nsvcEdgeWeakIn") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperWeakFollowInEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperWeakFollowInEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperWeakFollowInEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperWeakFollowInEdges(nsss.subgraph2).size();

		    //if (n11==n21 && n12==n22) result =0;
		    //else result=almostOne();
		    result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeStrongIn = new NSVCEdge("nsvcEdgeStrongIn") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperStrongFollowInEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperStrongFollowInEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperStrongFollowInEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperStrongFollowInEdges(nsss.subgraph2).size();

		    //if (n11==n21 && n12==n22) result =0;
		    //else result=almostOne();
		    result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeWeakOut = new NSVCEdge("nsvcEdgeWeakOut") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperWeakFollowOutEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperWeakFollowOutEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperWeakFollowOutEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperWeakFollowOutEdges(nsss.subgraph2).size();

		    //if (n11==n21 && n12==n22) result =0;
		    //else result=almostOne();
		    result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
     public static final NSVCEdge nsvcEdgeStrongOut = new NSVCEdge("nsvcEdgeStrongOut") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperStrongFollowOutEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperStrongFollowOutEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperStrongFollowOutEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperStrongFollowOutEdges(nsss.subgraph2).size();

		    if (n11==n21 && n12==n22) result =0;
		    else result=almostOne();
		    //result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    //result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeWeakSimult = new NSVCEdge("nsvcEdgeWeakSimult") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperWeakSimultaneousInEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperWeakSimultaneousInEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperWeakSimultaneousInEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperWeakSimultaneousInEdges(nsss.subgraph2).size();

		    //if (n11==n21 && n12==n22) result =0;
		    //else result=almostOne();
		    result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
    
     public static final NSVCEdge nsvcEdgeStrongSimult = new NSVCEdge("nsvcEdgeStrongSimult") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDEdge vd= new NSVDEdge(0.0/*dummy*/,this);

		vd.similarities = getEdgeSimMatrix(nsss);
		return vd;
	    }
	    protected double compareEdges(MusicEdge me1, 
					  MusicEdge me2,
					  NonSeqSimStmt nsss) {
		double result = 0.0;

		if (me1.edgetype!=me2.edgetype) result =almostOne();
		else {
		    int n11 = me1.from.getProperStrongSimultaneousInEdges(nsss.subgraph1).size();
		    int n12 = me1.to.getProperStrongSimultaneousInEdges(nsss.subgraph1).size();
		    int n21 = me2.from.getProperStrongSimultaneousInEdges(nsss.subgraph2).size();
		    int n22 = me2.to.getProperStrongSimultaneousInEdges(nsss.subgraph2).size();

		    //if (n11==n21 && n12==n22) result =0;
		    //else result=almostOne();
		    result += (1-1/(1+(double)Math.abs(n11-n21)))/2;
		    result += (1-1/(1+(double)Math.abs(n12-n22)))/2;
		}		
		return result;
	    }
	};
    


    /* IMPORTANT!! When placed in this same file, the similarity
       measures should be placed AFTER the ViewComparator definitions,
       otherwise the latter will not have been initialised at the time
       when the SimilarityMeasures are initialised, resulting in null
       pointer exceptions. */
		
    /* ---------- SIMILARITY MEASURES ---------- */
    public static final NSSMEdge nonSeqEdgePLIOS = new NSSMEdge("nonSeqEdgePLIOS") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgePitchInterval);
		// viewComparators.add(SimLibEdge.nsvcEdgePitchContour);
		viewComparators.add(SimLibEdge.nsvcEdgeLengthInterval);
		// viewComparators.add(SimLibEdge.nsvcEdgeLengthContour);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		/* // make sure subgraph1 has most edges (for non-symmetric matching)
		if (nsss.subgraph2.getFollowEdges().size()>nsss.subgraph1.getFollowEdges().size()) {
		    Subgraph sg = nsss.subgraph2;
		    nsss.subgraph2=nsss.subgraph1;
		    nsss.subgraph1=sg;
		    }*/ 

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=er.getMeanEdgeSimMatrix();
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }
	};

    public static final NSSMEdge nonSeqEdgeAPAL = new NSSMEdge("nonSeqEdgeAPAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeAbsPitch);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeAbsPitch) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgePIAL = new NSSMEdge("nonSeqEdgePIAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgePitchInterval);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgePitchInterval) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgePCAL = new NSSMEdge("nonSeqEdgePCAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgePitchContour);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgePitchContour) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgeDAPAL = new NSSMEdge("nonSeqEdgeDAPAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDAP);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDAP) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgeDIAL = new NSSMEdge("nonSeqEdgeDIAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDI);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDI) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgeDFIAL = new NSSMEdge("nonSeqEdgeDFIAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDFI);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeWeakIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeWeakSimult);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDFI) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    /* for use without compounds: */
    public static final NSSMEdge nonSeqEdgeAPAL_nocompounds = new NSSMEdge("nonSeqEdgeAPAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeAbsPitch);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeAbsPitch) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};
    public static final NSSMEdge nonSeqEdgePIAL_nocompounds = new NSSMEdge("nonSeqEdgePIAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgePitchInterval);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgePitchInterval) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgePCAL_nocompounds = new NSSMEdge("nonSeqEdgePCAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgePitchContour);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgePitchContour) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};

    public static final NSSMEdge nonSeqEdgeDAPAL_nocompounds = new NSSMEdge("nonSeqEdgeDAPAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDAP);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDAP) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};
    public static final NSSMEdge nonSeqEdgeDIAL_nocompounds = new NSSMEdge("nonSeqEdgeDIAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDI);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDI) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};
    public static final NSSMEdge nonSeqEdgeDFIAL_nocompounds = new NSSMEdge("nonSeqEdgeDFIAL_nocompounds") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibEdge.nsvcEdgeDFI);
		viewComparators.add(SimLibEdge.nsvcEdgeAbsLength);
		/* IF YOU CHANGE THESE, remember to change the names
                   matched to in the weighting below*/

		viewComparators.add(SimLibEdge.nsvcEdgeStrongIn);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongOut);
		viewComparators.add(SimLibEdge.nsvcEdgeStrongSimult);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMEdgeResult er = new NSSMEdgeResult(this);

		// make sure edges are sorted
		nsss.subgraph1.sortEdges();
		nsss.subgraph2.sortEdges();
	   
		er.viewDifferences=evaluateAllViewComparators(nsss);
		er.meanSimilarities=getWeightedMeanEdgeSimMatrix(er);
		er.bestMatch= getBestMatch(nsss,er);
		er.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		er.measure = getFinalValue(nsss,er);
		/* END CORE. */
		
		nsss.smResult = er;
	    }

	    public double[][] getWeightedMeanEdgeSimMatrix(NSSMEdgeResult er) {
		NSVDEdge vd = (NSVDEdge) er.viewDifferences.get(0);
		if (vd.empty()) return new double[0][0];
		double[][] edgeSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<er.viewDifferences.size(); i++) {
		    vd = (NSVDEdge) er.viewDifferences.get(i);

		    /* match to different VC names, change these
                       according to VCs added above */
		    if (vd.origin==SimLibEdge.nsvcEdgeDFI) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==SimLibEdge.nsvcEdgeAbsLength) {
			edgeSimilarities = FuncLib.linearCombineMatrices(edgeSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			edgeSimilarities = FuncLib.addMatrices(edgeSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(edgeSimilarities,totalWeight);
	    }
	};


}
