/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** This is one of the three classes comprising the example of how to
    extend the SimFinder system. See also {@link
    StartSimSegmenterExample} and {@link
    StartSimFinderExample}. */
public class SimLibExample{

    /* SeqViewComparator, Compound enabled */ 
    public static final SeqViewComparator diatonicRetrogradeIntervalDiffCount=
	new SeqViewComparator("diatonicRetrogradeIntervalDiffCount"){

		public ViewDifference seqCompare(SeqSimStmt sss){
		    //first we make 2 arrays of the diatonic jumps of the subgraphs
		    MusicEdge[] mes1 = ((SeqSubgraph)sss.subgraph1).getMusicEdgeArray();
		    MusicEdge[] mes2 = ((SeqSubgraph)sss.subgraph2).getMusicEdgeArray();
		    int[] jumps1=new int[mes1.length];
		    int[] jumps2=new int[mes2.length];
		    for(int i=0;i<mes1.length;i++){
			SimpleVertex mv1_1 = mes1[i].fromAnchor();
			SimpleVertex mv1_2 = mes1[i].toAnchor();
			SimpleVertex mv2_1 = mes2[i].fromAnchor();
			SimpleVertex mv2_2 = mes2[i].toAnchor();
			jumps1[i]=SeqViewComparator.diatonicInterval(mv1_1.info, mv1_2.info);
			jumps2[i]=SeqViewComparator.diatonicInterval(mv2_1.info, mv2_2.info);
		    }

		    double differences=0.0;
		    for(int i=0;i<jumps1.length;i++){//run through the jumps
			// we reward the ones which have the reverse jumps by punishing
			// the ones that don't
			if(jumps1[i] != (-1)*jumps2[jumps2.length-i-1]){
			    differences+=1.0;
			}
			/*** The rest of the code takes care of compounds: ***/ 
			//check that vertices are of pairwise same type (comp same nt)
			MusicVertex from1=mes1[i].from;
			MusicVertex to1=mes1[i].to;
			MusicVertex from2=mes2[i].to;/*OBS here we switch to and from */
			MusicVertex to2=mes2[i].from;/*OBS here we switch to and from */
			//by doing that we can reuse the vertice comparison used for the other
			//viewcomparators...
			if(from1 instanceof CompoundVertex && from2 instanceof CompoundVertex) {
			    CompoundVertex from1c=(CompoundVertex)from1;
			    CompoundVertex from2c=(CompoundVertex)from2;
			    if(from1c.productionRule.nt!=from2c.productionRule.nt){
				differences+=1.0;//diff nt => punishment
			    }
			}else if(from1 instanceof SimpleVertex && from2 instanceof SimpleVertex) {//ok
			}else{//diff type => punishment
			    differences+=1.0;
			}
			if(to1 instanceof CompoundVertex && to2 instanceof CompoundVertex) {
			    CompoundVertex to1c=(CompoundVertex)to1;
			    CompoundVertex to2c=(CompoundVertex)to2;
			    if(to1c.productionRule.nt!=to2c.productionRule.nt){//check compound number
				differences+=1.0;
			    }//otherwise equal - no punishment
			}else if(to1 instanceof SimpleVertex && to2 instanceof SimpleVertex) {//ok
			}else{//forsk type =>straf
			    differences+=1.0;
			}
		    } 

		    ViewDifference result = new ViewDifference(differences,this);
		    return result;
		}
	    };

    public static final SeqSimMeasure seqDiatonicRetrogradeInterval = 
	new SeqSimMeasure("seqDiatonicRetrogradeInterval"){
		public void initialize(){//add the View Comparators
		    viewComparators.clear();
		    viewComparators.add(SimLibDiffCount.absLengthDiffCount);//SimLib function
		    viewComparators.add(diatonicRetrogradeIntervalDiffCount);
		}
		public void evaluateSeq(SeqSimStmt sss){
		    double result=0.0;
		    Vector viewdiffs=evaluateAllViewComparators(sss);
		    double absLength=Double.MAX_VALUE;
		    double retro=Double.MAX_VALUE;
		    
		    for(int i=0;i<viewdiffs.size();i++){
			ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
			if((vpr.origin.name).equals("diatonicRetrogradeIntervalDiffCount")){
			    retro=vpr.view;
			}else if((vpr.origin.name).equals("absLengthDiffCount")){
			    absLength=vpr.view;
			}else{
			    System.out.println("Unknown ViewComparator error:");
			    System.out.println(vpr.origin.name);
			    System.exit(0);
			}
		    }
		    result=retro;
		    result+=absLength*0.01;
		    result+=SimLibDiffCount.sizeValue(sss.meanNestedSize());
		    SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		    sss.smResult=smr;
		}
	    };




 
}
