/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;
import java.util.Vector;

/** A class not to be instantiated. Contains static instances of
   anonymous class extensions of SimilarityMeasure and
   ViewComparator. */
public class SimLibMeanDiff{

    /* ---------- VIEWCOMPARATORS ---------- */
    public static final SeqViewComparator absolutePitchMeanDiff=new SeqViewComparator("absPitchMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortVerticesByStarttime();
		ss2.sortVerticesByStarttime();

		int differences =0;
		for (int i=0; i<ss1.vertices.size(); i++) {
		    MusicVertex mv1 = (MusicVertex) ss1.vertices.get(i);
		    MusicVertex mv2 = (MusicVertex) ss2.vertices.get(i);
		    if (mv1 instanceof SimpleVertex) {
			if (mv2 instanceof SimpleVertex) {
			    int pitch1=((SimpleVertex)mv1).info.theNote.getPitch();
			    int pitch2=((SimpleVertex)mv2).info.theNote.getPitch();
			    if (pitch1!=pitch2) differences++;
			} else if (mv2 instanceof CompoundVertex) {
			    differences++;
			}
		    } else if (mv1 instanceof CompoundVertex) {
			if (mv2 instanceof SimpleVertex) {
			    differences++;
			} else if (mv2 instanceof CompoundVertex) {
			    CompoundVertex cv1 = (CompoundVertex) mv1;
			    CompoundVertex cv2 = (CompoundVertex) mv2;
			    //if (cv1.productionRule.nt!=cv2.productionRule.nt) differences++;
			    differences+=SimLib.compareCompoundsUnderViewComparator(cv1,cv2,this);
			}
		    }

		}
		double view = ((double) differences)/(double)ss1.vertices.size();

		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	};
    
    public static final SeqViewComparator pitchIntervalMeanDiff= new SeqViewComparator("pitchIntervalMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortEdges();
		ss2.sortEdges();
		final int REST_TO_NOTE=-10000;
		final int REST_TO_REST=-20000;
		final int NOTE_TO_REST=-30000;

		int differences =0;
		for (int i=0; i<ss1.edges.size(); i++) {
		    MusicEdge me1 = (MusicEdge) ss1.edges.get(i);
		    MusicEdge me2 = (MusicEdge) ss2.edges.get(i);
		    int pitch11=me1.fromAnchor().info.theNote.getPitch();
		    int pitch12=me1.toAnchor().info.theNote.getPitch();
		    int pitch21=me2.fromAnchor().info.theNote.getPitch();
		    int pitch22=me2.toAnchor().info.theNote.getPitch();
		    
		    int interval1=0;
		    int interval2=0;

		    if (pitch11==Integer.MIN_VALUE) {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=REST_TO_REST;
			} else {
			    interval1=REST_TO_NOTE;
			}
		    } else {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=NOTE_TO_REST;
			} else {
			    interval1=pitch12-pitch11;
			}
		    }
		    if (pitch21==Integer.MIN_VALUE) {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=REST_TO_REST;
			} else {
			    interval2=REST_TO_NOTE;
			}
		    } else {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=NOTE_TO_REST;
			} else {
			    interval2=pitch22-pitch21;
			}
		    }
		    if (interval1!=interval2) differences++;
		}		    
		    
		double view = ((double) differences)/(double)ss1.edges.size();

		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	};

    /* in this version intervals are always positive */
    public static final SeqViewComparator pitchAbsIntervalMeanDiff= new SeqViewComparator("pitchAbsIntervalMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortEdges();
		ss2.sortEdges();
		final int REST_TO_NOTE=-10000;
		final int REST_TO_REST=-20000;
		final int NOTE_TO_REST=-30000;

		int differences =0;
		for (int i=0; i<ss1.edges.size(); i++) {
		    MusicEdge me1 = (MusicEdge) ss1.edges.get(i);
		    MusicEdge me2 = (MusicEdge) ss2.edges.get(i);
		    int pitch11=me1.fromAnchor().info.theNote.getPitch();
		    int pitch12=me1.toAnchor().info.theNote.getPitch();
		    int pitch21=me2.fromAnchor().info.theNote.getPitch();
		    int pitch22=me2.toAnchor().info.theNote.getPitch();
		    
		    int interval1=0;
		    int interval2=0;

		    if (pitch11==Integer.MIN_VALUE) {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=REST_TO_REST;
			} else {
			    interval1=REST_TO_NOTE;
			}
		    } else {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=NOTE_TO_REST;
			} else {
			    interval1=Math.abs(pitch12-pitch11);
			}
		    }
		    if (pitch21==Integer.MIN_VALUE) {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=REST_TO_REST;
			} else {
			    interval2=REST_TO_NOTE;
			}
		    } else {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=NOTE_TO_REST;
			} else {
			    interval2=Math.abs(pitch22-pitch21);
			}
		    }
		    if (interval1!=interval2) differences++;
		}		    
		double view = ((double) differences)/(double)ss1.edges.size();
		ViewDifference result = new ViewDifference(view,this);
		return result;
	    }
	};

    public static final SeqViewComparator pitchContourMeanDiff= new SeqViewComparator("pitchContourMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortEdges();
		ss2.sortEdges();
		final int REST_TO_NOTE=-10000;
		final int REST_TO_REST=-20000;
		final int NOTE_TO_REST=-30000;

		int differences =0;
		for (int i=0; i<ss1.edges.size(); i++) {
		    MusicEdge me1 = (MusicEdge) ss1.edges.get(i);
		    MusicEdge me2 = (MusicEdge) ss2.edges.get(i);
		    int pitch11=me1.fromAnchor().info.theNote.getPitch();
		    int pitch12=me1.toAnchor().info.theNote.getPitch();
		    int pitch21=me2.fromAnchor().info.theNote.getPitch();
		    int pitch22=me2.toAnchor().info.theNote.getPitch();
		    
		    int interval1=0;
		    int interval2=0;

		    if (pitch11==Integer.MIN_VALUE) {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=REST_TO_REST;
			} else {
			    interval1=REST_TO_NOTE;
			}
		    } else {
			if (pitch12==Integer.MIN_VALUE) {
			    interval1=NOTE_TO_REST;
			} else {
			    interval1=pitch12-pitch11;
			}
		    }
		    if (pitch21==Integer.MIN_VALUE) {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=REST_TO_REST;
			} else {
			    interval2=REST_TO_NOTE;
			}
		    } else {
			if (pitch22==Integer.MIN_VALUE) {
			    interval2=NOTE_TO_REST;
			} else {
			    interval2=pitch22-pitch21;
			}
		    }
		    if (interval1!=interval2) {
			if (interval1>-9000) {
			    if (interval1<0) interval1=-1;
			    if (interval1>0) interval1=1;
			}
			if (interval2>-9000) {
			    if (interval2<0) interval2=-1;
			    if (interval2>0) interval2=1;
			}
			if (interval1!=interval2) differences++;
		    }  
		}		    
		    
		double view = ((double) differences)/(double)ss1.edges.size();

		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	    /*	    public ViewDifference seqCompare(SeqSimStmt sss) {
		int[] p1 =((SeqSubgraph)sss.subgraph1).getPitchArray();
		int[] p2 =((SeqSubgraph)sss.subgraph2).getPitchArray();
		p1 = convertPauses(p1);
		p2 = convertPauses(p2);

		double view = 0.0;
		for (int i=0; i<p1.length-1; i++) {
		    int diff1=0, diff2=0;
		    // first entry might still be a pause
		    if (p1[i]==Integer.MIN_VALUE) diff1=0;
		    else diff1 = p1[i+1] -p1[i];
		    if (p2[i]==Integer.MIN_VALUE) diff2=0;
		    else diff2 = p2[i+1] -p2[i];
		    
		    if (diff1>0) diff1=1;
		    else if (diff1<0) diff1=-1;
		    if (diff2>0) diff2=1;
		    else if (diff2<0) diff2=-1;
		    
		    if (diff1!=diff2) view +=1;
		}

		view =view/(double)(p1.length-1);

		ViewDifference result = new ViewDifference(view,this);

		return result;
		}*/
	};
		
    public static final SeqViewComparator pitchClassMeanDiff=new SeqViewComparator("pitchClassMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortVerticesByStarttime();
		ss2.sortVerticesByStarttime();

		int differences =0;
		for (int i=0; i<ss1.vertices.size(); i++) {
		    MusicVertex mv1 = (MusicVertex) ss1.vertices.get(i);
		    MusicVertex mv2 = (MusicVertex) ss2.vertices.get(i);
		    if (mv1 instanceof SimpleVertex) {
			if (mv2 instanceof SimpleVertex) {
			    int pitch1=((SimpleVertex)mv1).info.theNote.getPitch();
			    int pitch2=((SimpleVertex)mv2).info.theNote.getPitch();
			    if ((pitch1%12)!=(pitch2%12)) differences++;
			} else if (mv2 instanceof CompoundVertex) {
			    differences++;
			}
		    } else if (mv1 instanceof CompoundVertex) {
			if (mv2 instanceof SimpleVertex) {
			    differences++;
			} else if (mv2 instanceof CompoundVertex) {
			    CompoundVertex cv1 = (CompoundVertex) mv1;
			    CompoundVertex cv2 = (CompoundVertex) mv2;
			    // if (cv1.productionRule.nt!=cv2.productionRule.nt) differences++;
			    differences+=SimLib.compareCompoundsUnderViewComparator(cv1,cv2,this);
			}
		    }
		}
		double view = ((double) differences)/(double)ss1.vertices.size();

		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	    /*public ViewDifference seqCompare(SeqSimStmt sss) {
		int[] p1 =((SeqSubgraph)sss.subgraph1).getPitchClassArray();
		int[] p2 =((SeqSubgraph)sss.subgraph2).getPitchClassArray();
		
		double view = ((double) countDifferencesInt(p1,p2))/(double)p1.length;

		ViewDifference result = new ViewDifference(view,this);
		
		return result;
		}*/
	};
    
    public static final SeqViewComparator absoluteLengthMeanDiff=new SeqViewComparator("absLengthMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}
		
		int differences=0;
		for (int i=0; i<ss1.vertices.size(); i++) {
		    MusicVertex mv1 = (MusicVertex) ss1.vertices.get(i);
		    MusicVertex mv2 = (MusicVertex) ss2.vertices.get(i);
		    if (mv1.getLength()!=mv2.getLength()) differences++;
		}
		

		double view =((double) differences)/(double)(sss.subgraph1.vertices.size());

		ViewDifference result = new ViewDifference(view,this);

		return result;
	    }
	};

    public static final SeqViewComparator lengthIntervalMeanDiff=new SeqViewComparator("lengthIntervalMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortEdges();
		ss2.sortEdges();

		int differences =0;
		for (int i=0; i<ss1.edges.size(); i++) {
		    MusicEdge me1 = (MusicEdge) ss1.edges.get(i);
		    MusicEdge me2 = (MusicEdge) ss2.edges.get(i);
		    double len11=me1.from.getLength();
		    double len12=me1.to.getLength();
		    double len21=me2.from.getLength();
		    double len22=me2.to.getLength();
		    
		    double interval1=len12/len11;
		    double interval2=len22/len21;

		    if (interval1!=interval2) differences++;
		}		    
		double view = ((double) differences)/(double)ss1.edges.size();
		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	};

    public static final SeqViewComparator lengthContourMeanDiff=new SeqViewComparator("lengthContourMD") {
	    public ViewDifference seqCompare(SeqSimStmt sss) {
		//System.out.println("seqCompare");
		SeqSubgraph ss1 = (SeqSubgraph)sss.subgraph1;
		SeqSubgraph ss2 = (SeqSubgraph)sss.subgraph2;
		if (ss1.vertices.size() != ss2.vertices.size()) {
		    return new ViewDifference(1,this);
		}

		ss1.sortEdges();
		ss2.sortEdges();

		int differences =0;
		for (int i=0; i<ss1.edges.size(); i++) {
		    MusicEdge me1 = (MusicEdge) ss1.edges.get(i);
		    MusicEdge me2 = (MusicEdge) ss2.edges.get(i);
		    double len11=me1.from.getLength();
		    double len12=me1.to.getLength();
		    double len21=me2.from.getLength();
		    double len22=me2.to.getLength();
		    
		    double interval1=0, interval2=0;
		    if (len12>len11) interval1=1;
		    else if (len12<len11) interval1=-1;
		    if (len22>len21) interval2=1;
		    else if (len22<len21) interval2=-1;

		    if (interval1!=interval2) differences++;
		}		    
		double view = ((double) differences)/(double)ss1.edges.size();
		ViewDifference result = new ViewDifference(view,this);
		
		return result;
	    }
	};


    /* IMPORTANT!! When placed in this same file, the similarity
       measures should be placed AFTER the ViewComparator definitions,
       otherwise the latter will not have been initialised at the time
       when the SimilarityMeasures are initialised, resulting in null
       pointer exceptions. */
		
    /* ---------- SIMILARITY MEASURES ---------- */
    public static final SeqSimMeasure seqOriginalMBD6 = new SeqSimMeasure("seqOriginalMBD6") {

	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.absolutePitchMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchContourMeanDiff);
		viewComparators.add(SimLibMeanDiff.absoluteLengthMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthContourMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/

		double bonusValue = 0.5;
		double k=Math.pow(2,viewdiffs.size());
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}
		result = product/(k-(2*(k-1)/Math.pow(Math.sqrt(2),sss.meanNestedSize())));
		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};
    public static final SeqSimMeasure seqMD_APAL = new SeqSimMeasure("seqMD_APAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.absolutePitchMeanDiff);
		viewComparators.add(SimLibMeanDiff.absolutePitchMeanDiff);
		viewComparators.add(SimLibMeanDiff.absoluteLengthMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/

		double bonusValue = bval;
		double maxValue=Math.pow(2,viewdiffs.size());
		double minValue=Math.pow(bonusValue,viewdiffs.size());
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}


		double mu=mew;
		double value2=(product-minValue+mu)/(maxValue-minValue+mu);
		double valueWeight=value2;
		double sizeWeight=1-valueWeight;
		double fi = phi; // flattening of the sizemod curve

		double value3=(valueWeight+(fi*sizeWeight)/(sss.meanNestedSize()+fi-1)); /// (sizeWeight+valueWeight);
		result=value2*value3;

		/*		System.out.println("pr="+product+"|v2="+value2+"|v3="+value3+"|res="+result);
				System.out.println("bval="+bonusValue+"|mu="+mu+"|v3="+value3+"|res="+result);*/

		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};
    public static final SeqSimMeasure seqMD_PIAL = new SeqSimMeasure("seqMD_PIAL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.absoluteLengthMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/
		double bonusValue = bval;
		double k=Math.pow(2,viewdiffs.size());
		double maxValue=1;
		double minValue=Math.pow(bonusValue,viewdiffs.size())/k;
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}

		double value1=product/k;
		double mu=mew;
		double value2=(value1-minValue+mu)/(maxValue-minValue+mu);
		double valueWeight=value2;
		double sizeWeight=1-valueWeight;
		double fi = phi; // flattening of the sizemod curve

		double value3=(valueWeight+(fi*sizeWeight)/(sss.meanNestedSize()+fi-1));///(sizeWeight+valueWeight);
		result=value2*value3;

		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};
    public static final SeqSimMeasure seqMD_PILI = new SeqSimMeasure("seqMD_PILI") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthIntervalMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/

		/*		double bonusValue = 0.5;
		double k=Math.pow(2,viewdiffs.size());
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}
		result = product/(k-(2*(k-1)/Math.pow(Math.sqrt(2),sss.meanNestedSize())));*/
		double bonusValue = bval;
		double k=Math.pow(2,viewdiffs.size());
		double maxValue=1;
		double minValue=Math.pow(bonusValue,viewdiffs.size())/k;
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}


		double value1=product/k;
		double mu=mew;
		double value2=(value1-minValue+mu)/(maxValue-minValue+mu);
		double valueWeight=value2;
		double sizeWeight=1-valueWeight;
		double fi = phi; // flattening of the sizemod curve

		double value3=(valueWeight+(fi*sizeWeight)/(sss.meanNestedSize()+fi-1))/
		    (sizeWeight+valueWeight);
		result=value2*value3;

		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};
    public static final SeqSimMeasure seqMD_PCLC = new SeqSimMeasure("seqMD_PCLC") {

	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.pitchContourMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchContourMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthContourMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/

		/*double bonusValue = 0.5;
		double k=Math.pow(2,viewdiffs.size());
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}
		result = product/(k-(2*(k-1)/Math.pow(Math.sqrt(2),sss.meanNestedSize())));*/
		double bonusValue = bval;
		double k=Math.pow(2,viewdiffs.size());
		double maxValue=1;
		double minValue=Math.pow(bonusValue,viewdiffs.size())/k;
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}


		double value1=product/k;
		double mu=mew;
		double value2=(value1-minValue+mu)/(maxValue-minValue+mu);
		double valueWeight=value2;
		double sizeWeight=1-valueWeight;
		double fi = phi; // flattening of the sizemod curve

		double value3=(valueWeight+(fi*sizeWeight)/(sss.meanNestedSize()+fi-1))/
		    (sizeWeight+valueWeight);
		result=value2*value3;


		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};
    public static final SeqSimMeasure seqMBD7 = new SeqSimMeasure("seqMBD7") {

	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibMeanDiff.absolutePitchMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchClassMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.pitchContourMeanDiff);
		viewComparators.add(SimLibMeanDiff.absoluteLengthMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthIntervalMeanDiff);
		viewComparators.add(SimLibMeanDiff.lengthContourMeanDiff);
	    }
	    public void evaluateSeq(SeqSimStmt sss) {
		Vector viewdiffs=evaluateAllViewComparators(sss);

		double result=0.0;
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * SeqSimMeasure*/

		double bonusValue = 0.5;
		double k=Math.pow(2,viewdiffs.size());
		double product =1.0;
		for (int i=0; i<viewdiffs.size(); i++) {
		    ViewDifference vpr = (ViewDifference) viewdiffs.get(i);
		    if (vpr.view==0.0) {product = product*bonusValue; }
		    else { product = product * (1+vpr.view) ; }
		}
		result = product/(k-(2*(k-1)/Math.pow(Math.sqrt(2),sss.meanNestedSize())));
		/* END CORE. */

		SimMeasureResult smr = new SimMeasureResult(result,viewdiffs,this);
		sss.smResult = smr;
	    }
	    
	};

}
