/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.*;
import java.util.Vector;

/** A class not to be instantiated. Contains static instances of
   anonymous class extensions of SimilarityMeasure and ViewComparator. */
public class SimLibVertex{


    public static double nestedSizeRatio(NonSeqSimStmt nsss) {
	double ns1=nsss.subgraph1.nestedSize();
	double ns2=nsss.subgraph2.nestedSize();

	return Math.min(ns1,ns2)/Math.max(ns1,ns2);
    }

    /* ---------- VIEWCOMPARATORS ---------- */
    public static final NSVCVertex nsvcVertexPitch = new NSVCVertex("nsvcVertexPitch") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		double result = 0.0;
		if ((mv1 instanceof SimpleVertex) && (mv2 instanceof SimpleVertex)) {
		    // both mv1 and mv2 are SimpleVertices
		    int pitch1=((SimpleVertex)mv1).info.theNote.getPitch();
		    int pitch2=((SimpleVertex)mv2).info.theNote.getPitch();
		    
		    result += 1.0-1.0/(1.0+(double)Math.abs(pitch1-pitch2));
		} else if ((mv1 instanceof CompoundVertex) && (mv2 instanceof CompoundVertex)) {
		    // both mv1 and mv2 are CompoundVertices
		    CompoundVertex cv1 =(CompoundVertex)mv1;
		    CompoundVertex cv2 =(CompoundVertex)mv2;
		    /*NonSeqSimStmt ns = new NonSeqSimStmt((NonSeqSubgraph)cv1.subgraph,
							 (NonSeqSubgraph)cv2.subgraph,
						 	 SimLib.nonSeqVertexP,null);
							 result = ns.value();*/
		    if (cv1.productionRule.nt==cv2.productionRule.nt) {
			result=0;
		    } else result=1;
		} else {
		    // mv1 and mv2 are not of same type.
		    result=1.0;
		}
		return result;
		
	    }
	};
    
    public static final NSVCVertex nsvcVertexLength = new NSVCVertex("nsvcVertexLength") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		double length1=mv1.getLength();
		double length2=mv2.getLength();
		result += 1-1/(1+(double)Math.abs(FuncLib.lengthInterval(length1,length2)));
		return result;
	    }
	};
    
    /* Count both weak and strong edges */
    public static final NSVCVertex nsvcVertexIn = new NSVCVertex("nsvcVertexIn") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int in1= mv1.getProperFollowInEdges(nsss.subgraph1).size();
		int in2= mv2.getProperFollowInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(in1-in2));
		return result;
	    }
	};
    public static final NSVCVertex nsvcVertexOut = new NSVCVertex("nsvcVertexOut") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
	
		int out1= mv1.getProperFollowOutEdges(nsss.subgraph1).size();
		int out2= mv2.getProperFollowOutEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(out1-out2));
		return result;
	    }
	};
    public static final NSVCVertex nsvcVertexSim = new NSVCVertex("nsvcVertexSim") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int simult1=mv1.getProperSimultaneousInEdges(nsss.subgraph1).size();
		int simult2=mv2.getProperSimultaneousInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(simult1-simult2));
		return result;
	    }
	};

    /* Count only weak edges */
    public static final NSVCVertex nsvcVertexWeakIn = new NSVCVertex("nsvcVertexWeakIn") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int in1= mv1.getProperWeakFollowInEdges(nsss.subgraph1).size();
		int in2= mv2.getProperWeakFollowInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(in1-in2));
		return result;
	    }
	};
    public static final NSVCVertex nsvcVertexWeakOut = new NSVCVertex("nsvcVertexWeakOut") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
	
		int out1= mv1.getProperWeakFollowOutEdges(nsss.subgraph1).size();
		int out2= mv2.getProperWeakFollowOutEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(out1-out2));
		return result;
	    }
	};
    public static final NSVCVertex nsvcVertexWeakSim = new NSVCVertex("nsvcVertexWeakSim") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int simult1=mv1.getProperWeakSimultaneousInEdges(nsss.subgraph1).size();
		int simult2=mv2.getProperWeakSimultaneousInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(simult1-simult2));
		return result;
	    }
	};

    /* Count only strong edges */
    public static final NSVCVertex nsvcVertexStrongIn = new NSVCVertex("nsvcVertexStrongIn") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int in1= mv1.getProperStrongFollowInEdges(nsss.subgraph1).size();
		int in2= mv2.getProperStrongFollowInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(in1-in2));
		return result;
	    }
	};
    
    public static final NSVCVertex nsvcVertexStrongOut = new NSVCVertex("nsvcVertexStrongOut") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
	
		int out1= mv1.getProperStrongFollowOutEdges(nsss.subgraph1).size();
		int out2= mv2.getProperStrongFollowOutEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(out1-out2));
		return result;
	    }
	};
    
    public static final NSVCVertex nsvcVertexStrongSim = new NSVCVertex("nsvcVertexStrongSim") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
		
		int simult1=mv1.getProperStrongSimultaneousInEdges(nsss.subgraph1).size();
		int simult2=mv2.getProperStrongSimultaneousInEdges(nsss.subgraph2).size();
		result += 1-1/(1+(double)Math.abs(simult1-simult2));
		return result;
	    }
	};
    
     public static final NSVCVertex nsvcVertexPitchClass = new NSVCVertex("nsvcVertexPitchClass") {
	    public ViewDifference nonSeqCompare(NonSeqSimStmt nsss) {
		NSVDVertex vd=
		    new NSVDVertex(0.0/*dummy*/,this);

		vd.similarities = getVertexSimMatrix(nsss);
		return vd;
	    }
	    protected double compareVertices(MusicVertex mv1, 
					     MusicVertex mv2,
					     NonSeqSimStmt nsss) {
		
		double result = 0.0;
	
		if ((mv1 instanceof SimpleVertex) && (mv2 instanceof SimpleVertex)) {
		    // both mv1 and mv2 are SimpleVertices
		    int pc1=((SimpleVertex)mv1).info.theNote.getPitch() %12;
		    int pc2=((SimpleVertex)mv2).info.theNote.getPitch() %12;
		    
		    int diff = Math.max(pc1,pc2)-Math.min(pc1,pc2);
		    if (diff>6) diff=12-diff;
		    
		    result += 1-1/(1+(double)Math.abs(diff));
		} else if ((mv1 instanceof CompoundVertex) && (mv2 instanceof CompoundVertex)) {
		    // both mv1 and mv2 are CompoundVertices
		    CompoundVertex cv1 =(CompoundVertex)mv1;
		    CompoundVertex cv2 =(CompoundVertex)mv2;
		    /*NonSeqSimStmt ns = new NonSeqSimStmt((NonSeqSubgraph)cv1.subgraph,
							 (NonSeqSubgraph)cv2.subgraph,
							 SimLib.nonSeqVertexC,null);
							 result = ns.value();*/
		    if (cv1.productionRule.nt==cv2.productionRule.nt) {
			result=0;
		    } else result=1;

		} else {
		    // mv1 and mv2 are not of same type.
		    result=1.0;
		}
		return result;
	    }
	};
    




    /* IMPORTANT!! When placed in this same file, the similarity
       measures should be placed AFTER the ViewComparator definitions,
       otherwise the latter will not have been initialised at the time
       when the SimilarityMeasures are initialised, resulting in null
       pointer exceptions. */
		
    /* ---------- SIMILARITY MEASURES ---------- */
    public static final NSSMVertex nonSeqVertexPLIOS = new NSSMVertex("nonSeqVertexPLIOS") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexPitch);
		viewComparators.add(SimLibVertex.nsvcVertexLength);

		viewComparators.add(SimLibVertex.nsvcVertexWeakIn);
		viewComparators.add(SimLibVertex.nsvcVertexWeakOut);
		viewComparators.add(SimLibVertex.nsvcVertexWeakSim);

		viewComparators.add(SimLibVertex.nsvcVertexStrongIn);
		viewComparators.add(SimLibVertex.nsvcVertexStrongOut);
		viewComparators.add(SimLibVertex.nsvcVertexStrongSim);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=weightedMeanVertexSimMatrices(vr);
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		double val = getFinalValue(nsss,vr);
		vr.measure=val*(2-nestedSizeRatio(nsss));
		/* END CORE. */
		
		nsss.smResult = vr;
	    }

	    public double[][] weightedMeanVertexSimMatrices(NSSMVertexResult vr) {
		NSVDVertex vd = (NSVDVertex) vr.viewDifferences.get(0);
		double[][] vertexSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=5;
		double lengthWeight=5;
		double totalWeight = pitchWeight+lengthWeight+6;
		
		for (int i=0; i<vr.viewDifferences.size(); i++) {
		    vd = (NSVDVertex) vr.viewDifferences.get(i);
		    if (vd.origin==nsvcVertexPitch) {
			vertexSimilarities = FuncLib.linearCombineMatrices(vertexSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==nsvcVertexLength) {
			vertexSimilarities = FuncLib.linearCombineMatrices(vertexSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			vertexSimilarities = FuncLib.addMatrices(vertexSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(vertexSimilarities,totalWeight);
	    }
	    
	    
	};
    
    public static final NSSMVertex nonSeqVertexPLIOS_nocompounds = new NSSMVertex("nonSeqVertexPLIOS") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexPitch);
		viewComparators.add(SimLibVertex.nsvcVertexLength);

		viewComparators.add(SimLibVertex.nsvcVertexStrongIn);
		viewComparators.add(SimLibVertex.nsvcVertexStrongOut);
		viewComparators.add(SimLibVertex.nsvcVertexStrongSim);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=weightedMeanVertexSimMatrices(vr);
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		double val = getFinalValue(nsss,vr);
		vr.measure=val;
		/* END CORE. */
		
		nsss.smResult = vr;
	    }

	    public double[][] weightedMeanVertexSimMatrices(NSSMVertexResult vr) {
		NSVDVertex vd = (NSVDVertex) vr.viewDifferences.get(0);
		double[][] vertexSimilarities = new double[vd.similarities.length][vd.similarities[0].length];
		double pitchWeight=3;
		double lengthWeight=3;
		double totalWeight = pitchWeight+lengthWeight+3;
		
		for (int i=0; i<vr.viewDifferences.size(); i++) {
		    vd = (NSVDVertex) vr.viewDifferences.get(i);
		    if (vd.origin==nsvcVertexPitch) {
			vertexSimilarities = FuncLib.linearCombineMatrices(vertexSimilarities,1,vd.similarities,pitchWeight);
		    } else if (vd.origin==nsvcVertexLength) {
			vertexSimilarities = FuncLib.linearCombineMatrices(vertexSimilarities,1,vd.similarities,lengthWeight);  
		    } else {
			vertexSimilarities = FuncLib.addMatrices(vertexSimilarities,vd.similarities);  
		    }
		}
		return FuncLib.divideMatrixBy(vertexSimilarities,totalWeight);
	    }
	    
	    
	};


    /* deprecated: */
    public static final NSSMVertex nonSeqVertexP = new NSSMVertex("nonSeqVertexP") {
	    public void initialize() {
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexPitch);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    
    public static final NSSMVertex nonSeqVertexL = new NSSMVertex("nonSeqVertexL") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexLength);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    
    public static final NSSMVertex nonSeqVertexI = new NSSMVertex("nonSeqVertexI") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexIn);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    
    public static final NSSMVertex nonSeqVertexO = new NSSMVertex("nonSeqVertexO") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexOut);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    
    public static final NSSMVertex nonSeqVertexS = new NSSMVertex("nonSeqVertexS") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexSim);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    
    public static final NSSMVertex nonSeqVertexC = new NSSMVertex("nonSeqVertexC") {
	    public void initialize() {
		// System.out.println("init called");
		viewComparators.clear();
		viewComparators.add(SimLibVertex.nsvcVertexPitchClass);
	    }
	    public void evaluateNonSeq(NonSeqSimStmt nsss) {
		/* BEGIN CORE. This is the heart of the similarity
		 * measure, replace with other code to create new
		 * NonSeqSimMeasure*/
		NSSMVertexResult vr = new NSSMVertexResult(this);
	   
		vr.viewDifferences=evaluateAllViewComparators(nsss);
		vr.meanSimilarities=vr.meanVertexSimMatrices();
		vr.bestMatch= getBestMatch(nsss,vr);
		vr.completeMatchData(nsss); // computes multipleMatchings
		                            // and vector of matched vertices
		vr.measure = getFinalValue(nsss,vr);
		/* END CORE. */
		
		nsss.smResult = vr;
	    }
	    
	};
    

}
