/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** A similarity measure is an arithmetic combination of view
    differences that is used to evaluate {@link
    SimilarityStatement}s. */
public abstract class SimilarityMeasure {

    /** A label to identify this measure by. */
    public String name;
    /** A Vector of {@link ViewComparator}s whose results (view
        differences) are combined in this measure. */
    public Vector viewComparators=new Vector(); // of ViewComparator

    /** vps is a Vector of {@link ViewComparator}. */
    public SimilarityMeasure(String n, Vector vps /* of ViewComparator*/) {
	name = n;
	viewComparators=vps;
	initialize();
    }

    /** Neutral constructor. */
    public SimilarityMeasure(String n) {
	name = n;
	initialize();
    }

    /** Returns the number of view comparators added to 'viewComparators'. */
    public final int getNumberOfViewComparators() {
	return viewComparators.size();
    }
    
    // define evaluate in the final class definition
    public abstract void evaluate(SimilarityStatement ss);
    public abstract void initialize();

    // not used yet
    ViewComparator getMostRestrictivePitchVC() {
	ViewComparator result=null;
	for (int i=0; i<viewComparators.size(); i++) {
	    ViewComparator vc = (ViewComparator) viewComparators.get(i);
	    if (SimLib.viewComparatorPitchSubsumption(result, vc) ){
		result=vc;
	    }
	}
	return result;
    }

}
