/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;
import java.awt.Point;

/** A SimpleVertex is how a note is represented in a {@link
    MusicGraph}. The Most important information on the note is saved
    in a {@link NoteInfo} object. */
public class SimpleVertex extends MusicVertex {

    /** Musical information on the note represented by this
        SimpleVertex. */
    public NoteInfo info;

    /* Constructor; the time stamp passed as parameter to this
       constructor may be created using the function
       FuncLib.getTimeStamp().  */
    public SimpleVertex(NoteInfo ni, long stamp) {
	inEdges = new Vector();
	outEdges = new Vector();
	info = ni;
	starttime=info.starttime;
	objectTimeStamp=stamp;
    }

    /** The length of a SimpleVertex is the length of its note. */
    public double getLength() {
	return info.theNote.getRhythmValue();
    }

    public String toString() {
	String result = "SV of Note=<"+info.theNote.getPitch()+
	    ","+info.theNote.getRhythmValue()+
	    "> start="+starttime;//+", "+graphPosToString()+" "+distToString();
	return result;
    }

    /** Anchor resolution on this SimpleVertex always returns this
        SimpleVertex.*/
    public SimpleVertex resolveAnchor(Vector anchor /*of Integer */) {
	if (anchor.size()>1) {
	    System.out.println("SimpleVertex.resolveAnchor() called with anchor vector of size>1");
	}
	return (SimpleVertex) this;
    }

    /** A deep clone of a SimpleVertex clones its {@link NoteInfo}
        object and gives the new clone another time stamp. */
    public MusicVertex deepClone() {
	SimpleVertex sv = new SimpleVertex((NoteInfo)info.clone(),
					   FuncLib.getTimeStamp());
	/* inEdges and outEdges cannot be set here, do it from outside */
	return sv;
    }
}

