/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */


/** Class StatusInfo groups a data set describing the status of the GA
    in class {@link SimFinder} at one particular time (after one
    particular generation. */
public class StatusInfo  {
    /* Constructor */
    public StatusInfo() {}
    public StatusInfo(SimilarityStatement best) {
	bestCopy=best;
    }
    
    // row numbers in the gnuplot file are given in the comments
    public int generation=0; // 1
    public int popSize=0;    // 2
    
    public double bestFitness=0.0;  //3
    public double worstFitness=0.0; //4
    public double meanFitness=0.0; //5
    public double stddevFitness=0.0;//6
    
    public int bestSize=0;          //7
    public int worstSize=0;         //8
    public double meanSize=0.0;     //9
    public int longestLength=0;     //10
    public int minOverlap=0;        //11
    public int maxOverlap=0;        //12
    public double meanOverlap=0.0;  //13
    public int vertexCoverage=0;    //14
    public double vertexCovPercent=0.0;//15
    public int edgeCoverage=0;         //16
    public double edgeCovPercent=0.0;  //17
    
    public SimilarityStatement bestCopy=null;
    
    public String gnuplotHeader() {
	String result="# generation popSize ";
	result+="bestFitness worstFitness meanFitness ";
	result+="stddevFitness bestSize worstSize meanSize longestLength ";
	result+="minOverlap maxOverlap meanOverlap "; 
	result+="vertexCoverage vertexCovPercent "; 
	result+="edgeCoverage edgeCovPercent ";
	result+= bestCopy.gnuplotHeader();
	return result;
    }
    
    public String toGnuplotString() {
	String result="";
	result+=generation+" ";
	result+=popSize+" ";
	
	result+=bestFitness+" "+worstFitness+" ";
	result+=meanFitness+" "+stddevFitness+" ";
	result+=bestSize+" "+worstSize+" "+meanSize+" "+longestLength+" ";
	result+=minOverlap+" "+maxOverlap+" "+meanOverlap+" ";
	result+=vertexCoverage+" "+vertexCovPercent+" ";
	result+=edgeCoverage+" "+edgeCovPercent+" ";
	
	result+=bestCopy.toGnuplotString();
	return result;
    }
    
    public String toScreenString() {
	String result="STATUSINFO generation "+generation+"\n";
	result+="\tpop size="+popSize+"\n";
	result+="\tBest fitness:  "+bestFitness+" size = "+bestSize+"\n";
	result+="\tWorst fitness: "+worstFitness+" size = "+worstSize+"\n";
	result+="\tMean fitness:  "+meanFitness+"\n";
	result+="\tStddev.fitness:"+stddevFitness+"\n";
	result+="\tMean size:     "+meanSize+"\n";
	result+="\tLongest:       "+longestLength+"\n";
	result+="\tMin overlap:   "+minOverlap+"\n";
	result+="\tMax overlap:   "+maxOverlap+"\n";
	result+="\tMean overlap:  "+meanOverlap+"\n";
	result+="\tVertexCoverage:"+vertexCoverage+" ("+vertexCovPercent+")\n";
	result+="\tEdgeCoverage:"+edgeCoverage+" ("+edgeCovPercent+")\n";
	result+=bestCopy.toString()+"\n";
	return result;
    }
}

