/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.util.Vector;

/** The Trie is a data structure used in the bag intersection ratio
    similarity measures (see e.g. the comparator {@link NSVCBagIR}). */
public class Trie {

    private char letter;
    private boolean word;
    private Trie next, more;
    private int count;
    public Vector repeats;
    
    public Trie(){
	letter = 0;
	word = false;
	count=0;
    }
    
    public boolean member(String s){
	boolean answer = false;
	if(letter==(char)s.charAt(0)){
	    if(s.length()>1){
		if(more!=null){
		    answer=more.member(s.substring(1));
		}else{
		    answer= false;
		}
	    }else{
		boolean occ=(word && count>0);
		count--;
		answer=occ;
	    }
	}else{
	    if(next!=null){answer=next.member(s);}
	}
	return answer;
    }
    
    public void print(){
	if(letter==0){
	    System.out.println("The trie is empty");
	}else{
	    System.out.println("all:");
	    print("");
	    System.out.println("");
	}
    }
    
    private void print(String s){
	if(next.letter!=0){next.print(s);}
	s+=letter;
	if(word){System.out.print(s+"\n");}
	if(more.letter!=0){more.print(s);}
    }

    private void printOcc(String s, int occ){
	if(next.letter!=0){next.printOcc(s,occ);}
	s+=letter;
	if(word&&count==occ){System.out.print(s+"\n");}
	if(more.letter!=0){more.printOcc(s,occ);}
    }
    /* skriver en vector (TrieInput.reps), der indeholder hvilke patterns 
       der er gentaget howmany gange */
    /*   public void getRepeats(int howMany){
	getOcc("",howMany);
	}*/

    /*    private void getOcc(String s, int occ){
	if(next.letter!=0){next.getOcc(s,occ);}
	s+=letter;
	if(word && count==occ){
	    //refererer til en global Vector i TrieInput!
	    TrieInput.reps.add(s);
	}
	if(more.letter!=0){more.getOcc(s,occ);}
	}*/

     public void insert(String s){
	if(letter==0){
	    more = new Trie();
	    next = new Trie();
	    letter = (char)s.charAt(0);
       	    if(s.length()>1){
		more.insert(s.substring(1));
	    }else{
		word = true;
		count++;
	    }
	}else if(letter==s.charAt(0)){
	    if(s.length()>1){
		more.insert(s.substring(1));
	    }else{
		word=true;
		count++;
	    }
	}else{
	    next.insert(s);
	}
    }

    public int maxOccOf_The_Length(){
	if(word){
	    //	    System.out.println("word!, count= "+count+", word= "+word+", len= "+len);
	    return count;
	}
	//	System.out.println("count= "+count+", word= "+word+", len= "+len);
	int moreBest=0;
	int nextBest=0;
	if(next.letter!=0){
	    nextBest=next.maxOccOf_The_Length();
	}
	if(more.letter!=0){
	    moreBest=more.maxOccOf_The_Length();
	}
	return Math.max(count,Math.max(nextBest,moreBest));
    }

    /*
    public int maxOccOfLength(int len, int max){
	if(next.letter!=0 && len>0){
	    return Math.max(max,next.maxOccOfLength(len,max));
	}
	if(more.letter!=0 && len>0){
	    return more.maxOccOfLength(len-1,max);
	}
	//	System.out.println();
	return Math.max(count,max);
    }
    */

}
