/* Copyright (C) 2003 Sren Tjagvad Madsen & Martin Elmer Jrgensen
 * email: {tjagvad,elmer}@daimi.au.dk */

import java.awt.*;
import java.util.Vector;
import java.awt.event.*;

/** VertexUsageGui is a GUI-related class used to show the relative
    usage frequency of all {@link MusicVertex}'s of a {@link
    MotherGraph} for a given generation of the {@link SimFinder}. An
    example of the vertex usage graphics can be seen in figure 32,
    p.88 of our thesis.*/
public class VertexUsageGui extends Frame implements AdjustmentListener, MouseListener {

    Scrollbar sbHor = new Scrollbar(Scrollbar.HORIZONTAL);
    UsageCanvas cv;
    MotherGraph mg;
    int groups=0;
    int largestGroupSize=0;
    int tilesize=0;
    SimFinder simfinder=null;

    public VertexUsageGui(MotherGraph mgraph, 
			  SimFinder sf,
			  int locx, int locy, int tsize) {
	super("Vertex usage in "+mgraph.title);

	mg=mgraph;
        sbHor.addAdjustmentListener(this);
	setLocation(locx,locy);
	tilesize=tsize;
	simfinder=sf;

	groups=numberOfGroups(mgraph);
	largestGroupSize=sizeOfLargestGroup(mgraph);

	cv = new UsageCanvas(sbHor, mg);
	cv.setSize(600,largestGroupSize*tilesize*2);
	cv.addMouseListener(this);
	addMouseListener(this);
	add(cv,BorderLayout.CENTER);

	add(sbHor, BorderLayout.SOUTH);
	pack();
	show();
	// cv.paint(getGraphics());
	
	// try { Thread.sleep(3000);} catch(Exception e) {System.out.println("sleep exception: "+e.getMessage());}
    }

    public int sizeOfLargestGroup(MotherGraph mgraph) {
	int result=0;
	mgraph.initGroupTokenizer(0.0);
	Vector group=mgraph.getNextGroup();
	while(group.size()>0) {
	    if (group.size()>result) result=group.size();
	    group = mgraph.getNextGroup();
	}
	return result;
    }

    public int numberOfGroups(MotherGraph mgraph) {
	int result=0;
	mgraph.initGroupTokenizer(0.0);
	Vector group=mgraph.getNextGroup();
	while(group.size()>0) {
	    result++;
	    group = mgraph.getNextGroup();
	}
	return result;
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
      Scrollbar sb = (Scrollbar)evt.getSource();
      cv.sbChanged();
    }

    public void updateColours() {
	cv.repaint();
    }

    public void mousePressed(MouseEvent e) {
	// huh
    }
    public void mouseReleased(MouseEvent e) {
	// huh
    }
    public void mouseEntered(MouseEvent e) {
	// huh
    }
    public void mouseExited(MouseEvent e) {
	// huh
    }

    public void mouseClicked(MouseEvent e) {
	/* MusicGraph mg = (MusicGraph) highlights.get(0);
	if (e.getButton()==MouseEvent.BUTTON1) {
	    try {mg.shortenRight(1);}
	    catch(Exception ex) { System.out.println(ex.getMessage());}
	}
	else if (e.getButton()==MouseEvent.BUTTON2) {
	    System.out.println("Hah, mouse button 2 has no effect!");
	}
	else if (e.getButton()==MouseEvent.BUTTON3) {
	    try {mg.extendRight(1);}
	    catch(Exception ex) { System.out.println(ex.getMessage());}
	}
	showSubgraph(mg);*/ 
    }

    private class UsageCanvas extends Canvas {
	int originX=0, originY=0;
	Scrollbar sbHor;
	int maxWidth=50000; // should be 0<maxWidth<ca.65000
	int graphWidth=groups*tilesize;
	MotherGraph mothergraph=null;
	
	/* Constructor */
	public UsageCanvas(Scrollbar sb, MotherGraph mgraph) {
	    this.sbHor=sb;
	    mothergraph=mgraph;
	}
	
	public void paint(Graphics g) {
	    g.setFont(new Font("Dialog",Font.PLAIN,10));
	    g.translate(originX, originY);
	    doLayout();
	    drawColours(mothergraph,g);
	}

	// If the size of the component changes, we need to update
	// the scrollbar values.
	public void doLayout() {
	    super.doLayout();
	    int w = getSize().width;
	    int h = getSize().height;
	    
	    /*sbVer.setValues(sbVer.getValue(), h, sbVer.getMinimum(), gridSize);
	    sbVer.setBlockIncrement(Math.min(cellSize, h));
	    sbVer.setUnitIncrement(Math.min(cellSize, h/5));*/
	    sbHor.setValues(sbHor.getValue(), w, sbHor.getMinimum(), graphWidth);
	    // System.out.println("gw="+graphWidth);
	    sbHor.setBlockIncrement(4*w/5);
	    sbHor.setUnitIncrement(w/20);
	    
	    // This ensures that if the grid is larger than the viewable
	    // area, you will never see space around the grid.
	    // originX = -Math.max(0, Math.min(gridSize-w, sbHor.getValue()));
	    // originY = -Math.max(0, Math.min(gridSize-h, sbVer.getValue()));
	}
	public void sbChanged() {
	    // originY = -sbVer.getValue();
	    originX = -sbHor.getValue();
	    //System.out.println("sbChanged: "+sbHor.getValue());
	    repaint()
;
	}

	/* The general layout */
	public void drawColours(MotherGraph mgraph, Graphics g) {
	    int groupCounter=0;
	    int vertexInGroup=0;

	    Vector forbiddenVertices=getForbiddenVertices();

	    int maxUse=mgraph.maxUsage();
	    mgraph.initGroupTokenizer(0.0);
	    Vector group = mgraph.getNextGroup();
	    while (group.size()>0) {
		// draw this group
		group = sortByPitch(group,true);
		vertexInGroup=0;
		for (int i=0; i<group.size(); i++) {
		    MusicVertex mv = (MusicVertex) group.get(i);
		    float redComponent=0.0f;
		    float blueComponent=0.6f;
		    if (forbiddenVertices.contains(mv)) {
			redComponent+=0.5f;
		    }
		    g.setColor(new Color(redComponent,
					 (float) mv.useCounter/maxUse,
					 blueComponent));
		    g.fillRect(groupCounter*tilesize,
			       vertexInGroup*tilesize,
			       tilesize, 
			       tilesize);
		    g.setColor(new Color(0f,0f,0f));
		    g.drawRect(groupCounter*tilesize,
			       vertexInGroup*tilesize,
			       tilesize, 
			       tilesize);
		    vertexInGroup++;
		}

		// get the next group
		group = mgraph.getNextGroup();
		groupCounter++;
	    }

	}

	public Vector getForbiddenVertices() {
	    Vector result = new Vector(); // of MusicVertex
	    for (int i=0; i<simfinder.forbiddenSubgraphs.size(); i++) {
		Subgraph sg= (Subgraph) simfinder.forbiddenSubgraphs.get(i);
		result.addAll(sg.vertices);
	    }
	    return result;
	}
    }

    /* Simple vertices are sorted by pitch, compound vertices are
       sorted with the others by objectTimeStamp */
    Vector sortByPitch(Vector group, boolean inverse) {
	Vector result = new Vector();
	MusicVertex[] gr = new MusicVertex[group.size()];
	for (int i=0; i<group.size(); i++) {
	    gr[i] = (MusicVertex) group.get(i);
	}

	// bubblesort
	for (int i=0; i<gr.length; i++) {
	    for (int j=0; j<gr.length-1; j++) {
		boolean swap=false;
		if(gr[j] instanceof SimpleVertex && 
		   gr[j+1] instanceof SimpleVertex) {
		    // both vertices are simple, compare their pitches
		    SimpleVertex sv1=(SimpleVertex) gr[j];
		    SimpleVertex sv2=(SimpleVertex) gr[j+1];
		    
		    if (!inverse && (sv1.info.theNote.getPitch() > sv2.info.theNote.getPitch()) ||
		    inverse && (sv1.info.theNote.getPitch() < sv2.info.theNote.getPitch())
		    ) { swap=true;}
		} else {
		    // at least one vertex is not simple, compare
		    // their timestamps
		    if (!inverse && (gr[j].objectTimeStamp > gr[j+1].objectTimeStamp) ||
		    inverse && (gr[j].objectTimeStamp < gr[j+1].objectTimeStamp)
			) { swap=true;}
		}
		if (swap) {
		    // swap
		    MusicVertex tmp = gr[j];
		    gr[j]=gr[j+1];
		    gr[j+1]=tmp;
		}
	    }
	}
	for (int i=0; i<gr.length; i++) {
	    result.add(gr[i]);
	}
	return result;
    }
}
